//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of CPCtelera: An Amstrad CPC Game Engine
//  Copyright (C) 2018 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------



//los hombres y mujeres son de ese tipo independientemente de que sean el prota o no

//sin tipo, la entidad esta libre para usarse
#define TYPE_EMPTY 0

//el prota directamente sin más  (bit 0 a 1 es el prota)
#define TYPE_PROTA 1

//define si la persona va a ser hombre (bit 1 a 1) o mujer (bit 1 a 0) o dicho de otra forma es el bit man
#define TYPE_MAN 2

//un ciudadano de la ciudad que esta paseando o haciendo sus mierdad (el sexo lo define el TYPE_MAN a 0 es mujer, TYPE_MAN a 1 es hombre)
//tambien vale para saber que es un NPC ya sea un pringao de la ciudad o uno de los personajes principales
#define TYPE_CITIZEN 4

//define un objeto arma o un item o una base
#define TYPE_AUX_OBJECT 8

//este npc es un policia (tiene caracteristicas especiales)
#define TYPE_POLICE 16

//una entidad de tipo coche (el tipo del coche ira en OBJ_SELECTED_CAR)  (le afectan las colisiones y puede moverse por cualquier terreno solido)
#define TYPE_CAR 32

//una entidad de tipo lancha           (le afectan las colisiones pero solo se puede mover por terreno liquido, osea solo puede ir por los tiles de agua)
#define TYPE_LANCHA 64

//una entidad para controlar las luces de los coches y otras luces
#define TYPE_LIGHT 128



#include <cpctelera.h>

//incluimos el engine
#include "armony.h"

//incluimos datos de graficos
//#include "tileset.h"

//incluimos datos de graficos
//#include "metatiles.h"

//incluimos una fuente pequeñita
#include "minifont.h"

//incluimos los sprites semicompilados
#include "spritesCompilados.h"

//incluimos los mapas de metatiles o tiles
//#include "maps.h"

#include "mapObjects.h"

#include "talking.h"

#include "mision.h"

#include "musicRob.h"


#define POSICION_SCROLL_X_PLAYER 19

#define POSICION_SCROLL_Y_PLAYER 11

//indica desde el centro de la pantalla, cuanto de lejos tienen que estar los objetos para que sean eliminados (en tiles)
//si el objeto está más a la izq que CENTROX-LIMITE_HORIZONTAL_ELIMINAR_OBJETO se elimina
//si el objeto está más a la der que CENTROX+LIMITE_HORIZONTAL_ELIMINAR_OBJETO se elimina
//a priori dejo 5 tiles de margen por cada lado aunque se pueden poner menos si hiciera falta
//40 de ancho, la mitad 20 + 10 de margen 30
#define LIMITE_HORIZONTAL_ELIMINAR_OBJETO 25
//25 de alto, la mitad 13 + 10 de margen 23
#define LIMITE_VERTICAL_ELIMINAR_OBJETO 18

//en la zona BF00 a BF50 vamos a guardar un array que contiene un listado de puntos 3 bytes, 1 byte para el numero de frames que se mantiene y 2 bytes para la posicion en la pantalla
#define PREDRAW_PIXELS_ZONE 0xBF00

//numero de luces maxima que voy a renderizar (esto es más que nada para que no explote el engine, las luces consumen muchos recursos al tener que limpiar muchos tiles)
#define MAX_LUCES 3

#define CICLOS_FUEGO_PULSADO_PARA_ROBAR_COCHE 6

#define MAX_OBJECTS_LIMIT_FOR_CREATE_NEW 10
#define MAX_CARS_IN_SCREEN_LIMIT 3
#define MAX_PERSON_IN_SCREEN_LIMIT 3

//velocidad minima para que el coche se mueva algo, si el coche va a menos de eso es como si tuviese el freno de mano echado y su velocidad debe ser 0 en ambos sentidos
#define MIN_SPEED_TO_MOVE_CAR 10

//velocidad minima para que se pueda cambiar el angulo del coche, si el coche va muy muy lento no puede girar
#define MIN_SPEED_TO_TORQUE_CAR 10

//velocidad minima y diferencia de angulo minimo a partir de ahi el coche derrapa
#define MIN_SPEED_TO_DRIFT 50
#define MIN_ANGLE_DIFF_TO_DRIFT 30

//si la IA ve otro coche a esta distancia por delante frena
#define DISTANCIA_COCHE_FRENA 5

//a partir de esta velocidad se aplica el turboscroll (scroll al tile siempre y ajustado a metatile)
#define SPEED_TO_TURBO_SCROLL 129

//a partir de este metatile se tiene que mirar los tiles para comprobar la colision de los coches (los de antes no hace falta porque es plano todo)
#define MAX_METATILE_NO_COLISION_CAR 26

//la vida con la que comienza cada objeto
#define PERSON_START_LIFE 40
#define CAR_START_LIFE 120
#define PROTA_START_LIFE 100

//vida que se le resta a la persona que es atropellada
#define RESTA_VIDA_EN_ATROPELLO 10

//velocidad minima para que un coche pueda atropellar a alguien, a menos de eso no se atropella, si acaso se empuja
#define VEL_MIN_TO_ATROPELLO 30

//numero de frames que el jugador esta tirado por los suelos tras ser atropellado (solo prota, la IA va por otro lado)
#define NUM_FRAMES_KO_TRAS_ATROPELLO 20

//distancia maxima en la que una explosion te hace daño (en tiles)
#define DISTANCIA_TILES_EXPLOSION_DAMAGE 5
//este es el daño que hace si te pilla una explosion cerca
#define EXPLOSION_DAMAGE 20
//numero de step/frames en los que se verá el destello de la explosion
#define FRAMES_EXPLOSION_LIGHT 5

//si el random sale menor que esto la IA huira acojonada, si sale mayor atacará al prota
#define VALOR_PERSONAS_IA 128

//velocidad a la que van los NPC cuando quieren atacar al prota
#define VELOCIDAD_NPC_ATACANDO 24

//tiempo entre ataque y ataque de los NPCS para que no ataquen a lo loco
#define RELAX_ATTACK_TIME_NPC 40

//velocidad a la que van los NPC paseando
#define VELOCIDAD_IA_PASEANDO 24      
#define VELOCIDAD_IA_PASEANDO_NEG 0xE8

//numero de veces que se va a calcular un lugar random para sacar un coche (el lugar tiene que ser una carretera sino no puede salir el coche)
#define NUMERO_INTENTOS_SACAR_COCHE 5

#define FX_FIRST 1
#define FX_SNARE 2
#define FX_DIN 3
#define FX_DINLARGO 4
#define FX_FREDGRAISH 5
#define FX_SUSTI3 7
#define FX_SUSTIB 6     //este suena a golpe o puño no se
#define FX_PUNO 8
#define FX_DISPARO 9
#define FX_GOLPE 10
#define FX_CORTE 11
#define FX_EMPTY 12







//PALETA MAÑANAS
#define PAL_MANANA_0 0x54    //negro
#define PAL_MANANA_1 0x5C    //rojo
#define PAL_MANANA_2 0x56    //verde
#define PAL_MANANA_3 0x5E    //amarillo
#define PAL_MANANA_4 0x44    //azul
#define PAL_MANANA_5 0x58    //magenta
#define PAL_MANANA_6 0x46    //cyan
#define PAL_MANANA_7 0x4E    //naranja

#define PAL_MANANA_8 0x40    //gris
#define PAL_MANANA_9 0x4C    //rojo brillo
#define PAL_MANANA_10 0x52   //verde brillo
#define PAL_MANANA_11 0x4A   //amarillo brillo
#define PAL_MANANA_12 0x55   //azul brillo
#define PAL_MANANA_13 0x4D   //magenta brillo
#define PAL_MANANA_14 0x53   //cyan brillo
#define PAL_MANANA_15 0x4B   //blanco



//PALETA TARDE
#define PAL_TARDE_0 0x54    //negro
#define PAL_TARDE_1 0x5C    //rojo
#define PAL_TARDE_2 0x56    //verde
#define PAL_TARDE_3 0x5E    //amarillo
#define PAL_TARDE_4 0x44    //azul
#define PAL_TARDE_5 0x58    //magenta
#define PAL_TARDE_6 0x46    //cyan
#define PAL_TARDE_7 0x4E    //naranja

#define PAL_TARDE_8 0x40    //gris
#define PAL_TARDE_9 0x47    //rojo pastel
#define PAL_TARDE_10 0x59   //verde pastel
#define PAL_TARDE_11 0x43   //amarillo pastel
#define PAL_TARDE_12 0x5F   //azul pastel
#define PAL_TARDE_13 0x4F   //magenta pastel
#define PAL_TARDE_14 0x5B   //cyan pastel
#define PAL_TARDE_15 0x4B   //blanco




//PALETA NOCHE
#define PAL_NOCHE_0 0x54    //negro
#define PAL_NOCHE_1 0x54    //rojo
#define PAL_NOCHE_2 0x54    //verde
#define PAL_NOCHE_3 0x54    //rojo
#define PAL_NOCHE_4 0x54    //azul
#define PAL_NOCHE_5 0x54    //magenta
#define PAL_NOCHE_6 0x54    //azul
#define PAL_NOCHE_7 0x5C    //rojo

#define PAL_NOCHE_8 0x54    //negro
#define PAL_NOCHE_9 0x5C    //rojo
#define PAL_NOCHE_10 0x56   //verde 
#define PAL_NOCHE_11 0x5E   //amarillo
#define PAL_NOCHE_12 0x44   //azul
#define PAL_NOCHE_13 0x58   //magenta 
#define PAL_NOCHE_14 0x46   //cyan 
#define PAL_NOCHE_15 0x40   //gris





#define PAL_EXPLO_0 0x40    //gris
#define PAL_EXPLO_1 0x4C    //rojo brillo
#define PAL_EXPLO_2 0x52   //verde brillo
#define PAL_EXPLO_3 0x4A   //amarillo brillo
#define PAL_EXPLO_4 0x55   //azul brillo
#define PAL_EXPLO_5 0x4D   //magenta brillo
#define PAL_EXPLO_6 0x53   //cyan brillo
#define PAL_EXPLO_7 0x4B   //blanco

#define PAL_EXPLO_8 0x40    //gris
#define PAL_EXPLO_9 0x4C    //rojo brillo
#define PAL_EXPLO_10 0x52   //verde brillo
#define PAL_EXPLO_11 0x4A   //amarillo brillo
#define PAL_EXPLO_12 0x55   //azul brillo
#define PAL_EXPLO_13 0x4D   //magenta brillo
#define PAL_EXPLO_14 0x53   //cyan brillo
#define PAL_EXPLO_15 0x4B   //blanco




//movimiento del coche tipo SIMPLE 32     (esto ira luego en una tabla mejor)
//lista de sprites de los coches (la tabla de animaciones)
//const u8* carPropSpriteAnim[]={(u8*)&animCarSimple[0],(u8*)&animCarSimple[0]};

//#define CAR_SIMPLE_TORQUE 16
const u8 carPropTorque[]={ 6,6,8,4,
                           4,5,7,6
                           };
//#define CAR_SIMPLE_ACCEL 3
const u8 carPropAccel[]={  2,3,7,1,
                           1,3,6,2};
//#define CAR_SIMPLE_BRAKE 6
const u8 carPropBrake[]={  3,3,6,2,
                           3,4,5,3};
//#define CAR_SIMPLE_AGARRE 28
const u8 carPropAgarre[]={ 27,27,30,29,
                           28,28,30,27};
//#define CAR_SIMPLE_MAX_SPEED 128 
const u8 carPropMaxSpeed[]={100,110,150,96,
                           74,124,140,110};

u8* const carPropAnim[]={(u8*)animCarSUV,(u8*)animCarSedan,(u8*)animCarPoli,(u8*)animCarFurgo,
                        (u8*)animCarFurgo,(u8*)animCarSUV,(u8*)animCarViper,(u8*)animCarSedan};


u8 gtaPaletaSeleccionada;
const u8 paletaManana[]={PAL_MANANA_0,PAL_MANANA_1,PAL_MANANA_2,PAL_MANANA_3,PAL_MANANA_4,PAL_MANANA_5,PAL_MANANA_6,PAL_MANANA_7,PAL_MANANA_8,PAL_MANANA_9,PAL_MANANA_10,PAL_MANANA_11,PAL_MANANA_12,PAL_MANANA_13,PAL_MANANA_14,PAL_MANANA_15};
const u8 paletaTarde[]={PAL_TARDE_0,PAL_TARDE_1,PAL_TARDE_2,PAL_TARDE_3,PAL_TARDE_4,PAL_TARDE_5,PAL_TARDE_6,PAL_TARDE_7,PAL_TARDE_8,PAL_TARDE_9,PAL_TARDE_10,PAL_TARDE_11,PAL_TARDE_12,PAL_TARDE_13,PAL_TARDE_14,PAL_TARDE_15};
const u8 paletaNoche[]={PAL_NOCHE_0,PAL_NOCHE_1,PAL_NOCHE_2,PAL_NOCHE_3,PAL_NOCHE_4,PAL_NOCHE_5,PAL_NOCHE_6,PAL_NOCHE_7,PAL_NOCHE_8,PAL_NOCHE_9,PAL_NOCHE_10,PAL_NOCHE_11,PAL_NOCHE_12,PAL_NOCHE_13,PAL_NOCHE_14,PAL_NOCHE_15};
const u8 paletaExplosion[]={PAL_EXPLO_0,PAL_EXPLO_1,PAL_EXPLO_2,PAL_EXPLO_3,PAL_EXPLO_4,PAL_EXPLO_5,PAL_EXPLO_6,PAL_EXPLO_7,PAL_EXPLO_8,PAL_EXPLO_9,PAL_EXPLO_10,PAL_EXPLO_11,PAL_EXPLO_12,PAL_EXPLO_13,PAL_EXPLO_14,PAL_EXPLO_15};

const u8 statusStr[]="XX POL:X ARC:X LUC:X BUR:X ALI:X SEA:X";


//este pekeño array es para guardar donde se pintaron luces en el ultimo frame (como las luces siempre van a ocupar 3x3 tiles, solo tengo que guardar la pos X e Y)
//y de esta forma puedo saber que tiles tengo que redibujar en el predraw (hasta que me encuentre con un 0)
//const u8 restauraLuces[]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
//BC00+64 del talkfrase  (coje 16 bytes)
#define RESTAURA_LUCES 0xBC40


u8* lostTimeIndex;    //esta variable tiene un puntero a un objeto y lo va a usar el metodo de limpiar objetos que se van saliendo fuera

i16 oldPosX;
i16 oldPosY;

u8 auxPosX;
u8 auxPosY;

u8 reintentos;

u8 auxTemp;          //simplemente una variable para guardar un valor que se vaya a usar pronto y luego se ignore (simplemente por no gastar registros)


u8* talkInstancePtr;       //puntero al personaje que esta hablando (o 0 si no hay nadie hablando)
//u8 talkFrase[64];          //2 cadenas de 32 caracteres con las frases que esta hablando (se deben ir montando en esta cadena)
#define TALK_FRASE 0xBC00

u8* talkImagePtr;          //puntero al bytecode de la cara que debe mostrarse que esta hablando (grafico 12x16)
u8 talkLastSize;           //esta guarda el tamaño de la ultima cadena que tengo que borrar

u8 explosionTime;          //0 no hace nada, numero mayor de 0 el numero de frames que se va a aclarar la paleta justo despues de una explosion

//este array es simplemente para contener segun que teclas se pulse, el angulo/velocidad/interpolacion a la que debe moverse
//primero interpolacion,despues velocidad y por ultimo el angulo
//000FUDLR
const u8 protaListaKeyMove[]={
   12,0,0,     //0000 nada pulsado
   12,64,0,    //0001 derecha
   12,64,128,  //0010 izquierda
   12,0,0,     //0011 izquierda y drecha

   12,64,64,   //0100 abajo
   12,90,32,   //0101 abajo derecha
   12,90,96,   //0110 abajo izquierda
   12,64,64,   //0111 abajo izquierda y derecha, osea solo abajo

   12,64,192,  //1000 arriba
   12,90,224,  //1001 arriba derecha
   12,90,160,  //1010 arriba izquierda
   12,64,192,  //1011 arriba izquierda y derecha, osea solo arriba

   12,0,0,     //1100 arriba y abajo osea nada
   12,64,0,    //1101 arriba y abajo y derecha osea derecha
   12,64,128,  //1110 arriba y abajo y izquierda osea izquierda
   12,64,0     //1111 arriba y abajo y izquierda y derecha, osea nada
};


//este array define de que forma se puede mover el trafico en cada metatile (por eso el array es de 64), cada valor significa como se mueve en el metatile de su posicion
//0- todos hacia derecha
//1- todos hacia abajo
//2- todos hacia izq
//3- todos hacia arriba
//4- seguir en la misma direccion
//5- horizontal mitad y mitad
//6- columnas del puente
//7- vertical mitad y mitad
//8- cruce pekeño (en teoria aqui podría ir por cualquier sitio siempre y cuando haya un tile de carretera)
//9- camino tierra
//10- arena horizontal
//11- arena vertical
//12- puerto (para colocar los barcos)
//13- parking
//14- agua (solo se pueden mover barcos)
//15- parque
//16- edificio
const u8 metatileTraficDirection[]={
   2, 0, 1, 3, 1, 2, 3, 0, 
   3, 0, 1, 2, 5, 7, 0, 3,
   1, 2, 2, 3, 0, 1, 4, 13,
   8, 9, 4,12,15,15,15, 4,
   16,16,14,16,16,16,16,16,
   16,16,16,16,16,16,16,16,
   16,16,16,16,16,16,16,11,
   10,14, 15,15,16,15,15,15
};


//esta tabla es relacionada con la anterior, tiene 4 valores por cada valor del anterior
//lo que indica es a que puntos (X e Y) debe ir el coche de la IA cuando este en ese metatileTraficDirection
//es una posicion relativa al metatile, pongo ejemplos:
//si es 0 que es la dirección que tiene dos carriles a la derecha meto dos puntos en el lado derecho para que el coche se dirija hacia allí:
//7,1, 7,5		esto indica que dentro del metatile se tiene que ir a esa posicion, de esta forma la IA calculara cual de los 
//dos puntos es más cercano e intentará ir hacia ese punto, lo hago asi porque de esta forma aunque empujemos un coche puede 
//volver a orientarse y seguir su curso por la carretera, en resumen, decirle a los coches hacia donde tienen que ir
const i8 traficGoToPos[]={
	11,1, 11,3,		   //0- todos hacia derecha
	2,11, 5,11,		   //1- todos hacia abajo
	-4,2, -4,5,		   //2- todos hacia izq
	1,-4, 5,-4,		   //3- todos hacia arriba
	-1,-1, -1,-1,		//4- seguir en la misma direccion	(aqui pongo 0 y ya interpretará que no debe cambiar
	-4,2, 11,5, 		//5- horizontal mitad y mitad
	-1,-1, -1,-1,		//6- columnas del puente
	2,11, 5,-4,		   //7- vertical mitad y mitad
	-1,-1, -1,-1,		//8- cruce pekeño (en teoria aqui podría ir por cualquier sitio siempre y cuando haya un tile de carretera)
	-1,-1, -1,-1,		//9- camino tierra
	-1,-1, -1,-1,		//10- arena horizontal
	-1,-1, -1,-1,		//11- arena vertical
	-1,-1, -1,-1,		//12- puerto (para colocar los barcos)
	-1,-1, -1,-1,		//13- parking
   -1,-1, -1,-1,     //14- agua (solo se pueden mover barcos)
   -1,-1, -1,-1,     //15- parque
   -1,-1, -1,-1      //16- edificio
};


//esta tabla guarda hacia donde se tiene que dirigir si estamos en un metatile que no tiene definida la direccion y tenemos que dirigirnos a alguno cercano
//dos valores si vamos a la derecha, dos para abajo, dos para la izq, y dos para arriba.
const i8 gtaReinsercionAddTable[]={
   
   4,7,     //si vamos a la derecha giramos hacia abajo
   0,4,     //si vamos hacia abajo, giramos hacia la izq
   4,0,     //si vamos hacia la izq, giramos hacia arriba
   7,4      //si vamos hacia arriba giramos hacia la derecha

  
/*   4,11,     //si vamos a la derecha giramos hacia abajo
   -4,4,     //si vamos hacia abajo, giramos hacia la izq
   4,-4,     //si vamos hacia la izq, giramos hacia arriba
   11,4      //si vamos hacia arriba giramos hacia la derecha*/
};


const u8 carColorsPrepare[]={
   CUSTOM_A_PAL_15|CUSTOM_B_PAL_8, //blanco-gris
   CUSTOM_A_PAL_9|CUSTOM_B_PAL_1, //rojo-
   CUSTOM_A_PAL_10|CUSTOM_B_PAL_2, //verde-
   CUSTOM_A_PAL_11|CUSTOM_B_PAL_3, //amarillo-
   CUSTOM_A_PAL_12|CUSTOM_B_PAL_4, //azul-
   CUSTOM_A_PAL_8|CUSTOM_B_PAL_0, //gris-negro
   CUSTOM_A_PAL_7|CUSTOM_B_PAL_1, //naranja-rojo
   CUSTOM_A_PAL_0|CUSTOM_B_PAL_0 //negro full
};

//estas dos variables guardan el desplazamiento que se tiene que aplicar desde la posicion del coche para dibujar el pixel del derrape en la carretera
//el desplazamiento va en pares x e y de 16 bits, osea, grupos de 32 bits con el desplazamiento X y el desplazamiento Y
//por lo tanto este array tiene 8x2 16 elementos que pillan 32 bytes
//para hacerlo en pixeles multiplica por 32 el valor que sea
const i16 desplazamientoXYDrift[]={
   96,320,     //derecha
   96,544,
   64,448,     //derecha abajo
   256,320,
   128,192,     //abajo
   512,192,
   704,448,     //abajo izq
   512,320,
   672,320,     //izq
   672,544,
   448,576,     //izq arriba
   704,448,
   128,608,     //arriba
   512,608,
   320,576,     //derecha arriba
   64,448
   };


//contiene array para pintar el menu principal a base de cuadrados, 
//cada cuadrado lleva 
//un u8 para los bytes verticales a pintar  (0 para acabar de pintar)
//un u8 para los bytes horizontales a pintar
//un u8 para el color a usar
//u16 para la direccion de video  (va al reves para que sea más facil leer, byte alto, byte bajo)

const u8 menuPrincipalCuadrados[]={
   40,40,CUSTOM_A_PAL_0|CUSTOM_B_PAL_0,0xC0, 0x00,    //C000 caracter 1-5           --------------CIELO----------------
   40,40,CUSTOM_A_PAL_0|CUSTOM_B_PAL_0,0xC0, 0x28,    //C028
   
   16,40,CUSTOM_A_PAL_0|CUSTOM_B_PAL_4,0xC1, 0x90,    //C190  caracter 6-7
   16,40,CUSTOM_A_PAL_0|CUSTOM_B_PAL_4,0xC1, 0xB8,    //C1B8

   40,40,CUSTOM_A_PAL_4|CUSTOM_B_PAL_4,0xC2, 0x30,    //C230 caracter 8-12
   40,40,CUSTOM_A_PAL_4|CUSTOM_B_PAL_4,0xC2, 0x58,    //C258

   16,40,CUSTOM_A_PAL_4|CUSTOM_B_PAL_12,0xC3, 0xC0,    //C3C0  caracter 13-14
   16,40,CUSTOM_A_PAL_4|CUSTOM_B_PAL_12,0xC3, 0xE8,    //C3E8

   40,40,CUSTOM_A_PAL_12|CUSTOM_B_PAL_12,0xC4, 0x60,    //C460 caracter 15-19
   40,40,CUSTOM_A_PAL_12|CUSTOM_B_PAL_12,0xC4, 0x88,    //C488

   48,40,CUSTOM_A_PAL_12|CUSTOM_B_PAL_12,0xC4, 0x60,    //C460 caracter 15-20
   48,40,CUSTOM_A_PAL_12|CUSTOM_B_PAL_12,0xC4, 0x88,    //C488





   36,1,CUSTOM_A_PAL_0|CUSTOM_B_PAL_0,0xE4, 0xEF,    //D4B0 + 63 caracter 16-19    120-144      ---------------PUENTE ------------------
   36,1,CUSTOM_A_PAL_0|CUSTOM_B_PAL_0,0xE4, 0xE0,    //D4B0 + 48 caracter 16-19    120-144
   36,1,CUSTOM_A_PAL_0|CUSTOM_B_PAL_0,0xE4, 0xD1,    //D4B0 + 32 caracter 16-19    120-144
   
   
   8,20,CUSTOM_A_PAL_0|CUSTOM_B_PAL_0,0xC5, 0xF0,    //C5F0 caracter 20       //el puente negro
   5,60,CUSTOM_A_PAL_1|CUSTOM_B_PAL_1,0xC6, 0x04,    //C604                   //el puente rojo


   40,2,CUSTOM_A_PAL_8|CUSTOM_B_PAL_8,0xC4, 0xF1,    //C4B0 + 65 caracter 16-19    120-144
   40,2,CUSTOM_A_PAL_8|CUSTOM_B_PAL_8,0xC4, 0xE2,    //C4B0 + 50 caracter 16-19    120-144
   40,2,CUSTOM_A_PAL_8|CUSTOM_B_PAL_8,0xC4, 0xD3,    //C4B0 + 35 caracter 16-19    120-144






   40,4,CUSTOM_A_PAL_0|CUSTOM_B_PAL_0,0xC4, 0x60,    //C460  caracter 15-19    120-144          ----------EDIFICIOS BACKGROUND----------
   32,8,CUSTOM_A_PAL_0|CUSTOM_B_PAL_0,0xC4, 0xB4,    //C4B0  caracter 15-19    120-144
   24,6,CUSTOM_A_PAL_0|CUSTOM_B_PAL_0,0xC5, 0x0D,    //C4B0  caracter 16-19    120-144


   48,8,CUSTOM_A_PAL_4|CUSTOM_B_PAL_4,0xC4, 0x11,    //C410  caracter 14-19    120-144          ----------EDIFICIOS FOREGROUND----------
   40,5,CUSTOM_A_PAL_4|CUSTOM_B_PAL_4,0xC4, 0x6A,    //C410  caracter 15-19    120-144          
   
   48,4,CUSTOM_A_PAL_14|CUSTOM_B_PAL_14,0xC4, 0x15,    //C4B0  caracter 16-19    120-144       
   40,3,CUSTOM_A_PAL_14|CUSTOM_B_PAL_14,0xC4, 0x6C,    //C410  caracter 15-19    120-144          

   32,1,CUSTOM_A_PAL_4|CUSTOM_B_PAL_4,0xC4, 0xB8,    //C4B0  caracter 16-19    120-144      
   30,1,CUSTOM_A_PAL_4|CUSTOM_B_PAL_4,0xD4, 0xB7,    //C4B0  caracter 16-19    120-144      
   28,1,CUSTOM_A_PAL_4|CUSTOM_B_PAL_4,0xE4, 0xB6,    //C4B0  caracter 16-19    120-144      
   26,1,CUSTOM_A_PAL_4|CUSTOM_B_PAL_4,0xF4, 0xB5,    //C4B0  caracter 16-19    120-144      


   4,2,CUSTOM_A_PAL_11|CUSTOM_B_PAL_11,0xD4, 0x12,    //C410  caracter 14-19    120-144       -------LUCES ------------------
   4,2,CUSTOM_A_PAL_0|CUSTOM_B_PAL_0,0xD4, 0x62,    //C410  caracter 14-19    120-144       -------LUCES ------------------
   4,2,CUSTOM_A_PAL_11|CUSTOM_B_PAL_11,0xD4, 0xB2,    //C410  caracter 14-19    120-144       -------LUCES ------------------
   4,2,CUSTOM_A_PAL_11|CUSTOM_B_PAL_11,0xD5, 0x02,    //C410  caracter 14-19    120-144       -------LUCES ------------------
   4,2,CUSTOM_A_PAL_0|CUSTOM_B_PAL_0,0xD5, 0x52,    //C410  caracter 14-19    120-144       -------LUCES ------------------

   4,2,CUSTOM_A_PAL_0|CUSTOM_B_PAL_0,0xD4, 0x16,    //C410  caracter 14-19    120-144       -------LUCES ------------------
   4,2,CUSTOM_A_PAL_11|CUSTOM_B_PAL_11,0xD4, 0x66,    //C410  caracter 14-19    120-144       -------LUCES ------------------
   4,2,CUSTOM_A_PAL_0|CUSTOM_B_PAL_0,0xD4, 0xB6,    //C410  caracter 14-19    120-144       -------LUCES ------------------
   4,2,CUSTOM_A_PAL_0|CUSTOM_B_PAL_0,0xD5, 0x06,    //C410  caracter 14-19    120-144       -------LUCES ------------------
   4,2,CUSTOM_A_PAL_11|CUSTOM_B_PAL_11,0xD5, 0x56,    //C410  caracter 14-19    120-144       -------LUCES ------------------


   2,2,CUSTOM_A_PAL_11|CUSTOM_B_PAL_11,0xD4, 0x6A,    //C410  caracter 14-19    120-144       -------LUCES ------------------
   2,2,CUSTOM_A_PAL_0|CUSTOM_B_PAL_0,0xD4, 0xBA,    //C410  caracter 14-19    120-144       -------LUCES ------------------
   2,2,CUSTOM_A_PAL_11|CUSTOM_B_PAL_11,0xD5, 0x0A,    //C410  caracter 14-19    120-144       -------LUCES ------------------
   2,2,CUSTOM_A_PAL_11|CUSTOM_B_PAL_11,0xD5, 0x5A,    //C410  caracter 14-19    120-144       -------LUCES ------------------
   2,2,CUSTOM_A_PAL_0|CUSTOM_B_PAL_0,0xD5, 0xAA,    //C410  caracter 14-19    120-144       -------LUCES ------------------

   4,1,CUSTOM_A_PAL_0|CUSTOM_B_PAL_0,0xD4, 0x6D,    //C410  caracter 14-19    120-144       -------LUCES ------------------
   4,1,CUSTOM_A_PAL_11|CUSTOM_B_PAL_11,0xD4, 0xBD,    //C410  caracter 14-19    120-144       -------LUCES ------------------
   4,1,CUSTOM_A_PAL_0|CUSTOM_B_PAL_0,0xD5, 0x0D,    //C410  caracter 14-19    120-144       -------LUCES ------------------
   4,1,CUSTOM_A_PAL_11|CUSTOM_B_PAL_11,0xD5, 0x5D,    //C410  caracter 14-19    120-144       -------LUCES ------------------
   4,1,CUSTOM_A_PAL_11|CUSTOM_B_PAL_11,0xD5, 0xAD,    //C410  caracter 14-19    120-144       -------LUCES ------------------

   0

};


const u8 sonidosFXarmas[]={
   FX_PUNO,    //puño
   FX_CORTE,      //navaja
   FX_PUNO,    //bate
   FX_GOLPE,      //nunchuck
   FX_CORTE,      //katana
   FX_DISPARO,       //pistola
   FX_DISPARO,       //shotgun
   FX_SUSTIB,       //granada
   FX_SUSTIB       //bazoka

};


void gtaMainMenu();
void gtaMisionStart();
void gtaRestauraCRTCeinterrupt();
void gtaMostrarFinal();






//funciones de prota
void gtaProtaStepFunc();         //step, mueve al prota del juego (andando)
void gtaTargetStepFunc();           //step, coloca el target para saber donde tenemos que ir (suponiendo que tengamos que ir a algun sitio)
void gtaPersonaIAStepFunc();        //draw, dibujar una persona del juego (andando) vale para mujer y hombre
void gtaColisionPersonaFunc();      //funcion de colision para las personas

void gtaCocheStepFunc();         //step, mueve a un coche (mira si tiene conductor o no)
void gtaCocheDrawFunc();        //draw, dibujar una persona del juego (andando) vale para mujer y hombre
void gtaCocheColisionFunc();     //funcion de colision para los coches
void gtaItemStepFunc();          //funcion de step para el item (controla si se tiene que desactivar el item o no)
void gtaItemColisionFunc();      //funcion de colision para los items (le da el arma al jugador y destruye el item)
void gtaAnimationStepFunc();     //funcion de step para un objeto que solo es una animacion y se destruye, no hace nada más, ni colision ni nada

void gtaBaseCleanAndPaintFunc();    //funcion step de la base de clean and paint, este step lo unico que hace es pintar el coche si el prota esta cerca, solo eso


//void gtaCreaHit();               //crea un objeto de animacion con grafiquito de un circulito rojo simulando el golpe para dar feedback al jugador

void gtaDisparoColisionFunc();      //funcion de colision para los disparos


void compruebaRoboCoche();       //comprueba si IX tiene que robar un coche o no, devuelve 0 si ha robado el coche o >1 si no lo ha robado
void gtaCreaArma();              //crea el arma pegada al personaje para donde este mirando IX (o hace un puñetazo si no tiene arma)
void gtaCreaBala();              //cuando se acaba la animación de pegar/disparar se llama a esta funcion con IX el objeto weapon para que haga el daño o cree el objeto bullet

void gtaEsPersonaHL();           //comprueba si HL es una persona A=1 o no lo es A=0
//void gtaEsCocheHL();             //comprueba si HL es un coche A=1 o no lo es A=0

void gtaDrawLight();             //dibuja una luz de un coche o farola, DE=sprite de luz a dibujar, HL=posicion pantalla


//void moveCamera();               //ajusta donde debe dirigirse la camara seteando las variables scrollToTileX y scrollToTileY
void moveCamera2();
void moveCamera3();

void debug1();
void debug2();

void gtaDistanceIX_IY();                //calcula la distancia entre el objeto IX y el objeto IY y la devuelve en HL
void gtaCocheCheckColisionScene();     //calcula si un coche colisiona con algun tile del escenario solido

void gtaPersonaCheckColisionScene();      //comprueba si la persona IX colisiona con el escenario o no, corrije su posicion en funcion de la velocidad
//void gtaPersonaCheckDisableObject();      //comprueba si el objeto activo esta muy lejos de la camara para desactivarlo
void gtaLostTimeFunc();                   //funcion que usa el engine para perder tiempo (se aprovecha este tiempo para reordenar instancias o desactivarlas si estan fuera)
void gtaPreDrawFunc();                 //funcion que se llama al principio de comenzar a dibujar antes de los eventos draw
void gtaPostDrawFunc();                //funcion que se llama al final de cada step del juego, de momento la voy a usar para dibujar las conversaciones

void gtaGameController();              //esta funcion controlara el juego (creará personajes para que vaya entrando en camara, controlara ese tipo de cosas del juego)
void gtaCambiaPaleta();                //va haciendo el ciclo mañana/tarde/noche cambiando la paleta

void gtaVuelveAtrasUnPaso();           //hace que IX vuelva atras un paso (usando su velocidad actual)
u8 checkTileMyCar();
u8 checkTilePerson();
void iaPersonCheckCanWalk();        //devuelve A=1 si el personaje puede andar en la dirección en la que va (hay aceras o pasos de cebra), o 0 si no "debe" andar por ahi (colision o carretera)

void iaPersonRelajado();      //se espera en el sitio mirando a izq y derecha parado un ratito, luego pasa a paseando
void iaPersonPaseando();      //se pone a andar despacio por la acera, cuando llegue a una esquina se para a pensar si cruzar la calle o cambiar de direccion
void iaPersonBuscandoAcera();    //cuando se queda perdido en medio de la carretera, busca la acera más cercana y va rapido hacia alla (igual que el Paseando, pero primero busca una acera y luego va el doble de rapido)
void iaPersonEmpujada();         //cuando se atropella a esta persona y tiene que mantener la velocidad del empujon un poco antes de cambiar a relajado o huyendo
//void iaPersonCruzandoCalle(); //va a cruzar por el paso de cebra a la otra acera
//void iaPersonEsperandoSemaforo();   //espera a que la circulación se detenga en ese sentido y entonces pasa a cruzando
void iaPersonHuyendo();       //despues de escuchar un disparo o golpe intenta salir de la pantalla lo más rapido posible
void iaPersonEnCoche();          //va montado en un coche (en principio no debe hacer nada)
void iaPersonAtacandoProta();		//ajusta angulo hacia el prota, si lleva melee y esta cerca ataca al prota, si lleva range ataca desde donde este apuntando al prota
void iaPersonSiguiendoProta();            //sigue al prota andando
void iaPersonSiguiendoProtaSubeCoche();      //sigue al prota andando y si el prota se monta en un coche, se monta él tambien
void iaPersonSiguiendoPunto();         //es igual que paseando pero no cambia de estado ni cambia su aceleración hasta que llega al punto correcto

void gtaCocheIAgotoPoint();         //hace que el coche vaya a punto concreto marcado por OBJ_CAR_TILETO_X y OBJ_CAR_TILETO_Y
void gtaCocheIAgotoProta();         //hace que el coche vaya a saco a por el prota para atropellarlo o chocarle (usa la func anterior)

void gtaAsustaViejas();			//funcion que se llamara cuando alguien dispare o saque un arma, recorre todas las personas y las asusta o las enfada random

void gtaMisionController();      //controlador de misiones, va gestionando la mision actual (si hay alguna activa)



void main(void) {
   cpct_disableFirmware();
	cpct_setStackLocation(0xBF00);   //dejo una pagina de seguridad sin nada entre BF00 y C000, esto lo hago porque los sprites precompilados 
   //cuando estan dibujando la linea no miran que se hayan pasado de C000 hacia abajo, y por lo tanto podrían sobreescribir la pila
   //como una linea son 80 bytes solo es necesario dejarles ese hueco de más por si se pasan, luego, cuando cambie a la siguiente linea
   //se vuelve a ajustar que caiga dentro de los ultimos 16KB >0xC000 y no vuelve a pasar más
   //por ejemplo, si el sprite empieza a dibujarse en C010 y tiene que dibujar 20 bytes llegara a ser menor que C000 pero nunca menor que BF00

   vtPointerAux=&spr_minifont_00[0];      //seteamos la fuente que va a usar el engine
   vtSetFont();

   vtStartEngine();                       //configuramos el arranque del engine

   vtTimeLostFunc=gtaLostTimeFunc;     //seteo la funcion de perdida de tiempo, en nuestro caso ira añadiendo personas por el escenario
   vtPreDrawFunc=gtaPreDrawFunc;
   vtPostDrawFunc=gtaPostDrawFunc;     //seteo la funcion de postdraw, de momento la usaremos para dibujar las conversaciones
   vtGameController=gtaGameController;
   
   while(1)
   {
		gtaPaletaSeleccionada=2;      //hago que use la paleta normal reusando las funciones del juego
		gtaCambiaPaleta();
      vtUsePaletteGame();        

      

      //aqui comprobar si nos han matado o nos hemos pasado el juego, bastaria con ver si la mision actual es mayor que X (la ultima)
      if(misionActual==MAX_MISIONES)
      {
	      //y en caso afirmativo mostrar el final, sino simplemente volver al bucle
	      gtaMostrarFinal();
	   }

      gtaMainMenu();       //menu principal simplon

      vtResetEngine();       //esto inicializa todos los objetos y el scroll del engine (se tiene que llamar siempre despues de mostrar el menu)

      vtSceneScrollX=55;      //posicion x de la scena donde estamos (en tiles)  (va a ir de 0 a 256)  (tamaño 6 pantallas en horizontal)
      vtSceneScrollY=104;     //posicion y ...									         (va a ir de 0 a 256)  (tamaño 10 pantallas en vertical)
      lostTimeIndex=GAME_OBJECTS_PTR+48;     //el cero es el prota, no me lo quiero cargar

      
      
      vtGetFreeGameObjPtr();                 //preparamos el objeto del prota
      vtGameObjPtr->activeType=TYPE_PROTA;
      vtGameObjPtr->stepFunc=gtaProtaStepFunc;  //el prota que lo sigue la camara
      vtGameObjPtr->life=PROTA_START_LIFE;
      vtGameObjPtr->customColor=CUSTOM_A_PAL_5 | CUSTOM_B_PAL_0;
      vtGameObjPtr->colisionFunc=gtaColisionPersonaFunc;
      vtGameObjPtr->prepareDrawFunc=vtDrawObjectFunc;
      vtGameObjPtr->animationArrayPtr=(u8*)&animWomanWalk[0];    
      vtGameObjPtr->frameSpeed=96;
      vtGameObjPtr->animationMax=4;

      vtGameObjPtr->posx=(70)*256; //aqui uno menos porque empieza a partir del bit 7 y no bit 8
      vtGameObjPtr->posy=(122)*256;
      vtGameObjPtr->widthTiles=2;
      vtGameObjPtr->heightTiles=2;     //cuanto ocupa en vertical el bixo (en tiles), ojo que el punto cero es abajo en el centro
      vtGameObjPtr->flags=FLAG_AUTO_MOVE | FLAG_AUTO_ANIM;

      vtGameObjPtr->animationArrayPtr2=(u8*)&animHairWalkLargo[0]; 
      vtGameObjPtr->customColor2=CUSTOM_A_PAL_0 | CUSTOM_B_PAL_0;

      vtGameObjPtr->aux5=WEAPON_PUNO;   //SELECTED WEAPON empiezo con puños
      


      vtGetFreeGameObjPtr();        //preparamos el objeto para el punto de mira, que se va a quedar ahi con el mismo tipo del prota pero sin colision para que no influya en nada
      vtGameObjPtr->activeType=TYPE_PROTA;
      vtGameObjPtr->stepFunc=gtaTargetStepFunc;  //el prota que lo sigue la camara
      vtGameObjPtr->animationArrayPtr=(u8*)&animTarget[0];
      vtGameObjPtr->animationMax=2;
      vtGameObjPtr->widthTiles=1;
      vtGameObjPtr->heightTiles=1;     //cuanto ocupa en vertical el bixo (en tiles), ojo que el punto cero es abajo en el centro
      vtGameObjPtr->flags=FLAG_NO_COLISION|FLAG_NO_DESTROY;
      vtGameObjPtr->iaState1=128;      //esto es para que no borre el objeto aunque este fuera

      gtaMisionResetAll();       //resetea todas las misiones y arranca la mision 0
      
      
      cpct_akp_musicInit(g_mysong);
      cpct_akp_SFXInit(g_mysong);

      vtRunEngine();	//entro en el bucle del engine (importante, crea algun objeto que controle algo o el engine no hara nada)

	   cpct_akp_stop();				//detengo todos los sonidos
	   
      gtaRestauraCRTCeinterrupt();     //aqui se tendria que parar la interrupcion custom y recuperar los valores del CRTC
      
      
   }
}




void gtaMainMenu() __naked {
   __asm
//un u8 para los bytes verticales a pintar  (0 para acabar de pintar)
//un u8 para los bytes horizontales a pintar
//un u8 para el color a usar
//u16 para la direccion de video  (va al reves para que sea más facil leer, byte alto, byte bajo)


      ld hl,#_menuPrincipalCuadrados

   gtaMainMenus1:
      ld b,(hl)
      xor a
      or b
      jr Z,gtaMainMenuFlipLinesDither
      inc hl 
      ld c,(hl)
      inc hl 
      ld a,(hl)
      inc hl 
      ld d,(hl)
      inc hl 
      ld e,(hl)
      inc hl 
      push hl 

      call cpct_drawSolidBox_asm

      pop hl 
      jr gtaMainMenus1


   gtaMainMenuFlipLinesDither:
      ld hl,#0xC190
      call gtaMainMenuFlipLine
      /*ld hl,#0xD190
      call gtaMainMenuFlipLine
      ld hl,#0xE190
      call gtaMainMenuFlipLine
      ld hl,#0xF190
      call gtaMainMenuFlipLine*/
      ld hl,#0xC1E0
      call gtaMainMenuFlipLine
      /*ld hl,#0xD1E0
      call gtaMainMenuFlipLine
      ld hl,#0xE1E0
      call gtaMainMenuFlipLine
      ld hl,#0xF1E0
      call gtaMainMenuFlipLine*/

      ld hl,#0xC3C0
      call gtaMainMenuFlipLine
      /*ld hl,#0xD3C0
      call gtaMainMenuFlipLine
      ld hl,#0xE3C0
      call gtaMainMenuFlipLine
      ld hl,#0xF3C0
      call gtaMainMenuFlipLine*/
      ld hl,#0xC410
      call gtaMainMenuFlipLine
      /*ld hl,#0xD410
      call gtaMainMenuFlipLine
      ld hl,#0xE410
      call gtaMainMenuFlipLine
      ld hl,#0xF410
      call gtaMainMenuFlipLine*/



      


   gtaMainMenus2:
      ld hl,#0xCDB2
      ld (#_vtScreenPtr),hl 
      ld hl,#_entradaSpr
      ld (#_vtSpritePtr),hl
      call _vtRenderTS

      ld hl,#0xC410+70
      ld (#_vtScreenPtr),hl 
      ld hl,#_lunaSpr
      ld (#_vtSpritePtr),hl
      call _vtRenderTS





   graMainMenuDrawTexts:
      ld a,#CUSTOM_A_PAL_9|CUSTOM_B_PAL_9        //el color de las letras
      ld de,#0xC01E
      ld hl,#_frase8
      call graMainMenuDrawTextsSub


      ld a,#CUSTOM_A_PAL_7|CUSTOM_B_PAL_7        //el color de las letras
      ld de,#0xC0A7
      ld hl,#_frase9
      call graMainMenuDrawTextsSub


      ld a,#CUSTOM_A_PAL_11|CUSTOM_B_PAL_11
      ld de,#0xC154
      ld hl,#_frase10
      call graMainMenuDrawTextsSub


      ld a,#CUSTOM_A_PAL_15|CUSTOM_B_PAL_15 
      ld de,#0xC1EF
      ld hl,#_frase11
      call graMainMenuDrawTextsSub


      //pekeño bucle para que se suelten todas las teclas antes de seguir
   /*gtaMainMenuReleaseKeys:
      call cpct_scanKeyboard_asm
      call cpct_isAnyKeyPressed_asm
      or a 
      jr NZ,gtaMainMenuReleaseKeys*/




   gtaMainMenuWaitFire:
      call gtaMainMenuAnimWater

      call cpct_scanKeyboard_asm

      ld hl,#0x4007        //Key_C
	  	call _cpct_isKeyPressed   			//mete en l 1 si se esta pulsando la tecla
      ld a,l                        //meto el pressed en a
      or a
      jr Z,gtaMainMenuWaitFires2


   gtaMainMenuWaitFires1:
      call cpct_scanKeyboard_asm
      ld hl,#0x4007        //Key_C
	  	call _cpct_isKeyPressed   			//mete en l 1 si se esta pulsando la tecla
      ld a,l                        //meto el pressed en a
      or a
      jr NZ,gtaMainMenuWaitFires1


      //si entra aqui hay que configurar las teclas
      ld a,#CUSTOM_A_PAL_15|CUSTOM_B_PAL_15 
      ld de,#0xC28F
      ld hl,#_frase12
      call graMainMenuDrawTextsSub
      call gtaWaitKeyPress
      //tengo en HL la tecla de arriba
      ld (#protaStepFuncCheckKeys4+1),hl

      ld a,#CUSTOM_A_PAL_15|CUSTOM_B_PAL_15 
      ld de,#0xC28F
      ld hl,#_frase13
      call graMainMenuDrawTextsSub
      call gtaWaitKeyPress
      //tengo en HL la tecla de abajo
      ld (#protaStepFuncCheckKeys3+1),hl

      ld a,#CUSTOM_A_PAL_15|CUSTOM_B_PAL_15 
      ld de,#0xC28F
      ld hl,#_frase14
      call graMainMenuDrawTextsSub
      call gtaWaitKeyPress
      //tengo en HL la tecla de izq
      ld (#protaStepFuncCheckKeys2+1),hl

      ld a,#CUSTOM_A_PAL_15|CUSTOM_B_PAL_15 
      ld de,#0xC28F
      ld hl,#_frase15
      call graMainMenuDrawTextsSub
      call gtaWaitKeyPress
      //tengo en HL la tecla de derecha
      ld (#protaStepFuncCheckKeys1+1),hl

      ld a,#CUSTOM_A_PAL_15|CUSTOM_B_PAL_15 
      ld de,#0xC28F
      ld hl,#_frase16
      call graMainMenuDrawTextsSub
      call gtaWaitKeyPress
      //tengo en HL la tecla de fire
      ld (#protaStepFuncCheckKeys5+1),hl
      ld (#gtaMainMenuWaitFires2+1),hl



   gtaMainMenuWaitFires2:
      //ld hl,#0x8005        //Key_SPACE
      ld hl,#0x1009        //JOY FIRE 1
	  	call _cpct_isKeyPressed   			//mete en l 1 si se esta pulsando la tecla
      ld a,l                        //meto el pressed en a
      or a
      jr Z,gtaMainMenuWaitFire
      ret








      //recibe hl el comienzo de una linea y le hace flip a todos los bytes para cambiar los dos pixeles de lugar
   gtaMainMenuFlipLine:
      push hl
      ld b,#80
      ld d,#FLIP_TABLE_PAGE
   gtaMainMenuFlipLineBuc:
      ld e,(hl)
      ld a,(de)
      ld (hl),a
      inc hl
      djnz gtaMainMenuFlipLineBuc

      pop hl
      ld a,#0x10
      add h
      ret C	//si hay carry ya hemos llegado a la ultima linea del caracter, tenemos que volver

      //sino le hemos sumado 0x10 a H y volvemos a flipLine
      ld h,a
      jr gtaMainMenuFlipLine
      
      
      



   gtaMainMenuAnimWater:
      call cpct_waitVSYNC_asm


      ld hl,#0xFD50
      ld de,#0xC640
      push de 
      push hl
      ld a,#39

      
      
   gtaMainMenuAnimWaterBucle:
      push af 
      push hl 
      push bc 

      call cpct_getRandom_lcg_u8_asm
      and #3
      add e 
      ld e,a

      pop bc 
      pop hl 
      pop af


      ld bc,#78
      ldir


      pop hl 
      pop de

      dec a
      ret Z 

      push af
      call gtaNextLineDE
      call gtaPrevLineHL
      pop af

      push de
      push hl 

      jr gtaMainMenuAnimWaterBucle
      





   gtaNextLineDE:
      ld a,#8           //le sumo a H = H+8
      add d
      ld d,a
      ret NC            //si no hay carry ya esta pa fuera
      add #0xC0         //si hay carry le sumo 0xC0 para que se coloque en su sitio
      ld d,a
      ld a,#0x50        //y ademas le sumo 0x50 a L para pasar al siguiente caracter
      add e 
      ld e,a
      ret NC            //si no hay carry ya esta, fuera
      inc d             //si hay carry hay que sumarle uno a H y fuera
      ret


   gtaPrevLineHL:
      ld a,h
      sub #8
      ld h,a
      bit 6,h
      ret NZ         //si H sigue siendo igual o mayor que 0xC0 ya esta, fuera

      push de
      ld de,#0x3FB0
      add hl,de
      pop de

      ret


   graMainMenuDrawTextsSub:
      push af
      push de
      call _gtaCreaFrase
      //ld hl,#_talkFrase
      ld hl,#TALK_FRASE
      ld (#_vtStringPtr),hl
      pop hl
      ld (#_vtScreenPtr),hl
      pop af
      ld (#_vtDrawSpriteCustomColor1),a
      call _vtDrawString
      ret



   gtaWaitKeyPress:
      call cpct_scanKeyboard_asm
      ld hl,#0x0100

   gtaWaitKeyPresss1:
      push hl
	  	call _cpct_isKeyPressed   			//mete en l 1 si se esta pulsando la tecla
      ld a,l                        //meto el pressed en a
      or a
      pop hl
      jr Z,gtaWaitKeyPresss2

   gtaWaitKeyPresss1re:
      push hl
      call cpct_scanKeyboard_asm
      pop hl 
      push hl
	  	call _cpct_isKeyPressed 
      ld a,l
      or a 
      pop hl
      jr NZ,gtaWaitKeyPresss1re

      //esta es la tecla pulsada
      ret 
   
   gtaWaitKeyPresss2:
      //paso a la siguiente tecla
      inc l
      ld a,#9
      cp l 
      jr NZ,gtaWaitKeyPresss1
      ld l,#0
      sla h
      jr NC,gtaWaitKeyPresss1
      //si hay carry hemos llegao al final, volvemos al principio
      jr gtaWaitKeyPress



   gtaMainMenuRand:
      .db #0

   __endasm;
}



/*
calcular si se tuviese que dibujar el background, en que lugar se haria, setear los parametros del pushTiles

calcular la IA
	- si se mueve pon flag(1) a 1
	- si cambia de frameactual pon flag(1) a 1
	

si ahora esta dentro de la pantalla

	si flag(2)=0	//si acaba de entrar
		flag(0)=0		//no dibujamos el fondo
		flag(1)=1		//si dibujamos el sprite
	sino			//si ya estaba dentro
		flag(0)=flag(1)		//redibujamos el fondo si vamos a redibujar el sprite, sino no
	
	flag(2)=1  
	
sino

	flag(0)=flag(2)			//solo redibujamos el fondo, si antes estabamos dentro, para limpiar al salir, sino no
	flag(1)=0				//si esta fuera de la pantalla no podemos dibujar el sprite en ningun caso, kaka, aunque se mueva
	flag(2)=0  				//actualizamos que ahora estamos fuera de la pantalla
*/




void gtaProtaStepFunc() __naked {
   __asm
   	//ld hl,(#_vtGameObjPtr)
   	//push hl
      //call _gtaGameController       //antes de nada, llamamos al controlador del juego y luego a todo lo demas
      ld ix,(#_vtGameObjPtr)
      //pop ix
      //ld (#_vtGameObjPtr),ix
      //ahora calculamos la IA o los controles, si cambia algo poner flag(1) a 1

      //vamos a usar E para is guardando todas las pulsaciones en los bits 000FUDLR ya que la instruccion cpct_isKeyPressed no destruye E
      xor a
      ld e,a      //E=0

      
      
   protaStepFuncCheckKeysPre:
      ld a,OBJ_IA_LOCK_MOVE(ix)     //antes de mirar si esta pulsando algun lado miro si podemos movernos o no
      or a
      jr Z,protaStepFuncCheckKeys

      dec a          //si entro aqui no me puedo mover, decremento 1 y me voy al final
      ld OBJ_IA_LOCK_MOVE(ix),a
      jp NZ,protaStepFuncCheckKeysFin


      //si despues de bajar 1 es cero, tengo que volver a poner la animacion de andar normal
      /*bit 1,OBJ_ACTIVE_TYPE(ix)           //compruebo si es hombre/mujer
      jr NZ,protaStepFuncRecuperaAniWalkMan        //si este bit es cero es mujer

      ld hl,#_animWomanWalk
      jr protaStepFuncRecuperaAniWalk

   protaStepFuncRecuperaAniWalkMan:
      ld hl,#_animManWalk

   protaStepFuncRecuperaAniWalk:
      ld OBJ_ANIMATION_LIST(ix),l
      ld OBJ_ANIMATION_LIST+1(ix),h*/
      call _gtaSetAnimWalkIX


   protaStepFuncCheckKeys:       //TODAS ESTAS COMPROBACIONES DE PULSACIONES
      ld hl,#0x0408        //Key_ESC
	  	call _cpct_isKeyPressed   			//mete en l 1 si se esta pulsando la tecla
      ld a,l                        //meto el pressed en a
      or a
      jr Z,protaStepFuncCheckKeys1

      //si entro aqui es que quiero sacar el status actual
      push ix
      ld (#_talkInstancePtr),ix
      call _gtaFillStatusStr
      ld de,#TALK_FRASE
      ld hl,#_statusStr
      call _gtaCopiaCadenaHLenDE
      pop ix



   protaStepFuncCheckKeys1:
      //ld hl,#0x2007        //Key_D
      ld hl,#0x0809
	  	call _cpct_isKeyPressed   			//mete en l 1 si se esta pulsando la tecla
      ld a,l                        //meto el pressed en a
      or a
      jr Z,protaStepFuncCheckKeys2
	  	//aqui ha pulsado la D
      set 0,e
   
   protaStepFuncCheckKeys2:
      //ld hl,#0x2008        //Key_A
      ld hl,#0x0409
	  	call _cpct_isKeyPressed   			//mete en l 1 si se esta pulsando la tecla
      ld a,l                        //meto el pressed en a
      or a
      jr Z,protaStepFuncCheckKeys3
	  	//aqui ha pulsado la A
      set 1,e
   

   protaStepFuncCheckKeys3:
      //ld hl,#0x1007        //Key_S
      ld hl,#0x0209
	  	call _cpct_isKeyPressed   			//mete en l 1 si se esta pulsando la tecla
      ld a,l                        //meto el pressed en a
      or a
      jr Z,protaStepFuncCheckKeys4
	  	//aqui ha pulsado la S
      set 2,e
     

   protaStepFuncCheckKeys4:
      //ld hl,#0x0807        //Key_W
      ld hl,#0x0109
	  	call _cpct_isKeyPressed   			//mete en l 1 si se esta pulsando la tecla
      ld a,l                        //meto el pressed en a
      or a
      jr Z,protaStepFuncCheckKeys5
	  	//aqui ha pulsado la W
      set 3,e

   protaStepFuncCheckKeys5:
      //ld hl,#0x8005        //Key_SPACE
      ld hl,#0x1009           //joy fire 1
	  	call _cpct_isKeyPressed   			//mete en l 1 si se esta pulsando la tecla
      ld a,l                        //meto el pressed en a
      or a
      jr Z,protaStepFuncCheckKeysFinPre
	  	//aqui ha pulsado la SPACE
      set 4,e
      inc OBJ_PROTA_ACUM_FIRE(ix)     //voy a usar la AUXILIAR 1 para que vaya cargando el espacio
      jp protaStepFuncCheckKeysFin


   protaStepFuncCheckKeysFinPre:
      ld a,OBJ_PROTA_ACUM_FIRE(ix)
      or a
      jr Z,protaStepFuncCheckKeysFin
      

      
      



      //push de
      cp #CICLOS_FUEGO_PULSADO_PARA_ROBAR_COCHE      //12 fps por segundo, 24 son 2 segundos pulsado el espacio
      call NC,_compruebaRoboCoche

      or a
      jr NZ,protaStepFuncCheckKeysFinPre2

      
      //si entro aqui es que he robado el coche, hago que se limpie el fondo para que no deje el muñeco a medio dibujar y luego sigo
      //ld a,OBJ_FLAGS(ix)
      //or #0b00000010
      //ld OBJ_FLAGS(ix),a
      set 1,OBJ_FLAGS(ix)

      xor a
      ld OBJ_PROTA_ACUM_FIRE(ix),a    //pongo a 0 el contador de frames de fuego pillado
      ld e,#0  //desactivo todas las pulsaciones cuando vaya a disparar
      push ix
      call _vtStepObjectFunc
      pop ix
      jr protaStepFuncCheckKeysFin



   protaStepFuncCheckKeysFinPre2:
      //or a        //compruebaRoboCoche devuelve 0 si ha robado el coche, >0 si no lo ha robado, por lo tanto tiene que disparar
      //call NZ,_gtaCreaArma
      call _gtaCreaArma
      xor a
      ld OBJ_PROTA_ACUM_FIRE(ix),a    //pongo a 0 el contador de frames de fuego pillado
      //pop de
      ld e,#0  //desactivo todas las pulsaciones cuando vaya a disparar


   protaStepFuncCheckKeysFin:
      //ahora hago el ajuste de velocidad maxima
      ld a,OBJ_PARENT_PTR(ix)
      or a
      jr NZ,protaStepFuncMueveCoche

      ld hl,#_protaListaKeyMove
      ld d,#0
      ld a,#0b00001111
      and e
      ld b,a
      add a    //x2
      add b    //x3
      ld e,a   //lo vuelvo a meter en DE
      add hl,de
      ld c,(hl)      //pillo interpolacion
      inc hl
      ld b,(hl)      //pillo velocidad
      inc hl
      ld a,(hl)      //pillo angulo

      call _vtAccelToSpeedAng
   
      ld l,OBJ_POSX(ix)
      ld h,OBJ_POSX+1(ix)
      ld (#_oldPosX),hl
      ld l,OBJ_POSY(ix)
      ld h,OBJ_POSY+1(ix)
      ld (#_oldPosY),hl


      


	  	//y ahora movemos al personaje
      ld OBJ_NEED_REDRAW(ix),#1     //vtGameObjPtr->needRedraw=1;
      //flags(1)=1 necesito redibujar sprite
      //vtGameObjPtr->flags=vtGameObjPtr->flags | 0b00000010; //le agregamos el 1 para que lo pinte el sprite      
      //ld a,OBJ_FLAGS(ix)
      //or #0b00000010
      //ld OBJ_FLAGS(ix),a
      set 1,OBJ_FLAGS(ix)
      

      //actualizo frameSpeed con la velocidad que llevo horizontal por lo menos
      ld c,OBJ_SPEEDX(ix)
      ld b,OBJ_SPEEDX+1(ix)
      ld e,OBJ_SPEEDY(ix)
      ld d,OBJ_SPEEDY+1(ix)
      call _vtDistance16


      ld a,OBJ_IA_LOCK_MOVE(ix)     //antes de ajustar la velocidad en funcion del movimiento, si estamos disparando no tenemos que calcularla
      or a
      jr NZ,protaStepFuncCheckColis
      ld OBJ_FRAME_SPEED(ix),l


   protaStepFuncCheckColis:
      //llamo al step basico que hace los ajustes de flags y redibuja el background si hace falta
      call _vtStepObjectFunc 

      //comprobamos colisiones con el escenario (esto ajusta la posicion en caso de haber colision)
      call _gtaPersonaCheckColisionScene


   protaStepFuncMoveCam:
      //ret
      //call _trapIX

      jp _moveCamera3    //el moveCamera ya hara el ret si le sale de los huevos
      //no hago ret, lo hace el movecamera
		



   protaStepFuncMueveCoche:
      //si entro aqui el prota va montado en un coche, 
      //puede pasar dos cosas, que conduzca yo, o que conduzca otro, en el primer caso tengo que actualizar yo los datos del coche, 
      //en el segundo solo tengo que actualizar mis datos
      ld OBJ_IA_LOCK_MOVE(ix),#0
      ld l,OBJ_PARENT_PTR(ix)
      ld h,OBJ_PARENT_PTR+1(ix)
      push hl
      pop iy         //pongo en IY el puntero al coche

      call _compruebaConductorIXenIY
      or a
      jr Z,protaStepFuncMueveCocheActualizameConCoche

      //si entro aqui soy YO el que controla el coche, por lo tanto puedo modificar el TORQUE y el SPEED (solo hago eso, ya el step del coche se movera en funcion de esto)
      

   protaStepFuncMueveCocheModificoCoche:
      //en E tengo las pulsaciones de las teclas, actualizo TORQUE y SPEED
      //e Derecha, izquierda, abajo, arriba, fuego
      ld a,OBJ_CAR_SPEED(iy)
      cp #MIN_SPEED_TO_TORQUE_CAR
      ld a,OBJ_CAR_TORQUE(iy)
      jr C,protaStepFuncMueveCocheModificoCoche3           //si no vamos a una velocidad minima MIN_SPEED_TO_TORQUE_CAR no podemos girar el coche

      ld a,OBJ_CAR_TORQUE(iy)
      bit 0,e
      jr Z,protaStepFuncMueveCocheModificoCoche2
      push de
      ld b,a         //me guardo el torque
      ld d,#0
      ld e,OBJ_SELECTED_CAR(iy)
      ld hl,#_carPropTorque
      add hl,de
      ld a,(hl)
      add b
      pop de

   protaStepFuncMueveCocheModificoCoche2:
      bit 1,e
      jr Z,protaStepFuncMueveCocheModificoCoche3
      //sub #CAR_SIMPLE_TORQUE
      push de
      ld b,a         //me guardo el torque
      ld d,#0
      ld e,OBJ_SELECTED_CAR(iy)
      ld hl,#_carPropTorque
      add hl,de
      ld a,(hl)
      neg         //lo niego antes y luego lo sumo (es lo mismo que restar)
      add b
      pop de

   protaStepFuncMueveCocheModificoCoche3:
      ld OBJ_CAR_TORQUE(iy),a

      ld a,OBJ_CAR_SPEED(iy)
      bit 2,e
      jr Z,protaStepFuncMueveCocheModificoCoche4
      push de
      //sub #CAR_SIMPLE_BRAKE
      ld b,a         //me guardo la velocidad
      ld d,#0
      ld e,OBJ_SELECTED_CAR(iy)
      ld hl,#_carPropBrake
      add hl,de
      pop de
      ld a,(hl)
      ld c,a
      ld a,b
      sub c


      jr NC,protaStepFuncMueveCocheModificoCoche5
      xor a    //velocidad 0 

   protaStepFuncMueveCocheModificoCoche4:
      
      bit 3,e
      jr Z,protaStepFuncMueveCocheModificoCoche5
      //add #CAR_SIMPLE_ACCEL
      //cp #CAR_SIMPLE_MAX_SPEED
      push de
      ld b,a         //me guardo la velocidad
      ld d,#0
      ld e,OBJ_SELECTED_CAR(iy)
      ld hl,#_carPropAccel
      add hl,de
      ld a,(hl)
      add b
      ld d,#0
      ld e,OBJ_SELECTED_CAR(iy)
      ld hl,#_carPropMaxSpeed
      add hl,de
      pop de
      cp (hl)


      jr C,protaStepFuncMueveCocheModificoCoche5
      //ld a,#CAR_SIMPLE_MAX_SPEED      //velocidad a tope
      ld a,(hl)

   protaStepFuncMueveCocheModificoCoche5:
      ld OBJ_CAR_SPEED(iy),a
      //ya he modificado el coche, ahora me ajusto mi posx con el que tenga el coche


   protaStepFuncMueveCocheActualizameConCoche:
      //aqui tengo que actualizar los datos de mi muñeco con los del coche IY
      ld a,OBJ_POSX+1(iy)        //simplemente actualizo el byte alto del posx y ya ta, no me complico
      ld OBJ_POSX+1(ix),a
      ld a,OBJ_POSY+1(iy)
      ld OBJ_POSY+1(ix),a
      ld a,OBJ_SPEEDX(iy)
      ld OBJ_SPEEDX(ix),a
      ld a,OBJ_SPEEDX+1(iy)
      ld OBJ_SPEEDX+1(ix),a
      ld a,OBJ_SPEEDY(iy)
      ld OBJ_SPEEDY(ix),a
      ld a,OBJ_SPEEDY+1(iy)
      ld OBJ_SPEEDY+1(ix),a


      //y ahora movemos al personaje
      ld OBJ_NEED_REDRAW(ix),#0     //vtGameObjPtr->needRedraw=0;
      //flags(1)=0 no quiero redibuja el sprite 
      //ld a,OBJ_FLAGS(ix)
      //or #0b00000010
      //ld OBJ_FLAGS(ix),a
      res 1,OBJ_FLAGS(ix)     //indico que no quiero redibujar el sprite
      //llamo al step basico que hace los ajustes de flags y redibuja el background si hace falta
      //call _vtStepObjectFunc 

      //ld a,OBJ_CAR_SPEED(iy)
      //cp #SPEED_TO_TURBO_SCROLL
      //jp NC,_moveCamera2       //si la velocidad del coche es mayor que SPEED_TO_TURBO_SCROLL usamos turbo scroll, sino 
      //jp _moveCamera3         //usamos el scroll normal más lentito

      //call _trapIX

      jp _moveCamera2    //el moveCamera ya hara el ret si le sale de los huevos
      //ret      //y salgo, ya el coche se movera con la speed y la torque que le hemos puesto si quiere

/*
   protaStepKeyMapIndex:
      .db 0
*/
   __endasm;
}


void gtaNPCDrawFunc() __naked {
   __asm
      ld ix,(#_vtGameObjPtr)

      ld a,OBJ_PARENT_PTR+1(ix)
      or a
      ret NZ      //si voy en un coche no dibujo nada

      call _vtDrawObjectFunc

   __endasm;
}

void gtaPersonaIAStepFunc() __naked {
   __asm
      //quizas se pueda hacer un jump directo
      call _vtStepObjectFunc     //finalmente llamo al generico

      ld ix,(#_vtGameObjPtr)

      //42-47  
      //u8* iaMachineStatePtr;         //puntero a la funcion de la maquina de estados que controlara la IA de este personaje
      //u8 iaTimeToNextState;          //numero de frames para calcular el siguiente estado
      //u8 iaState3;                   //3 variables auxiliares para controlar ciertos elementos de la IA (cada cual que la use para lo que le haga falta)
      //u8 iaState2;
      //u8 iaState1;

      dec OBJ_FRAMES_FOR_NEXT_STATE(ix)         //le bajo uno a el temporizador de la ia y lo cargo en A
      ld a,OBJ_FRAMES_FOR_NEXT_STATE(ix)        //la cargo en A para que cuando llame a la funcion de la IA haga el cambio si es 0


      ld l,OBJ_MACHINE_STATE_PTR(ix)
      ld h,OBJ_MACHINE_STATE_PTR+1(ix)
      ld de,#gtaPersonaIAStepFuncVuelta
      push de        //pusheo la etiqueta gtaPersonaIAStepFuncVuelta para poder volver luego ahi
      push hl        //pusheo la funcion de la ia a la que tengo que ir
      ret            //y ahora hago un ret para que vaya a HL y luego cuando acabe con ret vuelva a DE osea a gtaPersonaIAStepFuncVuelta

   gtaPersonaIAStepFuncVuelta:

      ld a,OBJ_ID_NPC_CAR(ix)
      inc a
      jr NZ,gtaPersonaIAStepFuncVueltas2

      //si entra aqui me tengo que cargar el NPC ya que se ha solicitado la eliminacion desde gtaMisionDestroyNPC
      ld OBJ_ACTIVE_TYPE(ix),a      //como ya he llamado al step generico y se ha redibujado el fondo, solo tengo que poner a 0 el tipo para que no llame al draw y listo

   gtaPersonaIAStepFuncVueltas2:
		//cuando vuelva aqui miro si le queda vida o no 
      //call _trapIX
		ld a,OBJ_LIFE(ix)
		cp #200
		ret C			//si hay carry aun esta vivo puedo pirarme
		
		//si no hay carry es que ya ha llegado a 0 y se ha pasado (osea menos algo...)
		//le cambio el tipo a la persona y la pongo como un objeto de tipo animacion
		/*
		ld hl,#_animWomanFall
      
		//miro si es hombre o mujer  (si es mujer ya he puesto la ani antes, si es hombre la cambio
		bit 1,OBJ_ACTIVE_TYPE(ix)
		jr Z,gtaPersonaIAStepFuncVueltas3		//si es 0 es una mujer, si es 1 es un hombre
		
		ld hl,#_animManFall
		

	gtaPersonaIAStepFuncVueltas3:
		ld OBJ_ANIMATION_LIST(ix),l			//pongo la ani de caerse
      ld OBJ_ANIMATION_LIST+1(ix),h

		xor a						//comienzo la animacion
		ld OBJ_FRAME_INDEX(ix),a
		ld OBJ_FRAME_INDEX+1(ix),a		*/

      call _gtaSetAnimFallIX

	   ld OBJ_ACTIVE_TYPE(ix),#TYPE_AUX_OBJECT		//hago todos los cambios en la entidad para transformarla en una animacion simple

      ld hl,#_gtaAnimationStepFunc
      ld OBJ_STEP_FUNC(ix),l
      ld OBJ_STEP_FUNC+1(ix),h

      ld hl,#_vtDrawObjectFunc
      ld OBJ_PREPARE_FUNC(ix),l
      ld OBJ_PREPARE_FUNC+1(ix),h

      ld OBJ_ANIMATION_MAX(ix),#4
      ld OBJ_FRAME_SPEED(ix),#16

      ld OBJ_FLAGS(ix),#FLAG_NO_COLISION|FLAG_AUTO_ANIM

      //call _trapIX

      ret
   __endasm;
}


void gtaSemaforoStepFunc() __naked {
   __asm
      ld ix,(#_vtGameObjPtr)     //hago que vaya cambiando de color el semaforo
      ld a,(#_vtClock+1)
      and #3
      ld c,#0   //por defecto lo pongo en verde
      cp #2
      jr C,gtaSemaforoStepFuncs4    //si es <2 ya está, que se quede en verde

      ld c,#2   //si no es rojo menos en un pekeño caso
      jr NZ,gtaSemaforoStepFuncs4      //si no es 2 es que es 3, por lo tanto en rojo y pafuera
      
      //si llego aqui es 2 y aqui puede ser amarillo o rojo, dependerá de si byte bajo de clock es menor de 64
      ld a,(#_vtClock)
      cp #64
      jr NC,gtaSemaforoStepFuncs4

      ld c,#1   //amarillo

      
   gtaSemaforoStepFuncs4:
      ld OBJ_FRAME_INDEX+1(ix),c   //si no es rojo menos en un pekeño caso


   gtaSemaforoStepFuncs10:
      call _vtStepObjectFunc     //finalmente llamo al generico
      ret
   __endasm;
}


void gtaSemaforoDrawFunc() __naked {
   __asm
      //KABESA aqui debe ver si tiene que ir cambiando

      call _vtDrawObjectFunc     //finalmente llamo al generico
      ret
   __endasm;
}

//esta funcion dibuja una persona (osea, el muñeco por un lado con sus colores, y el pelo por otro lado encima)
//para el tipo de pelo que vamos a usar tenemos a OBJ_IA_STATE_1
//hace lo mismo que el draw standard pero al final dibuja el pelo encima
/*void gtaPersonaDrawFunc() __naked {
   __asm
      


      ld ix,(#_vtGameObjPtr)

      ld a,OBJ_NEED_REDRAW(ix)
      or a
      ret Z             //si es cero nos saltamos el dibujado


      ld a,OBJ_FLAGS(ix)
      bit 1,a        //compruebo si hay que dibujar el sprite o no
      jr Z,gtaDrawObjectFuncEnd      //no hay que dibujar el sprite, asi que nos piramos

      //calcula las variables donde se tiene que dibujar el sprite y de paso para restaurar el fondo en el proximo frame
      call _vtGetGameObjScreenPtr
      //agrega a la lista de sprites para dibujar
      call _vtPushSpriteToDraw
      

   gtaDrawObjectFuncEnd:
      ld a,OBJ_FLAGS(ix)
      and #0b11111100
      ld OBJ_FLAGS(ix),a
      ret
   
   __endasm;
}*/


//recuerda en IX esta el principal y en IY el otro que ha colisionado, revisa el tipo del otro antes de hacer nada
void gtaColisionPersonaFunc() __naked {
   __asm
      //call _vtBorderRed

      //miro si IY es una persona o un coche
      ld a,OBJ_ACTIVE_TYPE(iy)
      cp #TYPE_CAR       //miramos si es una persona o no
      jp NC,gtaColisionPersonaFuncPersonaCoche        //es mayor que 8 asi que sera un coche



      //  COLISION PERSONA-PERSONA------------------------------------------------------------------
   gtaColisionPersonaFuncPersonaPersona:
      and #TYPE_CITIZEN|TYPE_PROTA
      ret Z       //si es cero es que ni es una persona ni el prota, lo mejor es salir y no hacer ná


      call _gtaDistanceIX_IY
      xor a
      or h
      ret NZ    //si hay carry es que estan a 2 tiles o más, paso de esta colision entre personas

      
   gtaColisionPersonaFuncPersonaPersonaEmpujon:
      //como los dos son personas y tienen el mismo tamaño, solo tengo que calcular la diferencia para saber si lo tengo que echar a izq o der
      ld e,OBJ_POSX(iy)
      ld d,OBJ_POSX+1(iy)
      
      ld l,OBJ_POSX(ix)
      ld h,OBJ_POSX+1(ix)

      xor a
      sbc hl,de
      jr Z,gtaColisionPersonaFuncs4        //si es exactamente 0 no lo empujo hacia ningun lado

      bit 7,h
      ld hl,#0xFF80        //lo voy a empujar por defecto a la izquierda y segun la diferencia veo si pa un lao u otro
      jr NZ,gtaColisionPersonaFuncs2
      ld hl,#128

   gtaColisionPersonaFuncs2:

      ld e,OBJ_POSX(ix)
      ld d,OBJ_POSX+1(ix)

      push de

      add hl,de      //se lo sumamos a la posicion actual para que vaya modificandose

      ld OBJ_POSX(ix),l
      ld OBJ_POSX+1(ix),h


      call _checkTilePerson
      ld a,#64
      cp l
      pop de
      jr NC,gtaColisionPersonaFuncs4     //si no hay carry es que el tile L<65 y por lo tanto puedo seguir cayendo en esa dirección (aceras, carreteras y tiles sin colision), sino tengo que ver donde voy

      //si entra aqui es que al moverme me he quedado en una zona con colision, asi que la tengo que restaurar
      ld OBJ_POSX(ix),e
      ld OBJ_POSX+1(ix),d



   gtaColisionPersonaFuncs4:
      //ahora voy a hacer lo mismo pero para el eje Y
      //como los dos son personas y tienen el mismo tamaño, solo tengo que calcular la diferencia para saber si lo tengo que echar a izq o der
      ld e,OBJ_POSY(iy)
      ld d,OBJ_POSY+1(iy)
      
      ld l,OBJ_POSY(ix)
      ld h,OBJ_POSY+1(ix)

      xor a
      sbc hl,de
      jr Z,gtaColisionPersonaFuncs8        //si es exactamente 0 no lo empujo hacia ningun lado

      bit 7,h
      ld hl,#0xFF80        //lo voy a empujar por defecto a la izquierda y segun la diferencia veo si pa un lao u otro
      jr NZ,gtaColisionPersonaFuncs6
      ld hl,#128

   gtaColisionPersonaFuncs6:

      ld e,OBJ_POSY(ix)
      ld d,OBJ_POSY+1(ix)

      push de

      add hl,de      //se lo sumamos a la posicion actual para que vaya modificandose

      ld OBJ_POSY(ix),l
      ld OBJ_POSY+1(ix),h

      call _checkTilePerson
      ld a,#64
      cp l
      pop de
      jr NC,gtaColisionPersonaFuncs8     //si no hay carry es que el tile L<65 y por lo tanto puedo seguir cayendo en esa dirección (aceras, carreteras y tiles sin colision), sino tengo que ver donde voy

      //si entra aqui es que al moverme me he quedado en una zona con colision, asi que la tengo que restaurar
      ld OBJ_POSY(ix),e
      ld OBJ_POSY+1(ix),d

   
   gtaColisionPersonaFuncs8:
      //call _vtBorderBlack
      ret
      //antes de irnos, miro si la persona esta un poco cabreada con nosotros o muy cabreada
      /*call _debug1
      ld a,OBJ_ID_NPC_CAR(ix)
      or a
      ret Z    //si es cero no tengo que hacer nada más
      cp #8
      ret NC      //si es mayor que 8 tampoco tengo que hacer nada más

      //si esta entre 1 y 8 es un NPC, ahora tengo que ver si IX es B500 (el prota) o no
      ld a,#0xB5
      .dw #0x94DD    //sub ixh
      ret NZ      //si no es 0 IX no es el prota, asi que pa fuera

      xor a 
      .dw #0x95DD    //sub ixl
      ret NZ      //si no es 0 IX no es el prota, asi que pa fuera

      //si llego aqui IX es el prota, vuelvo a ver el hatemeter concreto del NPC
      ld e,OBJ_ID_NPC_CAR(ix)
      ld d,#0
      ld hl,#_hateLevel+1
      add hl,de
      ld a,(hl)
      cp #1
      ret C    //si hay carry IY no esta enfadado, no tenemos que hacer nada

      //si llego aqui IY esta medio enfadado y con el empujon se acaba de enfadar del todo, va a atacar al prota
      ld hl,#_iaPersonAtacandoProta
	   ld OBJ_MACHINE_STATE_PTR(iy),l
    	ld OBJ_MACHINE_STATE_PTR+1(iy),h
		ld OBJ_FRAMES_FOR_NEXT_STATE(iy),#127  */

      //ret      //y nos vamos






   gtaColisionPersonaFuncPersonaCoche:

      ld a,OBJ_PARENT_PTR+1(ix)
      or a
      ret NZ      //si voy montao en un coche no me pueden atropellar

      ld a,OBJ_CAR_SPEED(iy)
      cp #VEL_MIN_TO_ATROPELLO
      //si no va más rapido que la velocidad minima para atropellar en ese caso no atropella solo empuja al muñeco hacia fuera como si se hubiese chocado con otra persona
      jp C,gtaColisionPersonaFuncPersonaPersonaEmpujon
      //ret C    //si no va más rapido que la velocidad minima para atropellar en ese caso no atropella

      //si la persona ya esta cayendose no la atropellamos más (no hay que abusar)
      ld de,#_animWomanFall
      bit 1,OBJ_ACTIVE_TYPE(ix)           //compruebo si es hombre/mujer
      jr Z,gtaColisionPersonaFuncPersonaCoches2 
      ld de,#_animManFall

   gtaColisionPersonaFuncPersonaCoches2:
      ld h,OBJ_ANIMATION_LIST+1(ix)
      ld l,OBJ_ANIMATION_LIST(ix)
      xor a
      sbc hl,de
      ret Z       //si ya nos estamos cayendo en el suelo no lo atropellamos más, nos vamos directamente


      ld a,OBJ_LIFE(ix)
      sub #RESTA_VIDA_EN_ATROPELLO
      ld OBJ_LIFE(ix),a


      //voy a hacer una prueba a atropellar a alguien y expulsarlo lejos (si es IA, tengo que cambiarle la funcion de IA a KABESA)

      ld e,OBJ_POSX(iy)
      ld d,OBJ_POSX+1(iy)
      ld l,OBJ_POSX(ix)
      ld h,OBJ_POSX+1(ix)
      xor a 
      sbc hl,de
      add hl,hl
      add hl,hl
      ld b,h
      ld c,l

      ld e,OBJ_POSY(iy)
      ld d,OBJ_POSY+1(iy)
      ld l,OBJ_POSY(ix)
      ld h,OBJ_POSY+1(ix)
      xor a 
      sbc hl,de
      add hl,hl
      add hl,hl
      
      
      
      ld OBJ_SPEEDX(ix),c
      ld OBJ_SPEEDX+1(ix),b
      ld OBJ_SPEEDY(ix),l
      ld OBJ_SPEEDY+1(ix),h

      ld l,OBJ_CAR_SPEED(iy)
      ld h,#0
      add hl,hl
      //add hl,hl    con esto lo manda demasiado lejos, mejor dejar un x2 solo
      

      call _vtLimitSpeed

      

      ld a,OBJ_MACHINE_STATE_PTR+1(ix)
      or a
      jr NZ,gtaColisionPersonaFuncPersonaCocheIA
      
      //si OBJ_MACHINE_STATE_PTR es cero es que soy el prota, aqui hay que ajustar alguna cosilla
      ld a,#NUM_FRAMES_KO_TRAS_ATROPELLO
      ld OBJ_IA_LOCK_MOVE(ix),a
      call _gtaSetAnimFallIX

      jr gtaColisionPersonaFuncPersonaCocheExit


   gtaColisionPersonaFuncPersonaCocheIA:
      //soy una ia y me empuja el coche
      ld a,#15
      ld OBJ_FRAMES_FOR_NEXT_STATE(ix),a
      ld hl,#_iaPersonEmpujada     //si llega a cero de momento para probar voy a volver a ponerlo parado
      ld OBJ_MACHINE_STATE_PTR(ix),l
      ld OBJ_MACHINE_STATE_PTR+1(ix),h  


      //antes de irnos, miramos si el prota es el conductor del coche, en ese caso subimos 1 nivel de busqueda de la policia hatelevel[0]
      ld l,OBJ_PARENT_PTR(iy)
      ld h,OBJ_PARENT_PTR+1(iy)
      ld de,#GAME_OBJECTS_PTR    //cargo en DE el puntero al prota y comparo con el conductor del coche que ha atropellado a esta persona
      xor a 
      sbc hl,de
      jr NZ,gtaColisionPersonaFuncPersonaCocheExit    //si no es cero, ya podemos salir porque el coche lo lleva otro de la IA

      //si es cero, el prota es el conductor, subimos en uno el hateLevel[0]
      ld hl,#_hateLevel+1
      inc (hl)


   gtaColisionPersonaFuncPersonaCocheExit:
      ld a,#FX_GOLPE //golpe
      call _gtaSoundFX
      ret
   __endasm;
}






void gtaItemStepFunc() __naked {
   __asm

      call _vtStepObjectFunc


      ld	ix,(_vtGameObjPtr)
      //call _trapIX

      ld OBJ_POSX(ix),#0
      ld e,OBJ_SELECTED_WEAPON(ix)     //voy a comprobar si tengo que desactivar el item o no (si ya lo recogi antes)
      ld d,#0
      //ld hl,#_itemYaRecogido
      ld hl,#ITEM_YA_RECOGIDO
      add hl,de
      xor a
      or (hl)
      ret Z

      //si entro aqui tengo que desactivar el draw y la colision
      ld OBJ_FLAGS(ix),#FLAG_NO_COLISION     //con esto es suficiente, ya que desactivo todo y eso hara que no se dibuje y ademas no colisione
      xor a
      ld OBJ_PREPARE_FUNC(ix),a
      ld OBJ_PREPARE_FUNC+1(ix),a

      ret
   __endasm;
}


void gtaItemColisionFunc() __naked {
   __asm
      //miro si IY es B500 o no
      push iy
      pop hl
      ld de,#GAME_OBJECTS_PTR
      xor a
      sbc hl,de
      //si esto no da cero, no se pilla el item, pa fuera del tiron
      ret NZ

      //si da cero es que es el jugador el que ha pillao el item, podemos cambiarle el arma al jugador y autodestruir el item
      ld a,OBJ_SELECTED_WEAPON(ix)
      
      cp #9
      jp NC,gtaItemColisionFuncItemNoWeapon
      
      
      ld OBJ_SELECTED_WEAPON(iy),a

      ld OBJ_AMMO(iy),#50    //si es cualquier otra cosa le ponemos 255 de municion para que no se gaste nunca (o casi)


      cp #WEAPON_PISTOLA    //comprobamos si es pistola
      jr NZ,gtaItemColisionFuncs2
      ld OBJ_AMMO(iy),#50    //es pistola, le doy municion
      jr gtaItemColisionFuncs10
   gtaItemColisionFuncs2:
      cp #WEAPON_SHOTGUN    //comprobamos si es escopeta
      jr NZ,gtaItemColisionFuncs4
      ld OBJ_AMMO(iy),#40    //es escopeta, le doy municion
      jr gtaItemColisionFuncs10
   gtaItemColisionFuncs4:
      cp #WEAPON_GRANADA    //comprobamos si es granada
      jr NZ,gtaItemColisionFuncs6
      ld OBJ_AMMO(iy),#15    //es granada, le doy municion
      jr gtaItemColisionFuncs10
   gtaItemColisionFuncs6:
      cp #WEAPON_BAZOKA    //comprobamos si es bazoka
      jr NZ,gtaItemColisionFuncs10
      ld OBJ_AMMO(iy),#10    //es bazoka, le doy municion
      


      


   gtaItemColisionFuncs10:
      
      ld e,OBJ_SELECTED_WEAPON(ix)     //voy a comprobar si tengo que desactivar el item o no (si ya lo recogi antes)
      ld d,#0
      //ld hl,#_itemYaRecogido
      ld hl,#ITEM_YA_RECOGIDO
      add hl,de
      ld (hl),#1


      ld a,OBJ_SELECTED_WEAPON(ix)
      dec a
      jr NZ,gtaItemColisionBate
   gtaItemColisionNavaja:
      ld hl,#_frase0          //navaja
      jr gtaItemColisionFrase
   gtaItemColisionBate:
      dec a 
      jr NZ,gtaItemColisionNunchuck
      ld hl,#_frase1          //bate
      jr gtaItemColisionFrase
   gtaItemColisionNunchuck:
      dec a 
      jr NZ,gtaItemColisionKatana
      ld hl,#_frase2          //nunchuck
      jr gtaItemColisionFrase
   gtaItemColisionKatana:
      dec a 
      jr NZ,gtaItemColisionGun
      ld hl,#_frase3          //katana
      jr gtaItemColisionFrase
   gtaItemColisionGun:
      dec a 
      jr NZ,gtaItemColisionShotgun
      ld hl,#_frase4       //gun
      jr gtaItemColisionFrase
   gtaItemColisionShotgun:
      dec a 
      jr NZ,gtaItemColisionGranada
      ld hl,#_frase5       //shotgun
      jr gtaItemColisionFrase
   gtaItemColisionGranada:
      dec a 
      jr NZ,gtaItemColisionBazoka
      ld hl,#_frase6    //granada
      jr gtaItemColisionFrase
   gtaItemColisionBazoka:
      ld hl,#_frase7       //bazoka
      

   gtaItemColisionFrase:
      ld (#_talkInstancePtr),iy
      //xor a 
      //ld (#_talkTime),a
      
      call _gtaCreaFrase

      //ld OBJ_ACTIVE_TYPE(ix),#0     //autodestruimos el item

      ret
      
    gtaItemColisionFuncItemNoWeapon:
    	ld OBJ_ACTIVE_TYPE(ix),#0     //autodestruimos el item y ya está, sin más, el mision controller recorrerá los objetos buscando este item y cuando no lo encuentre entenderá que ya lo ha pillado
    	ret
   __endasm;
}


//recuerda en IX esta el principal y en IY el otro que ha colisionado, revisa el tipo del otro antes de hacer nada
void gtaCocheColisionFunc() __naked {
   __asm
      
      //miro si IY es una persona o un coche
      ld a,OBJ_ACTIVE_TYPE(iy)
      cp #TYPE_CAR       //miramos si es una persona o no
      jr NZ,gtaCocheColisionFuncColCochePersona        //es mayor que 8 asi que sera un coche

   gtaCocheColisionFuncColCocheCoche:
      //si entro aqui hay colision coche contra coche (tengo que ver desde que angulos estan chocando y empujarse mutuamente)
      //aqui pueden pasar varias cosas, tengo que ver porque lado estoy colisionando para aplicar una fisicas (medio realistas).
      //por simplificar simplemente voy a hacer que se repulsen uno del otro, o lo que es lo mismo que se aplique una fuera en la 
      // dirección contraria a la que esta el otro (con la fuerza que vaya el otro).
      ld h,OBJ_POSX+1(ix)
      ld l,OBJ_POSX(ix)
      ld d,OBJ_POSX+1(iy)
      ld e,OBJ_POSX(iy)
      xor a
      sbc hl,de
      ld b,h
      ld c,l         //diferencia X=BC

      ld h,OBJ_POSY+1(ix)
      ld l,OBJ_POSY(ix)
      ld d,OBJ_POSY+1(iy)
      ld e,OBJ_POSY(iy)
      xor a
      sbc hl,de
      ex de,hl       //diferencia Y=DE

      
      //la entrada es X=BC e Y=DE, y en A devuelve el angulo (256 based)
      call _vtAtan2_16     //pillo angulo en A
      ld c,#1      //pillo interpolacion (le doy poca importancia a la velocidad que lleve y toda la importancia a la velocidad del pepinazo)
      ld b,OBJ_CAR_SPEED(iy)     //pillo velocidad

      call _vtAccelToSpeedAng


      //despues del piñote, sumo las dos velocidades y resto vida a los coches (por hacerlo rapido, sumo las velocidades y resto la vida)
      ld a,OBJ_CAR_SPEED(ix)
      add OBJ_CAR_SPEED(iy)
      srl a    //divido entre 2 porke sino va a ser mucho creo yo
      srl a    //divido entre 2 porke sino va a ser mucho creo yo
      srl a    //divido entre 2 porke sino va a ser mucho creo yo
      srl a    //divido entre 2 porke sino va a ser mucho creo yo
      ld b,a
      ld a,OBJ_LIFE(ix)
      sub b
      ld OBJ_LIFE(ix),a
      set 3,OBJ_FLAGS(ix)     //le activamos el flag de flash sprite para dar feedback durante 1 frame

      ret


   gtaCocheColisionFuncColCochePersona:
      //si entro aqui hay colision coche contra persona u otro objeto 
      cp #TYPE_CITIZEN
      ret NZ      //si no es un ciudadano en principio no freno nada

      //(si es una persona esta controlado en la funcion de colision de persona, asi que aqui no tengo que hacer nada, si acaso frenar un poquito el coche)
      //sea lo que sea con lo que me estoy dando mejor freno
      ld a,OBJ_CAR_SPEED(ix)
      cp #32
      ret C    //si es menor de 32 no reduzco la velocidad en la colision con la persona

      srl a    //divido la velocidad entre 2 (número entero)
      ld OBJ_CAR_SPEED(ix),a     //y ya está, el resto de la colisión lo hará la funcion de colision con persona

      ret
   __endasm;
}



void gtaDisparoColisionFunc() __naked {
   __asm
      //antes de nada nos aseguramos que es una persona o un coche
      push iy
      pop hl
      call _gtaEsPersonaHL

      or a
      jr Z,gtaDisparoColisionFuncCheckCar   //si no es una persona no hago nada que siga palante el disparo


		//aqui tengo que ver si es un disparo normal, una granada, o un bazoka
		//si es un disparo normal, quitamos vida al otro y eliminamos IX
		//si es una granada, hacemos un rebote random
		//si es un bazoka le bajamos el tiempo a 1 para que explote

		ld a,OBJ_SELECTED_WEAPON(ix)
		cp #8
		jr NZ,gtaDisparoColisionFuncPersons2		
		
		call explotaObjeto		//esto transforma el objeto en una explosion y vuelve
		ret
		
	gtaDisparoColisionFuncPersons2:
		cp #7
		jr NZ,gtaDisparoColisionFuncPersons3
		
		//hago rebotar la granada random
		call cpct_getRandom_lcg_u8_asm			//pongo un numero random en A
		ld c,#1						//interpolacion a 1 (no interpolacion)
      ld b,#64						//velocidad 64 ?
		call _vtAccelToSpeedAng				//llamo a vtAccelToSpeedAng	para que cambie la velocidad el objeto
		
		ret
		
	gtaDisparoColisionFuncPersons3:
		//disparo normal 
		
      //le restamos vida al IY
      ld a,OBJ_LIFE(iy)
      sub OBJ_WEAPON_DAMAGE(ix)     //le resto el daño que haga esta bala
      ld OBJ_LIFE(iy),a
      bit 7,a
      jr Z,gtaDisparoColisionFuncPersons3b

      //si llegamos a la vida 0 es que lo hemos matado, subimos el nivel de busqueda de la poli (no hace falta mirar quien ha matao a quien, porke si nos matan a nosotros el juego se acaba de todas formas)
      ld hl,#_hateLevel+1
      inc (hl)

   gtaDisparoColisionFuncPersons3b:
      //creamos feedback del golpe
      //call _gtaCreaHit
      set 3,OBJ_FLAGS(iy)     //le activamos el flag de flash sprite para dar feedback durante 1 frame

      //intentamos crear sangre
      ld a,OBJ_WEAPON_DAMAGE(ix)
      srl a    //divido entre 2  para que no salga demasiada sangre
      srl a    //divido entre 2  para que no salga demasiada sangre
      ld (#gtaCreaBalaBlood),a      //por simplificar creo tanta sangre como vida quite...
      call gtaCreaSangre   //esto crea la sangre y vuelve (en ix va el arma, y en iy va el personaje)

      //para ver si es el prota o la IA me vale simplemente con mirar si iyl es cero, si es cero es el prota
      xor a
      .dw #0xBDFD       //cp iyl opcode
      jr Z,gtaDisparoColisionFuncProta




   gtaDisparoColisionFuncIA:
      //si es de la IA le pusheamos como si lo hubieses atropellado (más o menos) y le cambiamos la ani de pusheo
      //KABESA falta hacer esto
      jp gtaDisparoColisionFuncDestroy




   gtaDisparoColisionFuncProta:
      //si es el prota solo la paramos y le quitamos vida 
      //KABESA falta hacer esto
      jp gtaDisparoColisionFuncDestroy





   gtaDisparoColisionFuncCheckCar:
      //push iy
      //pop hl
      //call _gtaEsCocheHL
      bit 5,OBJ_ACTIVE_TYPE(iy)
      //or a
      jr Z,gtaDisparoColisionFuncNoDestroy       //tampoco es un coche, asi que tire palante el disparo sin hacer nada


		//aqui tengo que ver si es un disparo normal, una granada, o un bazoka
		//si es un disparo normal, quitamos vida al otro y eliminamos IX
		//si es una granada, hacemos un rebote random
		//si es un bazoka le bajamos el tiempo a 1 para que explote

		ld a,OBJ_SELECTED_WEAPON(ix)
		cp #8
		jr NZ,gtaDisparoColisionFuncCoches2
		
		call explotaObjeto		//esto transforma el objeto en una explosion y vuelve
		ret
		
	gtaDisparoColisionFuncCoches2:
		cp #7
		jr NZ,gtaDisparoColisionFuncCoches3
		
		//hago rebotar la granada random
		call cpct_getRandom_lcg_u8_asm			//pongo un numero random en A
		ld c,#1						//interpolacion a 1 (no interpolacion)
      ld b,#64						//velocidad 64 ?
		call _vtAccelToSpeedAng				//llamo a vtAccelToSpeedAng	para que cambie la velocidad el objeto
		ret
		
	gtaDisparoColisionFuncCoches3:

      //si llega aqui es la colision con un coche, le quitamos la vida y ya esta
      ld a,OBJ_LIFE(iy)
      sub OBJ_WEAPON_DAMAGE(ix)     //le resto el daño que haga esta bala
      ld OBJ_LIFE(iy),a
      bit 7,a
      jr Z,gtaDisparoColisionFuncCoches3b

      //si llegamos a la vida 0 hemos reventao el coche, subimos el nivel de busqueda de la poli 
      ld hl,#_hateLevel+1
      inc (hl)

   gtaDisparoColisionFuncCoches3b:
      //call _gtaCreaHit
      set 3,OBJ_FLAGS(iy)     //le activamos el flag de flash sprite para dar feedback durante 1 frame

      jp gtaDisparoColisionFuncDestroy




   gtaDisparoColisionFuncDestroy:
      ld OBJ_ACTIVE_TYPE(ix),#0
      ret



   gtaDisparoColisionFuncNoDestroy:
      ret

   __endasm;
}


//crea un feedback de golpeo (para que el jugador se de cuenta de que le ha dado a la persona/coche)
//lo crea en la misma posicion que tenga el IY actual (que suele ser la persona o el coche al que se le esta metiendo la ostia)
/*void gtaCreaHit() __naked {
   __asm
      push ix
      push de
      push hl

      ld e,OBJ_POSX+1(iy)
      ld d,OBJ_POSY+1(iy)
      push de

      call _vtGetFreeGameObjPtr        //preparamos el objeto del prota
      ld ix,(#_vtGameObjPtr)
      
      ld OBJ_ACTIVE_TYPE(ix),#TYPE_AUX_OBJECT

      ld hl,#_gtaAnimationStepFunc
      ld OBJ_STEP_FUNC(ix),l
      ld OBJ_STEP_FUNC+1(ix),h

      ld hl,#_vtDrawObjectFunc
      ld OBJ_PREPARE_FUNC(ix),l
      ld OBJ_PREPARE_FUNC+1(ix),h

      ld hl,#_animHit
      ld OBJ_ANIMATION_LIST(ix),l
      ld OBJ_ANIMATION_LIST+1(ix),h

      ld OBJ_ANIMATION_MAX(ix),#6
      ld OBJ_FRAME_SPEED(ix),#255

      ld OBJ_FLAGS(ix),#FLAG_NO_COLISION|FLAG_FORCE_DRAW|FLAG_AUTO_ANIM

      
      pop de

      ld OBJ_POSX+1(ix),e
      dec d  //lo pongo un poco más abajo para que se renderice por arriba y se vea (sino el muñeco lo tapará)
      ld OBJ_POSY+1(ix),d

      pop hl
      pop de
      pop ix
      ret
   __endasm;
}*/


void gtaTargetStepFunc() __naked {
   __asm
      ld ix,(#_vtGameObjPtr)

      xor a                            //en principio pongo que no voy a dibujar ningun sprite
      ld OBJ_PREPARE_FUNC(ix),a
      ld OBJ_PREPARE_FUNC+1(ix),a   

      ld a,(#_misionTargetTileX)       //miro si hay algun target o no
      or a 
      jr Z,gtaTargetStepFuncs4

      //si no es cero hay target, lo primero recupero el drawfunc porque voy a dibujar
      ld hl,#_vtDrawObjectFunc
      ld OBJ_PREPARE_FUNC(ix),l
      ld OBJ_PREPARE_FUNC+1(ix),h

      ld OBJ_POSX+1(ix),a        //lo segundo ajusto la posicion del target a el tile exacto donde debo ir
      ld a,(#_misionTargetTileY)
      ld OBJ_POSY+1(ix),a
      
      ld iy,#GAME_OBJECTS_PTR          //ahora calculo el angulo entre el target y el prota
      call _gtaDistanceIX_IY     //esto me deja en HL la distancia
      ld a,#8
      cp h
      jr C,gtaTargetStepFuncs2

      //si entra aqui estoy ya muy cerca, lo situo directamente en el sitio para que se vea el punto exacto
      ld a,(#_misionTargetTileX)       //miro si hay algun target o no
      ld OBJ_POSX+1(ix),a
      ld a,(#_misionTargetTileY)       //miro si hay algun target o no
      ld OBJ_POSY+1(ix),a
      xor a
      ld OBJ_POSX(ix),a
      ld OBJ_POSY(ix),a
      jr gtaTargetStepFuncs4     //y salimos





   gtaTargetStepFuncs2:
      call _vtAngleIXtoIY
      add #128                         //le doy la vuelta ya que lo he calculado al reves
      ld c,a                           //lo meto en C

      ld b,#8
      push bc
      ld a,c
      
      call _vtCosAdvance      //A pasamos el angulo (256 based), en B pasamos la velocidad, y devuelve en DE=cos(A)*B
      ld a,e
      add OBJ_POSX+1(iy)              //ajusto la posicion del target a la misma que tiene el prota
      ld OBJ_POSX+1(ix),a

      ld a,OBJ_POSX(iy)
      ld OBJ_POSX(ix),a

      pop bc

      ld a,c
      call _vtSinAdvance      //A pasamos el angulo (256 based), en B pasamos la velocidad, y devuelve en DE=sin(A)*B
      ld a,e

      add OBJ_POSY+1(iy)
      inc a       //le sumo uno para que se ajuste a los pies mejor
      ld OBJ_POSY+1(ix),a

      ld a,OBJ_POSY(iy)
      ld OBJ_POSY(ix),a

   gtaTargetStepFuncs4:
      call _vtStepObjectFunc
   
      ret
   __endasm;
}


void gtaAnimationStepFunc() __naked {
   __asm 

      call _vtStepObjectFunc
      //call _trapIX
      ld ix,(#_vtGameObjPtr)
      //call _trapIX
      ld a,OBJ_ANIMATION_MAX(ix)
      dec a
      sub OBJ_FRAME_INDEX+1(ix)
      ret NZ

      ld OBJ_ACTIVE_TYPE(ix),#0  //ya ha acabado la animacion, nos lo cargamos
      ret
   __endasm;
}



//devuelve si el objeto HL es una persona (ya sea ciudadano o el prota)
void gtaEsPersonaHL() __naked {
   __asm
      ld a,(hl)
      bit 2,a
      ld a,#1
      ret NZ    //si el bit 2 es 1, devolvemos 1, es un ciudadano por lo tanto una persona

      //si llega aqui puede ser el prota, o no ser una persona (para que sea el prota debe ser igual que el GAME_OBJECTS_PTR)
      ld de,#GAME_OBJECTS_PTR
      xor a
      sbc hl,de
      ret NZ    //si no es cero, es que no es el prota y tampoco es un ciudadano, por lo tanto devolvemos cero (del xor a)

      ld a,#1     //si llega aqui es el prota, devolvemos 1 porque tambien es una persona
      ret
   __endasm;
}

/*
//devuelve si el objeto HL es un coche
void gtaEsCocheHL() __naked {
   __asm
      ld a,(hl)
      bit 5,a
      ld a,#0
      ret Z    //si el bit 5 es cero, devolvemos 0, no es un coche
      inc a    //sino le incremento uno a A para devolver 1 porque si es un coche
      ret
   __endasm;
}
*/

void gtaCocheStepFunc() __naked {
   __asm
      //TODO aqui debe calcular para donde debe mirar el coche
      ld ix,(#_vtGameObjPtr)


      //COMPROBAMOS SI ESTE COCHE SE DEBE MOVER SOLO CON IA O NO (si OBJ_PARENT_PTR = 0xFFFF entonces auto IA)
      push ix
      ld a,OBJ_PARENT_PTR+1(ix)
      inc a
      call Z, _gtaCocheIA    //si sumandole uno es 0 es que era FF y por lo tanto tengo que llamar a la IA
      pop ix


      ld a,OBJ_ID_NPC_CAR(ix)
      inc a
      jr NZ,gtaCocheStepFuncs2

      //si entra aqui me tengo que cargar el NPC ya que se ha solicitado la eliminacion desde gtaMisionDestroyNPC
      ld OBJ_ACTIVE_TYPE(ix),a      //como ya he llamado al step generico y se ha redibujado el fondo, solo tengo que poner a 0 el tipo para que no llame al draw y listo

   gtaCocheStepFuncs2:

      //MUEVE EL COCHE----------------------------------------     
      ld d,#0
      ld e,OBJ_SELECTED_CAR(ix)
      ld hl,#_carPropAgarre
      add hl,de
      ld c,(hl)
      
      ld b,OBJ_CAR_SPEED(ix)

      ld a,b               //comprobamos si va muy lento o no
      cp #MIN_SPEED_TO_MOVE_CAR
      jr NC,gtaCocheStepFuncMueveCoche
      ld b,#0        //si la velocidad es menor que MIN_SPEED_TO_MOVE_CAR va muy lento, le ponemos como si fuese a velocidad 0 para que se vaya parando poco a poco del todo

   gtaCocheStepFuncMueveCoche:
      ld a,OBJ_CAR_TORQUE(ix)
      call _vtAccelToSpeedAng



      //para ver si puedo driftear el coche tiene que estar dibujandose sino nada
      bit 2,OBJ_FLAGS(ix)
      jp Z,gtaCocheStepFuncs3

      ld a,OBJ_CAR_SPEED(ix)        //y tiene que ir a un minimo de velocidad sino tampoco hace drift
      cp #MIN_SPEED_TO_DRIFT
      jp C,gtaCocheStepFuncs3
     

      //si entro aqui voy rapidito, voy a ver si tengo que derrapar o no
      //voy a calcular el angulo en el que me estoy moviendo y el angulo visible del coche,
      //si hay mucha diferencia tenemos que derrapar

      
      ld b,OBJ_SPEEDX+1(ix)
      ld c,OBJ_SPEEDX(ix)
      ld d,OBJ_SPEEDY+1(ix)
      ld e,OBJ_SPEEDY(ix)
      call _vtAtan2_16     //la entrada es X=BC e Y=DE, y en A devuelve el angulo (256 based)

      ld b,OBJ_CAR_TORQUE(ix)
      sub b
      bit 7,a
      jr Z,gtaCocheStepFuncMueveCoches1aa
      neg

   gtaCocheStepFuncMueveCoches1aa:
      cp #MIN_ANGLE_DIFF_TO_DRIFT
      jr C,gtaCocheStepFuncs3

      //si entro aqui estoy derrapando, deberia pintar algo
      
      ld a,OBJ_FRAME_INDEX+1(ix)
      add a,a     //x2  
      add a,a     //x4
      add a,a     //x8

      ld hl,#_desplazamientoXYDrift
      ld e,a
      ld d,#0
      add hl,de
      
      ld c,(hl)
      inc hl 
      ld b,(hl)
      inc hl 
      ld e,(hl)
      inc hl 
      ld d,(hl)
      inc hl 

      push hl

      ld h,OBJ_POSX+1(ix)
      ld l,OBJ_POSX(ix)
      add hl,bc
      ld b,h
      ld c,l

      ld h,OBJ_POSY+1(ix)
      ld l,OBJ_POSY(ix)
      add hl,de
      ex de,hl
      
      call _vtGetGameScreenPtrBCDE        //esto me calcula la direccion de memoria de pantalla    
   
		ld (#gtaCocheStepDrift1),hl
      //ld (hl),#0     //pinto el derrape NO LO PINTO AUN, se pinta en el predraw

      pop hl


      ld c,(hl)
      inc hl 
      ld b,(hl)
      inc hl 
      ld e,(hl)
      inc hl 
      ld d,(hl)
      inc hl 

      ld h,OBJ_POSX+1(ix)
      ld l,OBJ_POSX(ix)
      add hl,bc
      ld b,h
      ld c,l

      ld h,OBJ_POSY+1(ix)
      ld l,OBJ_POSY(ix)
      add hl,de
      ex de,hl
      
      call _vtGetGameScreenPtrBCDE        //esto me calcula la direccion de memoria de pantalla    
	   
	   
	   
	  ex de,hl			//dejo el puntero a la pantalla en DE 
		
		
		//ahora voy a ir recorriendo el sitio de los predraw pixels 0xBF00-BF50 para guardar el pixel (son 3 bytes, 1 para el numero de frames que se tiene que pintar, y 2 para la posicion de la pantalla)
		ld hl,#PREDRAW_PIXELS_ZONE		
		ld b,#20
	gtaCocheStepFuncMueveCoches1aaBucleSavePixels:
		xor a
		or (hl)
		jr Z,gtaCocheStepFuncMueveCoches1aaBucleSavePixels2		//si es cero puedo escribirlo aqui
		inc hl 													//sino sumo 3 para pasar al siguiente
		inc hl 
		inc hl
		
		djnz gtaCocheStepFuncMueveCoches1aaBucleSavePixels
		
	gtaCocheStepFuncMueveCoches1aaBucleSavePixels2:
		ld (hl),#5							//5 frames se mantiene el pixel y luego escribo la posicion de la pantalla
		inc hl 
		ld (hl),e
		inc hl 
		ld (hl),d
		inc hl 
		

		ld de,(#gtaCocheStepDrift1)					//y ahora cargo en DE la posicion del primer pixel y sigo con el bucle por donde iba
	gtaCocheStepFuncMueveCoches1aaBucleSavePixels3:
		xor a
		or (hl)
		jr Z,gtaCocheStepFuncMueveCoches1aaBucleSavePixels4
		inc hl 
		inc hl 
		inc hl
		
		djnz gtaCocheStepFuncMueveCoches1aaBucleSavePixels3
	gtaCocheStepFuncMueveCoches1aaBucleSavePixels4:
		ld (hl),#5									//escribo el primer pixel y seguimos
		inc hl 
		ld (hl),e
		inc hl 
		ld (hl),d


   gtaCocheStepFuncs3:
      //comprobamos colisiones con el escenario (esto ajusta la posicion en caso de haber colision)
      push ix
      call _vtStepObjectFunc        //hace el step default (que redibuja los tiles de fondo y mueve en funcion del speed del objeto)
      pop ix
      
      push ix
      call _gtaCocheCheckColisionScene
      pop ix
      

	//antes de salir miro si tengo que petar el coche o no...
		ld a,OBJ_LIFE(ix)
		cp #200
		ret C			//si hay carry aun esta vivo puedo pirarme
		

		//importante, KABESA aqui al petar el coche tendria que matar a todos los que esten dentro, tal cual...
		ld hl,#GAME_OBJECTS_PTR
      ld de,#48
      ld b,#16
	gtaCocheStepExplotaPasajerosBucle:
      ld a,(hl)
      or a
      jr Z,gtaCocheStepExplotaPasajerosBucleFin

      push hl 
      pop iy
      ld a,OBJ_PARENT_PTR+1(iy)
      or a
      jr Z,gtaCocheStepExplotaPasajerosBucleFin

      //si entra aqui hay parent, tengo que ver si el parent de IY y IX son iguales
      .dw #0x94DD    //sub ixh
      jr NZ,gtaCocheStepExplotaPasajerosBucleFin

      ld a,OBJ_PARENT_PTR(iy)
      .dw #0x95DD    //sub ixl
      jr NZ,gtaCocheStepExplotaPasajerosBucleFin

      //si llego aqui IY va dentro de IX, tengo que ponerle la vida a -1
      ld OBJ_LIFE(iy),#255

   gtaCocheStepExplotaPasajerosBucleFin:
      add hl,de
      djnz gtaCocheStepExplotaPasajerosBucle


		
	explotaObjeto:
		//si no hay carry es que ya ha llegado a 0 y se ha pasado (osea menos algo...)
		//le cambio el tipo al coche y le pongo como un objeto de tipo animacion
      ld OBJ_ACTIVE_TYPE(ix),#TYPE_AUX_OBJECT		//hago todos los cambios en la entidad para transformarla en una animacion simple

		ld hl,#_animExplosion3t
		ld OBJ_ANIMATION_LIST(ix),l			//pongo la ani de caerse
      ld OBJ_ANIMATION_LIST+1(ix),h

		xor a						//comienzo la animacion
		ld OBJ_FRAME_INDEX(ix),a
		ld OBJ_FRAME_INDEX+1(ix),a		

      ld hl,#_gtaAnimationStepFunc
      ld OBJ_STEP_FUNC(ix),l
      ld OBJ_STEP_FUNC+1(ix),h

      ld hl,#_vtDrawObjectFunc
      ld OBJ_PREPARE_FUNC(ix),l
      ld OBJ_PREPARE_FUNC+1(ix),h

		ld OBJ_WIDTH(ix),#3
		ld OBJ_HEIGHT(ix),#3

      ld OBJ_ANIMATION_MAX(ix),#5
      ld OBJ_FRAME_SPEED(ix),#32

      ld OBJ_FLAGS(ix),#FLAG_NO_COLISION|FLAG_AUTO_ANIM


   explotaObjetoDano:
      //hago pupita a los que esten cerca
      ld hl,#GAME_OBJECTS_PTR
      ld de,#48
      ld b,#16

   explotaObjetoBucleDano:
      ld a,(hl)
      and #TYPE_PROTA|TYPE_CAR|TYPE_CITIZEN
      jr Z,explotaObjetoBucleDanoFin
      
      push hl
      exx
      pop iy

      //ahora que se que es algo que puede tener vida, le miro la distancia
      call _gtaDistanceIX_IY     //esto me deja en HL la distancia
      
      //si esta a menos de 5 tiles le quito vida y le pongo el flag a blanco
      ld a,#DISTANCIA_TILES_EXPLOSION_DAMAGE
      cp h
      jr C,explotaObjetoBucleDanoFinPre      //si hay carry nada, esta muy lejos, tirando

      //si no hay carry IY esta cerquita de la explosion, le quito vida

      ld b,#EXPLOSION_DAMAGE
      ld a,OBJ_LIFE(iy)
      sub b
      ld OBJ_LIFE(iy),a    //esto puede quedar como un numero negativo, en ese caso el propio objeto debe matarse a si mismo

      bit 7,a
      jr Z,explotaObjetoBucleDanos2
      
      //si llegamos a la vida 0 hemos reventao la persona o el coche, subimos el nivel de busqueda de la poli 
      ld hl,#_hateLevel+1
      inc (hl)

   explotaObjetoBucleDanos2:
      //call _gtaCreaHit
      set 3,OBJ_FLAGS(iy)     //le activamos el flag de flash sprite para dar feedback durante 1 frame

      ld a,#4
      ld (#gtaCreaBalaBlood),a      //y vamos con la sangria

      call gtaCreaSangre

   explotaObjetoBucleDanoFinPre:
      exx

   explotaObjetoBucleDanoFin:
      add hl,de
      dec b
      jr NZ,explotaObjetoBucleDano

      ld a,#FRAMES_EXPLOSION_LIGHT
      ld (#_explosionTime),a     //frames en los que ponemos la paleta más clara
      //ya le hemos quitado vida a todo el mundo, puedo salir
      ret  

		//sitio para guardar los dos puntos de la pantalla donde se está drifteando
   gtaCocheStepDrift1:
      .dw #0
   gtaCocheStepDrift2:
      .dw #0
      
   
   __endasm;
}





void gtaCocheDrawFunc() __naked {
   __asm
      ld ix,(#_vtGameObjPtr)
      
      ld a,OBJ_NEED_REDRAW(ix)
      or a
      ret Z             //si es cero nos saltamos el dibujado


      //luces del coche de policia cambiando cada poco...
      ld a,OBJ_SELECTED_CAR(ix)
      cp #2
      jr NZ,gtaCocheDrawFuncs2
      //si entra aqui es el coche de policia, le pongo las luces roja y azul cambiando en el custom color
      ld OBJ_CUSTOM_COLOR(ix),#CUSTOM_A_PAL_9|CUSTOM_B_PAL_12
      ld a,(#_vtClock)
      bit 2,a
      jr NZ,gtaCocheDrawFuncs2
      ld OBJ_CUSTOM_COLOR(ix),#CUSTOM_A_PAL_12|CUSTOM_B_PAL_9



   gtaCocheDrawFuncs2:
      ld a,OBJ_CAR_TORQUE(ix)
      add #16        //ajuste para que el angulo sea correcto el sprite
      and #0b11100000
      rlca
      rlca
      rlca

      //animCarSimple[]={carSimpleAba,carSimpleAbaDer,carSimpleDer,carSimpleArrDer,carSimpleArr}
      ld OBJ_FRAME_INDEX+1(ix),a


      sub #3  //le resto 3 para comprobar si el numero es 3, 4 o 5 para voltearlo o no
      cp #3

      ld a,OBJ_SPEEDX+1(ix)
      push af
      ld a,#192      //lo pongo con un numero negativo para que mire hacia la izq

      jr C,gtaCocheDrawFuncVolteado    //cp #3    ---<<<< viene de aqui

      ld a,#64       //lo pongo con un numero positivo para que mire hacia la der

   gtaCocheDrawFuncVolteado:
      
      ld OBJ_SPEEDX+1(ix),a      //engaño a la funcion modificando la velocidad
      call _vtDrawObjectFunc

      pop af                        //recupero la velocidad que tenia antes de dibujar
      ld OBJ_SPEEDX+1(ix),a

      ret


   __endasm;
}



//calcula la IA del coche (basicamente mira en que metatile y tile estamos y acelera, frena o gira)
//si el metatile es de carretera usa el array metatileTraficDirection para saber donde tiene que ir
//si el metatileTraficDirection devuelve -1 debe ver si puede seguir adelante, sino tiene que mirar si tiene que girar a la izq o a la derecha (donde haya algun metatile de carretera)
//si el metatile NO es de carretera, debe buscar el metatile de carretera más cercano (en arriba, izq, der, abajo) y dirigirse hacia ese lugar (puede ir al centro no pasa nada)
void gtaCocheIA() __naked {
   __asm
		//calculo el metatile lo primero
		//dependiendo del metatile miro hacia donde deberia estar mirando (si es que es un metatile forzado hacia una dirección)
		//compruebo los semaforos?
		//compruebo si hay algun coche delante?

      ld a,OBJ_CAR_TORQUE(ix)
      ld OBJ_IA_TORQUE_TO(ix),a			//esto es para que si no podemos calcular donde tenemos que ir pues por lo menos que siga la misma direccion
	

      //aqui puede pasar dos cosas, que tenga una funcion especial para calcular la IA del coche,
      //o que no la tenga y simplemente este dando un paseito siguiendo las reglas del trafico
      ld h,OBJ_MACHINE_STATE_PTR+1(ix)
      xor a
      or h 
      jr Z,gtaCocheIAbasica


      ld l,OBJ_MACHINE_STATE_PTR(ix)
      call	___sdcc_call_hl
      ret



      //si voy por aqui simplemente voy a dar un paseito intentando seguir las normas de trafico
	gtaCocheIAbasica:	
		ld b,OBJ_POSX+1(ix)
		inc b					//le sumo 1 para ponerlo en el medio del coche aprox
		ld c,OBJ_POSY+1(ix)
		inc c					//le sumo 1 para ponerlo en el medio del coche aprox
		call _vtCheckMetaTileAsm

      ld a,h
      //vamos a calcular hacia donde deberiamos apuntar la dirección y lo vamos a meter en OBJ_IA_TORQUE_TO
      	
      //vamos a indicar en C si es doble sentido C=metatile o es todos hacia el mismo lado C=0
      //ya que la IA cambiará un poco segun esto:
      ld c,#12					//pongo por defecto que es doble sentido
      cp #12
      jr Z,gtaCocheIAs1
      ld c,#13
    	cp #13
      jr Z,gtaCocheIAs1
    	ld c,#0		//si llega aki es unico sentido los dos puntos (simplemente pillamos el que este más cerca y vamos paya)
    	
	gtaCocheIAs1:
      
		//0- todos hacia derecha
		//1- todos hacia abajo
		//2- todos hacia izq
		//3- todos hacia arriba
		//4- seguir en la misma direccion
		//5- horizontal mitad y mitad (doble sentido)
		//6- puente horizontal			(doble sentido)
		//7- vertical mitad y mitad		(doble sentido)
		//8- arena horizontal
		//9- arena vertical
		//10- parking
		//11- agua (solo se pueden mover barcos)
		//12- parque
		//16- edificio
		ld hl,#_metatileTraficDirection
		add l
		jr NC,gtaCocheIAs2
		inc h
	gtaCocheIAs2:
		ld l,a
		
		ld a,(hl)
		ld hl,#_traficGoToPos
		add a		//por 2
		add a		//por 4
		add l
		jr NC,gtaCocheIAs3
		inc h
	gtaCocheIAs3:
		ld l,a			//ya tengo HL apuntando a los puntos donde puede ir (si es 255 el primer valor tengo que mantener el rumbo) 
		
		ld a,(hl)
		
		inc a
		jp Z,gtaCocheIABuscaCamino			//si le sumo 1 y es 0 es que es 255, en el metatile en el que estoy no esta definido donde tengo que ir, asi que llamo a buscar camino para que busque un sitio por donde ir


		//aqui pueden pasar dos cosas, que la carretera sea de un solo sentido C=0, o que tenga ambos sentidos C>0
		xor a
		or c
		jp NZ,gtaCocheIAs3DobleSentido
		
	gtaCocheIAs3UnSentido:
		//si entramos aqui la carretera solo tiene un sentido, asi que lo unico que tenemos que hacer es ir a cualquiera de los
		//dos puntos que indican HL, para que no se le vaya la olla mucho vamos a ir al que caiga más cerca
		ld b,(hl)
		inc hl
		ld c,(hl)
		inc hl
		ld d,(hl)
		inc hl
		ld e,(hl)		//y ya tengo en BC y en DE los dos puntos a los que puedo ir
		
		push bc		//me guardo BC porke me lo voy a petar en breve
		
		ld h,OBJ_POSX+1(ix)			//meto en HL la pos actual del coche
      inc h                      //le incremento uno para que tome como referencia más o menos el centro del coche
		ld l,OBJ_POSY+1(ix)
		inc l

		ld a,#0b00000111			//le quito la parte alta del metatile, solo quiero quedarme con el tile (de 0 a 7)
		and h
		ld h,a
		ld a,#0b00000111
		and l
		ld l,a
		



		//y ahora voy a ver que está más lejos o más cerca, miro el primer punto
		ld a,h
		sub b
		jr NC,gtaCocheIAs3UnSentidos2
		neg
	gtaCocheIAs3UnSentidos2:
		ld b,a	//meto la cuenta en B
		
		ld a,l
		sub c		
		jr NC,gtaCocheIAs3UnSentidos3
		neg
	gtaCocheIAs3UnSentidos3:
		add b		//lo que de aqui es la diferencias de distancias, osea, (H-B) + (L-C) o (diferencia horizontal)+(diferencia vertical)
		ld b,a		//y lo dejo finalmente en B
		
		
		
		//ahora miro el segundo punto
		ld a,h
		sub d
		jr NC,gtaCocheIAs3UnSentidos4
		neg
	gtaCocheIAs3UnSentidos4:
		ld c,a	//meto la cuenta en B
		
		ld a,l
		sub e		
		jr NC,gtaCocheIAs3UnSentidos5
		neg
	gtaCocheIAs3UnSentidos5:
		add c		//lo que de aqui es la diferencias de distancias, osea, (H-D) + (L-E) o (diferencia horizontal)+(diferencia vertical)
		ld c,a		//y lo dejo finalmente en C
		
		//ahora tengo en B la distancia hacia el primer punto y en C la distancia hacia el segundo punto
		ld a,b
		sub c
		
		pop BC 		//recupero BC porque quiero recuperar el primer punto despues de la resta anterior (el pop no altera flags)
		
		jr C,gtaCocheIAs3UnSentidoPrimerPunto



	gtaCocheIAs3UnSentidoSegundoPunto:
		//esta más cerca el segundo punto DE
		//hago que el coche ponga rumbo hacia alli
		//la entrada es X=BC e Y=DE, y en A devuelve el angulo (256 based)  void vtAtan2_16()
      ld a,d
      sub h
      ld h,a

      ld a,e
      sub l
      ld l,a

      jr gtaCocheIAs3UnSentidoCalculaAngulo
      
		

		
	gtaCocheIAs3UnSentidoPrimerPunto:
		//esta más cerca el primer punto BC
      ld a,b
      sub h
      ld h,a

      ld a,c
      sub l
      ld l,a




   gtaCocheIAs3UnSentidoCalculaAngulo:
      ld b,h
      ld c,l

		//call _vtAtan2_16			//devuelve el angulo entre BC (punto actual del personaje) y DE (segundo punto)
      call _vtAtan2           //entra en BC x e y (en i8) y devuelve en A la dirección (256 based)
		//aqui ya debo tener en A el angulo al que debo ir, asi que lo meto en el angulo deseado
		ld OBJ_IA_TORQUE_TO(ix),a		//y sigo mirando otras cosas
		jp gtaCocheIACheckSemaforos
		
		
		




		
	gtaCocheIAs3DobleSentido:
      //vamos leyendo la posicion
      ld h,OBJ_POSX+1(ix)			//meto en HL la pos actual del coche
      inc h                      //le incremento uno para que tome como referencia más o menos el centro del coche
		ld l,OBJ_POSY+1(ix)
		inc l

		ld a,#0b00000111			//le quito la parte alta del metatile, solo quiero quedarme con el tile (de 0 a 7)
		and h
		ld h,a
		ld a,#0b00000111
		and l
		ld l,a




		//aqui C puede ser 12 o 13 (si C=12 es doble sentido horizontal y si C=13 es doble sentido vertical)
      ld a,#12
      cp c
      jr Z,gtaCocheIAs3DobleSentidoHorizontal






   gtaCocheIAs3DobleSentidoVertical:
      //si llego aqui una de dos, o voy hacia arriba por la derecha (), o hacia abajo por la izquierda ()
      ld a,#3
      cp h
      jr C,gtaCocheIAs3DobleSentidoVerticalDer

   gtaCocheIAs3DobleSentidoVerticalIzq:
      ld b,#1                    //  2,11, 5,11,		   //1- todos hacia abajo
      ld c,#11
      jp gtaCocheIAs3UnSentidoPrimerPunto
      
   gtaCocheIAs3DobleSentidoVerticalDer:
      ld b,#4                 //1,-4, 5,-4,		   //3- todos hacia arriba
      ld c,#0xFC     //-4
      jp gtaCocheIAs3UnSentidoPrimerPunto





   gtaCocheIAs3DobleSentidoHorizontal:
		//si llego aqui una de dos, o voy hacia la derecha por abajo (), o hacia la izquierda por arriba ()
      ld a,#3
      cp l
      jr C,gtaCocheIAs3DobleSentidoHorizontalAbajo

   gtaCocheIAs3DobleSentidoHorizontalArriba:
      ld b,#0xFC         //-4       //  -4,2, -4,5,		   //2- todos hacia izq
      ld c,#1
      jp gtaCocheIAs3UnSentidoPrimerPunto

   gtaCocheIAs3DobleSentidoHorizontalAbajo:
      ld b,#11          // 11,1, 11,3,		   //0- todos hacia derecha
      ld c,#4
      jp gtaCocheIAs3UnSentidoPrimerPunto
		
		




		
	gtaCocheIABuscaCamino:
      //aqui hay que ver en que dirección voy y hacer lo siguiente
      //buscar 2 metatiles más adelante si sigue habiendo carretera, (en ese caso seguir por ahi)
      //en caso de no haber carretera ahi, mirar 2 metatiles a la derecha de la dirección que vaya (+64 grados) y ver si ahi hay carretera (en ese caso tiro paya)
      //en caso de tampoco haber carretera ahi, mirar 2 metatiles a la izquierda de la dirección que vaya (-64 grados) y ver si hay carretera
		
		
		ld b,OBJ_POSX+1(ix)
		inc b					//le sumo 1 para ponerlo en el medio del coche aprox
		ld c,OBJ_POSY+1(ix)
		inc c					//le sumo 1 para ponerlo en el medio del coche aprox
		
		
		ld a,OBJ_CAR_TORQUE(ix)
		add #32			//le sumo 32 para poder calcular hacia que lado va (arriba, abajo, derecha, izquierda)
		//ahora puedo saber para que lado va simplemente mirando los dos ultimos bits  00 derecha, 01 abajo, 10, izquierda, 11 arriba
		bit 7,a
		jr NZ,gtaCocheIABuscaCaminoIzqArr
		
	gtaCocheIABuscaCaminoDerAba:
		bit 6,a
		jr NZ,gtaCocheIABuscaCaminoAbajo
		
	gtaCocheIABuscaCaminoDerecha:
		//estoy yendo hacia la derecha, compruebo un X+2, Y a ver que hay------------------------------DERECHA--------------------------
      ld OBJ_IA_TORQUE_TO(ix),#0

		ld a,#8		//lo pongo en X+1,Y
		add b
		ld b,a
		call gtaCocheIABuscaCaminoCheckMetatile
		

      ld OBJ_IA_TORQUE_TO(ix),#64
      ld a,b         //lo pongo en X,Y+1
		sub #8
		ld b,a
      ld a,#8
      add c
      ld c,a
      call gtaCocheIABuscaCaminoCheckMetatile


      ld OBJ_IA_TORQUE_TO(ix),#192
		ld a,c            //lo pongo en X,Y-1
      sub #16
      ld c,a
		call gtaCocheIABuscaCaminoCheckMetatile


      ld OBJ_IA_TORQUE_TO(ix),#0       //no encuentro nada, asi que sigo adelante como iba
      jp gtaCocheIACheckSemaforos





	
	gtaCocheIABuscaCaminoAbajo:
      //estoy yendo hacia abajo, compruebo un X,Y+2 a ver que hay------------------------------ABAJO--------------------------
      ld OBJ_IA_TORQUE_TO(ix),#64

		ld a,#8		//lo pongo en X,Y+1
		add c
		ld c,a
		call gtaCocheIABuscaCaminoCheckMetatile
		

      ld OBJ_IA_TORQUE_TO(ix),#128
      ld a,b         //lo pongo en X-1,Y
		sub #8
		ld b,a
      ld a,c
      sub #8
      ld c,a
      call gtaCocheIABuscaCaminoCheckMetatile


      ld OBJ_IA_TORQUE_TO(ix),#0
		ld a,#16            //lo pongo en X+1,Y
      add b 
      ld b,a
		call gtaCocheIABuscaCaminoCheckMetatile


      ld OBJ_IA_TORQUE_TO(ix),#64       //no encuentro nada, asi que sigo adelante como iba
      jp gtaCocheIACheckSemaforos
	
		
	
		
	gtaCocheIABuscaCaminoIzqArr:
		bit 6,a
		jr NZ,gtaCocheIABuscaCaminoArriba
		
		
	gtaCocheIABuscaCaminoIzquierda:	
      //estoy yendo hacia la izquierda, compruebo un X-2, Y a ver que hay------------------------------IZQUIERDA--------------------------
      ld OBJ_IA_TORQUE_TO(ix),#128

		ld a,b		//lo pongo en X-1,Y
		sub #8
		ld b,a
		call gtaCocheIABuscaCaminoCheckMetatile
		

      ld OBJ_IA_TORQUE_TO(ix),#192
      ld a,#8         //lo pongo en X,Y-1
		add b
		ld b,a
      ld a,c
      sub #8
      ld c,a
      call gtaCocheIABuscaCaminoCheckMetatile


      ld OBJ_IA_TORQUE_TO(ix),#64
		ld a,#16            //lo pongo en X,Y+1
      add c 
      ld c,a
		call gtaCocheIABuscaCaminoCheckMetatile


      ld OBJ_IA_TORQUE_TO(ix),#128       //no encuentro nada, asi que sigo adelante como iba
      jp gtaCocheIACheckSemaforos
	
	
	gtaCocheIABuscaCaminoArriba:
		//estoy yendo hacia arriba, compruebo un X, Y-2 a ver que hay------------------------------ARRIBA--------------------------
      ld OBJ_IA_TORQUE_TO(ix),#192

		ld a,c		//lo pongo en X,Y-1
		sub #8
		ld c,a
		call gtaCocheIABuscaCaminoCheckMetatile
		

      ld OBJ_IA_TORQUE_TO(ix),#0
      ld a,#8         //lo pongo en X+1,Y
		add b
		ld b,a
      ld a,c
      add #8
      ld c,a
      call gtaCocheIABuscaCaminoCheckMetatile


      ld OBJ_IA_TORQUE_TO(ix),#128
		ld a,b            //lo pongo en X-1,Y
      sub #16
      ld b,a
		call gtaCocheIABuscaCaminoCheckMetatile


      ld OBJ_IA_TORQUE_TO(ix),#192       //no encuentro nada, asi que sigo adelante como iba
      jp gtaCocheIACheckSemaforos
		
		
		
	gtaCocheIABuscaCaminoCheckMetatile:
		call _vtCheckMetaTileAsm
		ld a,h
		cp #22
		ret NC		//si no hay carry es que aqui no hay un camino
		pop af		//si hay carry es que podemos tirar hacia este camino, simplemente hacemos un pop porque no vamos a volver con el ret y tiramos palante



      //reajuste, en vez de ir directamente al angulo 0,64,128,192 vamos a intentar entrar por el lado correcto
      //si vamos a la derecha lo mejor es ir ajustado abajo (medio abajo 4,6)
      //si vamos arriba lo mejor es ir ajustado a la derecha (derecha medio  6,4)
      //si vamos a la izquierda lo mejor es ir ajustado arriba (medio arriba 4,1)
      //si vamos abajo lo mejor es ir ajustado a la izquierda (izquierda medio 1,4)
      //lo tengo todo en gtaReinsercionAddTable
      //en BC tengo la posicion desplazada al metatile que tengo que ir, osea si le hago el AND me puedo quedar con la base del metatile
      ld a,#0b11111000
      and b
      ld b,a
      ld a,#0b11111000
      and c
      ld c,a         //y ahora ya tengo en BC el comienzo del metatile
      ld a,OBJ_IA_TORQUE_TO(ix)        //leo de la tabla gtaReinsercionAddTable por donde tengo que entrar al metatile al que me dirijo
      rlca
      rlca
      rlca
      ld hl,#_gtaReinsercionAddTable
      add l
      jr NC,gtaCocheIABuscaCaminoCheckMetatiles2
      inc h
   gtaCocheIABuscaCaminoCheckMetatiles2:
      ld l,a
      ld a,b
      add (hl)
      ld b,a
      inc hl
      ld a,c
      add (hl)
      ld c,a         //y ya en BC tengo a donde tengo que dirigirme
      //push bc        //me guardo BC para luego calcular la distancia

      //ahora leo la posicion actual en HL
      ld d,OBJ_POSX+1(ix)
		inc d					//le sumo 1 para ponerlo en el medio del coche aprox
		ld e,OBJ_POSY+1(ix)
		inc e					//le sumo 1 para ponerlo en el medio del coche aprox


      ld a,b            //ahora hago ATAN2 DE ( POSICION DESTINO (BC) - POSICION ORIGEN (DE) )
      sub d
      ld b,a
      ld a,c
      sub e
      ld c,a




		//call _vtAtan2_16			//devuelve el angulo entre BC (punto actual del personaje) y DE (segundo punto)
      call _vtAtan2           //entra en BC x e y (en i8) y devuelve en A la dirección (256 based)
		//aqui ya debo tener en A el angulo al que debo ir, asi que lo meto en el angulo deseado
		ld OBJ_IA_TORQUE_TO(ix),a		//y sigo mirando otras cosas

		jr gtaCocheIACheckSemaforos		//si hay carry hacia delante hay carretera, asi que siga palante

   
		





		
	gtaCocheIACheckSemaforos:
      //si algun semaforo nos esta afectando (lo tenemos delante y esta en rojo, frenamos y salimos)
      //de paso tambien vamos a mirar si tenemos un coche justo delante nuestra
      //pero hay excepciones, 
      //1º- si no estamos dibujando el coche en pantalla directamente no miramos nada (ojos que no ven, corazon que no siente)
      //2º- si soy una ambulancia o policia y me sale de la polla me salto el semaforo (llamadme rebelde si quereis)

      bit 2,OBJ_FLAGS(ix)           //no estoy en pantalla, asi que pasando de todas las comprobaciones
      jp Z,gtaCocheIAAcelera

      

      ld hl,#GAME_OBJECTS_PTR
      ld de,#48
      ld b,#16
   gtaCocheIACheckSemaforosBucle:
      ld a,(hl)
      or a
      jp Z,gtaCocheIACheckSemaforosBucleFin     //si es 0 no es nada, sigo con el bucle

      cp #TYPE_AUX_OBJECT
      jr NZ,gtaCocheIACheckSemaforosBucleChkCoches
      

      push hl        //miramos si es un semaforo o no
      pop iy 

      bit 2,OBJ_FLAGS(iy)           //si el semaforo o lo que sea no esta en pantalla, pasamos al siguiente, solo funcionan los semaforos que esten en pantalla
      jp Z,gtaCocheIACheckSemaforosBucleFin

      exx
      ld l,OBJ_STEP_FUNC(iy)     //comprobamos si es un semaforo o no
      ld h,OBJ_STEP_FUNC+1(iy)
      ld de,#_gtaSemaforoStepFunc
      xor a
      sbc hl,de
      exx
      jp NZ,gtaCocheIACheckSemaforosBucleFin       //NO es un semaforo tiramos palante

      //ahora miro si esta en rojo, sino no hay que hacer nada (OBJ_FRAME_INDEX+1 == 2)
      ld a,OBJ_FRAME_INDEX+1(iy)
      dec a
      dec a
      jp NZ,gtaCocheIACheckSemaforosBucleFin    //si restando 2 no es cero es que no esta en rojo, podemos seguir con el bucle


      //si llego aqui IY es un semaforo, ahora tendria que comprobar si el semaforo esta mirando en la direccion opuesta a la que voy
      //para eso miro si SPEEDX del semaforo es positivo (afecta si vas a la izq o abajo) o negativo (afecta si vas a la der o arriba)
      ld a,OBJ_CAR_TORQUE(ix) //cargo el angulo actual del coche en A 
      add #96                 //le añado 96 para que si va a la der o arriba me quede un numero positivo (bit 7 a 0) y si va a izq/aba me quede un numero negativo (bit 7 a 1)
      
      bit 7,OBJ_SPEEDX+1(iy)
      jr Z,gtaCocheIACheckSemaforosBucleSemaIzqAba

   gtaCocheIACheckSemaforosBucleSemaDerArr:
      bit 7,a
      jp NZ,gtaCocheIACheckSemaforosBucleFin          //el semaforo para a der/arr, pero el jugador va a izq/aba
      jr gtaCocheIACheckSemaforosBucleSemaforoCheckFrena
      

   gtaCocheIACheckSemaforosBucleSemaIzqAba:
      bit 7,a
      jp Z,gtaCocheIACheckSemaforosBucleFin          //el semaforo para a izq/aba, pero el jugador va a der/arr
      //jr gtaCocheIACheckSemaforosBucleSemaforoCheckFrena  no hace falta 
      

   gtaCocheIACheckSemaforosBucleSemaforoCheckFrena:

      //el semaforo esta mirando al coche, ahora tengo que ver si el mismo semaforo esta lo suficientemente cerca para pararlo o aun no
      push hl 
      push de
      push bc
      call _gtaDistanceIX_IY

      


      ld a,#6
      cp h
      pop bc      //unaffected carry pops
      pop de 
      pop hl
      jr C,gtaCocheIACheckSemaforosBucleFin        //si no estoy cerca sigo con el bucle

      //si entro aqui estoy cerkita, freno el coche
      ld a,OBJ_CAR_SPEED(ix)
      sra a                      //divido la velocidad entre 2
      ld OBJ_CAR_SPEED(ix),a
      jp gtaCocheIAGira       //salimos del bucle, ya no miramos nada más y me voy directamente a girar para que no vuelva a acelerar aun




      


   gtaCocheIACheckSemaforosBucleChkCoches:
      bit 5,a              //miramos si es otro coche
      jr Z,gtaCocheIACheckSemaforosBucleFin
   
      //primero miro que no sea yo mismo, si IX == HL no hago nada obviamente
      push de        // ME GUARDO HL Y DE
      push hl

      push ix        // meto en DE = IX
      pop de

      xor a          //resto HL - DE para saber si soy yo mismo o no
      sbc hl,de

      pop hl         // recupero HL y DE
      pop de
      jr Z,gtaCocheIACheckSemaforosBucleFin     //si la resta da 0 es que soy yo mismo, no miro nada sigo el bucle


      //si entro aqui es otro coche, vemos si esta cerca o no
      //vamos a hacer el calculo muy simple, primero miro para que cuarto estoy mirando (izq,der,arr,aba)
      push hl
      pop iy      //meto en IY el otro coche para hacer las comprobaciones

      exx         //me cambio al EXX porque voy a necesitar todos los registros
      ld d,OBJ_POSX+1(ix)
      ld e,OBJ_POSY+1(ix)

      ld a,OBJ_CAR_TORQUE(ix)
      add #32     //le añado 32 y ahora los dos bits más altos me indican para que lado estoy yendo
      bit 7,a
      jr NZ,gtaCocheIACheckSemaforosBucleChkCochesMirandoIzqArr

   gtaCocheIACheckSemaforosBucleChkCochesMirandoDerAba:
      bit 6,a
      jr NZ,gtaCocheIACheckSemaforosBucleChkCochesMirandoAba

   gtaCocheIACheckSemaforosBucleChkCochesMirandoDer:
      ld a,#5
      add d
      ld d,a
      jr gtaCocheIACheckSemaforosBucleChkCochesMirandoCheckDistance
      
   gtaCocheIACheckSemaforosBucleChkCochesMirandoAba:
      ld a,#5
      add e
      ld e,a
      jr gtaCocheIACheckSemaforosBucleChkCochesMirandoCheckDistance
      


   gtaCocheIACheckSemaforosBucleChkCochesMirandoIzqArr:
      bit 6,a
      jr NZ,gtaCocheIACheckSemaforosBucleChkCochesMirandoArr

   gtaCocheIACheckSemaforosBucleChkCochesMirandoIzq:
      ld a,#5
      sub d
      ld d,a
      jr gtaCocheIACheckSemaforosBucleChkCochesMirandoCheckDistance

   gtaCocheIACheckSemaforosBucleChkCochesMirandoArr:
      ld a,#5
      sub e
      ld e,a
      //jr gtaCocheIACheckSemaforosBucleChkCochesMirandoCheckDistance  NO HACE FALTA
      

   gtaCocheIACheckSemaforosBucleChkCochesMirandoCheckDistance:
      ld a,OBJ_POSX+1(iy)
      sub d
      jr C,gtaCocheIACheckSemaforosBucleChkCochesMirandoCheckDistances2
      neg
   gtaCocheIACheckSemaforosBucleChkCochesMirandoCheckDistances2:
      cp #DISTANCIA_COCHE_FRENA
      jr NC,gtaCocheIACheckSemaforosBucleChkCochesMirandoCheckDistanceSal     //esta muy lejos horizontalmente

      ld a,OBJ_POSY+1(iy)
      sub e
      jr C,gtaCocheIACheckSemaforosBucleChkCochesMirandoCheckDistances4
      neg
   gtaCocheIACheckSemaforosBucleChkCochesMirandoCheckDistances4:
      cp #DISTANCIA_COCHE_FRENA
      jr NC,gtaCocheIACheckSemaforosBucleChkCochesMirandoCheckDistanceSal     //esta muy lejos verticalmente


      //si llega aqui y no me he equivocado en los calculos lo tenemos delante, asi que frenamos
      //si entro aqui estoy cerkita, freno el coche
      ld a,OBJ_CAR_SPEED(ix)
      srl a                      //divido la velocidad entre 2
      srl a                      //divido la velocidad entre 2
      srl a                      //divido la velocidad entre 2      
      ld OBJ_CAR_SPEED(ix),a
      jp gtaCocheIAGira       //salimos del bucle, ya no miramos nada más y me voy directamente a girar para que no vuelva a acelerar aun


      
   gtaCocheIACheckSemaforosBucleChkCochesMirandoCheckDistanceSal:
      exx

   gtaCocheIACheckSemaforosBucleFin:
      add hl,de
      dec b
      jp NZ,gtaCocheIACheckSemaforosBucle







	
	
		
		
	gtaCocheIAAcelera:
		//si entramos aqui es que tenemos via libre para pisarle al coche, acelera lo que pueda (mirando los limites claro)
      //el limite va a ser 3/4 partes de lo maximo que corra el coche elegido
      //si esta cogiendo una curva, el limite va a ser la mitad 1/2 partes de lo maximo que corra el coche
      //KABESA

      ld d,#0
      ld e,OBJ_SELECTED_CAR(ix)
      ld hl,#_carPropMaxSpeed
      add hl,de
      ld c,(hl)      //guardo en C = max speed

      ld h,OBJ_MACHINE_STATE_PTR+1(ix)
      xor a
      or h
      jr Z,gtaCocheIAAceleraModoPaseo

   gtaCocheIAAceleraModoAsaco:
      ld a,OBJ_CAR_SPEED(ix)
      cp c
      jr NC,gtaCocheIAGira
      add #3
      ld OBJ_CAR_SPEED(ix),a     //#32      //por probar
      jr gtaCocheIAGira

   gtaCocheIAAceleraModoPaseo:
      ld a,c
      srl a    //entre 2  (modo paseo)
		ld OBJ_CAR_SPEED(ix),a     //#32      //por probar

   gtaCocheIAGira:
      ld a,OBJ_CAR_TORQUE(ix)	      //si es cero vamos flama, no hay que girar el coche, vamos en el angulo correcto, salimos
      sub OBJ_IA_TORQUE_TO(ix)
      ret Z          

      add #8
      cp #16
      jr NC,gtaCocheIAAceleras2

      //si entra aqui es que la diferencia era entre -8 y 8, por lo tanto esta a menos de 8 grados de donde debia estar, 
      ld a,OBJ_IA_TORQUE_TO(ix)     //por lo tanto puedo asignarle directamente el angulo para que no haya baile de angulos
      ld OBJ_CAR_TORQUE(ix),a
      ret 


   gtaCocheIAAceleras2:
      sub #8         //sino vuelvo a poner la diferencia como estaba (-8), y miro si la diferencia es un numero positivo o negativo

      bit 7,a
      jr NZ,gtaCocheIAAceleras3
      
      //si es positivo, en este caso tengo que restar al angulo
      ld a,OBJ_CAR_TORQUE(ix)	      //girando a la izquierda (restando)
      sub #8
      ld OBJ_CAR_TORQUE(ix),a
      ret


   gtaCocheIAAceleras3:
      //si es negativo, en este caso tengo que sumar al angulo
      ld a,OBJ_CAR_TORQUE(ix)	      //girando a la izquierda (restando)
      add #8
      ld OBJ_CAR_TORQUE(ix),a
      ret

   __endasm;
}


//está ia hace que el coche acelere al maximo hacia un punto concreto OBJ_CAR_TILETO_X y OBJ_CAR_TILETO_Y
void gtaCocheIAgotoProta() __naked {
   __asm
      ld a,(#0xB500+OBJ_POSX+1)
      ld OBJ_CAR_TILETO_X(ix),a

      ld a,(#0xB500+OBJ_POSY+1)
      ld OBJ_CAR_TILETO_Y(ix),a
      
      jp _gtaCocheIAgotoPoint

   __endasm;
}



//está ia hace que el coche acelere al maximo hacia un punto concreto OBJ_CAR_TILETO_X y OBJ_CAR_TILETO_Y
void gtaCocheIAgotoPoint() __naked {
   __asm
      ld a,OBJ_CAR_TILETO_X(ix)
      sub OBJ_POSX+1(ix)
      ld b,a

      ld a,OBJ_CAR_TILETO_Y(ix)
      sub OBJ_POSY+1(ix)
      ld c,a
      call _vtAtan2              //leo a que angulo voy, y lo meto en TORQUE_TO

      ld OBJ_IA_TORQUE_TO(ix),a  

      jp gtaCocheIAAcelera    //y llamo a esto hace que acelere y gire y retorne
      //jp gtaCocheIAGira

   __endasm;
}


/*
//por practicar vamos a ponerlo en el centro de la pantalla simplemente
//despues más adelante se podran tener en cuenta ciertas cosas más
void moveCamera() __naked {
   __asm
   
      ld	ix,(_vtGameObjPtr)

      ld e,OBJ_SPEEDX(ix)
      ld d,OBJ_SPEEDX+1(ix)

      bit 7,d
      jr Z,moveCameras2
      //de es negativo, lo paso a positivo fasi
      ld hl,#0
      xor a
      sbc hl,de
      ld de,#10
      sbc hl,de
      jr C,moveCamerasSaltaHorizontal
      //sino, hay que ajustar para que la camara se vaya un poco a la izq
      ld b,#POSICION_SCROLL_X_PLAYER+10
      jr moveCamerasMueveHorizontal

   moveCameras2:
      ld hl,#10
      ex de,hl
      xor a
      sbc hl,de
      jr C,moveCamerasSaltaHorizontal
      ld b,#POSICION_SCROLL_X_PLAYER-10
      jr moveCamerasMueveHorizontal



   //moveCamerasCentraHorizontal:
     // ld b,#POSICION_SCROLL_X_PLAYER


   moveCamerasMueveHorizontal:
      ld a,OBJ_POSX+1(ix)
      sub b
      jr NC,moveCamerasMueveHorizontal2
      ld a,#0
   moveCamerasMueveHorizontal2:
      ld (#_vtSceneScrollXto),a





   moveCamerasSaltaHorizontal:
      ld e,OBJ_SPEEDY(ix)
      ld d,OBJ_SPEEDY+1(ix)

      bit 7,d
      jr Z,moveCameras4
      //de es negativo, lo paso a positivo fasi
      ld hl,#0
      xor a
      sbc hl,de
      ld de,#10
      sbc hl,de
      jr C,moveCamerasSaltaVertical
      //sino, hay que ajustar para que la camara se vaya un poco a la izq
      ld b,#POSICION_SCROLL_Y_PLAYER+6
      jr moveCamerasMueveVertical

   moveCameras4:
      ld hl,#10
      ex de,hl
      xor a
      sbc hl,de
      jr C,moveCamerasSaltaVertical
      ld b,#POSICION_SCROLL_Y_PLAYER-6
      jr moveCamerasMueveVertical



   //moveCamerasCentraVertical:
     // ld b,#POSICION_SCROLL_Y_PLAYER


   moveCamerasMueveVertical:
      ld a,OBJ_POSY+1(ix)
      sub b
      jr NC,moveCamerasMueveVertical2
      ld a,#0
   moveCamerasMueveVertical2:
      ld (#_vtSceneScrollYto),a


   moveCamerasSaltaVertical:
      ret



   __endasm;
}
*/




void moveCamera2() __naked {
   __asm
      ld ix,(_vtGameObjPtr)
      
      //pongo el modo turbo pero solo si vtScrollCRTC3 != 245, sino me espero
      ld a,(#_vtScrollCRTC3)
      sub #245
      jr Z,moveCamera2s1

      ld a,#1                 //si vtScrollCRTC3 != 245, activo el modo turbo del scroll, se movera a tiles directamente en vez de a medio tile
      ld (#_vtScrollTurbo),a

   moveCamera2s1:
      ld l,OBJ_SPEEDX(ix)
      ld h,OBJ_SPEEDX+1(ix)
      add hl,hl
      add hl,hl
      add hl,hl
      add hl,hl

      ld a,OBJ_POSX+1(ix)
      add #2                  //le añado 2 porque los coches tienen 4 de ancho
      add h
      //and #0b11110000
      //add #8
      and #0b11111000
      add #4
      sub #POSICION_SCROLL_X_PLAYER
      jr NC,moveCamera2s2
      xor a		//si hay carry en la resta, lo dejamos a 0 y ya

      
   
   moveCamera2s2:
      ld (#_vtSceneScrollXto),a

      ld l,OBJ_SPEEDY(ix)
      ld h,OBJ_SPEEDY+1(ix)
      add hl,hl
      add hl,hl
      add hl,hl
      add hl,hl

      ld a,OBJ_POSY+1(ix)
      add #2                  //le añado 2 porque los coches tienen 4 de alto
      add h
      //and #0b11110000
      //add #8
      and #0b11111000
      add #4
      sub #POSICION_SCROLL_Y_PLAYER+2

      jr NC,moveCamera2s3
      xor a		//si hay carry en la resta, lo dejamos a 0 y ya

   moveCamera2s3:
      ld (#_vtSceneScrollYto),a

      ret
   __endasm;
}




void moveCamera3() __naked {
   __asm
      ld	ix,(_vtGameObjPtr)
      
      xor a                   //quitamos el modo turbo si tenemos esta camara
      ld (#_vtScrollTurbo),a

      ld a,(#_vtSceneScrollXto)
      ld b,a

      ld l,OBJ_SPEEDX(ix)
      ld h,OBJ_SPEEDX+1(ix)
      add hl,hl
      add hl,hl
      add hl,hl

      ld a,OBJ_POSX+1(ix)
      add h
      
     moveCamera3s0:
      sub #POSICION_SCROLL_X_PLAYER
      jr NC,moveCamera3s1
      xor a		//si hay carry en la resta, lo dejamos a 0 y ya
      
     moveCamera3s1:
      ld c,a

      
      //ld (#_vtSceneScrollXto),a
   
   moveCamera3s2:
      ld a,(#_vtSceneScrollYto)
      ld d,a


      ld l,OBJ_SPEEDY(ix)
      ld h,OBJ_SPEEDY+1(ix)
      add hl,hl
      add hl,hl
      add hl,hl

      ld a,OBJ_POSY+1(ix)
      add h
      sub #POSICION_SCROLL_Y_PLAYER
      
	  jr NC,moveCamera3s2b
      xor a		//si hay carry en la resta, lo dejamos a 0 y ya
      
    moveCamera3s2b:
      ld e,a
      //ld (#_vtSceneScrollYto),a

      //B y D donde estan los scroles actualmente, C y E donde deberian colocarse

      //ahora vemos si hay más diferencia en horizontal o en vertical para mover el scroll horizontal o el vertical
      ld a,b
      sub c
      jr NC,moveCamera3s3
      neg
   moveCamera3s3:
      ld b,a      //lo meto en B


      ld a,d
      sub e
      jr NC,moveCamera3s4
      neg
   moveCamera3s4:
      ld d,a      //lo meto en D


      //ahora en B y D tengo la diferencia de donde estan a donde deberian estar, 
      //si D > B muevo scroll vertical a E
      //sino muevo scroll horizontal a C
      ld a,b
      sub d
      jr C,moveCamera3s5mueveVertical
   moveCamera3s5mueveHorizontal:
      ld a,c
      ld (#_vtSceneScrollXto),a
      ret
   moveCamera3s5mueveVertical:
      ld a,e
      ld (#_vtSceneScrollYto),a
      ret
   __endasm;
}

//devuelve 0 si soy pasajero, 1 si soy conductor, siendo IX el personaje, e IY el coche
void compruebaConductorIXenIY() __naked {
   __asm
      ld a,OBJ_PARENT_PTR(iy)
      .dw #0x95DD    //sub ixl
      jr NZ,compruebaConductorIXenIYsalPasajero
      ld a,OBJ_PARENT_PTR+1(iy)
      .dw #0x94DD    //sub ixh
      jr NZ,compruebaConductorIXenIYsalPasajero

      ld a,#1     //salgo siendo el conductor
      ret

   compruebaConductorIXenIYsalPasajero:
      xor a       //salgo siendo el pasajero
      ret
   __endasm;
}

//comprueba si IX tiene que robar un coche o no, devuelve 0 si ha robado el coche o >1 si no lo ha robado
//si ya hubiese alguien en el coche, me monto en otro asiento (max 4)
void compruebaRoboCoche() __naked {
   __asm
      //ANTES DE NADA, MIRO SI YA VOY EN UN COCHE, ya que en ese caso lo que tengo que hacer es salir del coche
      ld a,OBJ_PARENT_PTR+1(ix)
      or a
      jr Z,compruebaRoboCocheIntentoRobar    //no estoy montado en ningun coche, intento ver si tengo que robar alguno

      //si entra aqui estoy montado ya en un coche, asi que lo que tengo que hacer es bajarme del coche
      xor a
      ld OBJ_PARENT_PTR(ix),a          //bajarse es facil, solo tengo que poner el enlace al parent a 0 y ya estoy desconectado del coche, por lo tanto estoy fuera
      ld OBJ_PARENT_PTR+1(ix),a
      res 7,OBJ_FLAGS(ix)           //pongo para que vuelva a colisionar
      ret


   compruebaRoboCocheIntentoRobar:
      //voy a ir recorriendo con iy todos los objetos hasta que me encuentre con un coche y entonces 
      //miro si estoy cerca para robarlo o no
      ld iy,#GAME_OBJECTS_PTR
      ld de,#48      //esto es para pasar al siguiente luego
      ld b,#0

   compruebaRoboCocheBuc1:
      ld a,OBJ_ACTIVE_TYPE(iy)
      and #TYPE_CAR
      jp Z,compruebaRoboCocheBucFin
      //bit 5,a
      //jp Z,compruebaRoboCocheBucFin    //los coches son 32+algo, comprobamos el bit 5 y si es cero no es coche, pasamos al siguiente

      //si entra aqui es que este objeto es un coche, tengo que mirar si estamos lo suficientemente cerca
      ld c,OBJ_POSX+1(iy)     //byte alto
      ld a,OBJ_POSX+1(ix)
      sub c    //resto B
      inc a    //si aqui el resultado es -2 -1 0 1 2 considero que esta cerca
      inc a    //si le sumo 2 el resultado podria ser 0,1,2,3,4
      cp #6    //asi que resto 5 y si hay carry es que estamos cerca
      jp NC,compruebaRoboCocheBucFin

      //si llego aqui, horizontalmente estamos cerca, voy a ver la vertical
      ld c,OBJ_POSY+1(iy)     //byte alto
      ld a,OBJ_POSY+1(ix)
      sub c    //resto B
      inc a    //si aqui el resultado es -2 -1 0 1 2 considero que esta cerca
      inc a    //si le sumo 2 el resultado podria ser 0,1,2,3,4
      cp #6    //asi que resto 5 y si hay carry es que estamos cerca
      jp NC,compruebaRoboCocheBucFin
      
      //si llego aqui estamos cerca horizontal y verticalmente, podemos montarnos en el coche
      //miro si el coche ya tiene conductor o no
      ld a,OBJ_PARENT_PTR+1(iy)
      or a
      jr Z,compruebaRoboCocheBucSinConductor		//NO TIENE CONDUCTOR, LO ROBO DIRECTAMENTE


	  //ahora pueden pasar 2 cosas, o que tenga conductor pero no este creada la instancia (caso 255) (en cuyo caso se tiene que crear en este momento)
		inc a				//compruebo si A+1 = 0
		jr Z,compruebaRoboCocheBucCreaConductor			//SI ES 0, ES QUE TENEMOS QUE CREAR EL CONDUCTOR EN ESE MOMENTO, Y DESPUES ROBAR EL COCHE COMO SI ESTUVIESE VACIO



		// o que tenga conductor y su instancia creada, con lo cual simplemente lo tenemos que sacar del coche
		dec a									//SI NO ES 0, ES QUE YA TENGO UNA INSTANCIA DE UN CONDUCTOR DENTRO DE ESTE COCHE, LO ECHO PA´FUERA Y LO ROBO COMO SI NO HUBIESE NADIE
		ld h,a
		ld l,OBJ_PARENT_PTR(iy)		//ya tengo en HL el puntero al conductor (tengo que sacarlo del coche, osea, poner su parent a 0)
		ld de,#7
		add hl,de		//le sumo 7 para irme al OBJ_PARENT_PTR(HL)
		xor a
		ld (hl),a		//lo pongo a 0		el OBJ_PARENT_PTR byte bajo
		inc hl
		ld (hl),a		//lo pongo a 0		el OBJ_PARENT_PTR byte alto
		
		//y ya tengo al conductor fuera, puedo robar el coche vacio sin problemas
		jr compruebaRoboCocheBucSinConductor


		//aqui ya no va a entrar nunca
      //xor a    //ld a,#0     he robado el coche
      //ret      //de momento no hago nada más


	compruebaRoboCocheBucCreaConductor:
		//tengo que crear el conductor del coche y luego robarlo
		//TODO  hay que crear un personaje aki random, y luego seguir robando el coche sin conductor
      //pongo en DE el posX y posY del prota IX que esta robando el coche

      
      ld d,OBJ_POSX+1(ix)
      ld e,OBJ_POSY+1(ix)

      push ix        //me guardo el IX porque al crear el nuevo personaje al ke le voy a robar el coche me voy a petar ix tambien
      push de     //me guardo DE porque el freeGameObj se lo peta

      call _vtGetFreeGameObjPtr        //preparamos el objeto del prota
      
      ld ix,(#_vtGameObjPtr)
      
      pop de         //recupero ya DE para usarlo en el gtaConfigRandomPerson

      // SAFE_GET_FREE
      //xor a       //hago una comprobacion rapida por si no hubiese sitio para meter más instancias en el engine
      //.dw #0xB4DD    //or ixh      //simplemente hago 0 or IXH (que tiene que ser 0xB5 o por ahi), si el or da Zero es que no hubo espacio para agregarlo, podemos eliminarlo
      //jp Z,compruebaRoboCocheBucSinConductorPre    //si no hay huecos libres para sacar más personajes (cosa que no creo pero weno) robo el coche sin sacar a nadie
   
      call _gtaConfigRandomPerson

      //ld hl,#_iaPersonBuscandoAcera    //como lo voy a dejar tirado en medio de la carretera seguramente, la primera IA que le pongo es que busque una acera y se quite del medio
      ld hl,#_iaPersonAtacandoProta      //como acabo de robarle el coche, lo cabreo y me ataca
      ld OBJ_MACHINE_STATE_PTR(ix),l      //aunque lo suyo seria dejarlo tirado en el suelo un tiempo primero
      ld OBJ_MACHINE_STATE_PTR+1(ix),h
      ld OBJ_FRAMES_FOR_NEXT_STATE(ix),#250


   compruebaRoboCocheBucCreaConductorConfigNPC:
      //leo si el coche era especial de algun npc ya que en ese caso, tengo que configurar al conductor
      ld a,OBJ_ID_NPC_CAR(iy)
      sub #16      //si resto 17 y no hay carry es que era un coche especial
      jr C,compruebaRoboCocheBucSinConductorPre

      //si llego aqui es un coche especial por lo tanto hay que configurar el NPC
      call _gtaConfigNPC


   compruebaRoboCocheBucSinConductorPre:
      pop ix
      ld (#_vtGameObjPtr),ix



   compruebaRoboCocheBucSinConductor:
      //si llego aqui el coche no tiene conductor, por lo tanto solo tengo que montarme yo
      //antes de montarme preparo para que se borre mi sprite de mi muñeco (ya que no se va a volver a pintar más y si no lo borro se kedara ahi pillao)
      push ix
      push iy
      call _vtPrepareRestoreBackground
      call _vtPushTilesToDraw
      pop iy
      pop ix

      .dw #0x7ddd    //ld a,ixl
      ld OBJ_PARENT_PTR(iy),a          //indico en el coche que ese indice va a ser el conductor
      .dw #0x7cdd    //ld a,ixh
      ld OBJ_PARENT_PTR+1(iy),a          //indico en el coche que ese indice va a ser el conductor

      .dw #0x7dfd    //ld a,iyl
      ld OBJ_PARENT_PTR(ix),a          //y ahora indico en el personaje, que esta montado en el coche IY
      .dw #0x7cfd    //ld a,iyh
      ld OBJ_PARENT_PTR+1(ix),a          //y ahora indico en el personaje, que esta montado en el coche IY

      ld a,OBJ_SELECTED_CAR(iy)     //selecciono el coche
      ld OBJ_SELECTED_CAR(ix),a

      set 7,OBJ_FLAGS(ix)     //pongo para que no colisione nuestro muñeco ahora mismo que va en el coche

      //ld OBJ_ID_NPC_CAR(iy),#0      //indico que el coche que acabo de robar ya no es UN NPC, queda liberado

		//antes de salir, pongo el vtScrollLockTime a 4 para que este 4 frames sin scrollear y poder estabilizar los bordes izq y der
		//al entrar en modo turbo cualquier scroll lo va a hacer al tile, por lo tanto tenemos que pintar la franja negra del borde izq/der para que no se quede ahi
      push ix
      push iy

		//ld a,#4
		//ld (#_vtScrollLockTime),a

		ld	a,(#_vtScrollCRTC3)			//ponemos el CRTC3 en la posicion inicial que va a usar el modo turbo
      cp #0xf5
      jr Z,compruebaRoboCocheBucSinConductorrestB

   compruebaRoboCocheBucSinConductorrestA:
		xor a							//para curarme en salud y que no salga la franja negra al pillar el coche, redibujo los dos bordes izq y der ademas de bloquear el scroll varios frames
      ld hl,#_vtPushTilesIniY
      ld (hl), a
      ld hl,#_vtPushTilesIniX
      ld (hl), a
  		inc a
  		ld hl,#_vtPushTilesEndX
      ld (hl), a
      ld hl,#_vtPushTilesEndY
		ld (hl), #25
		call _vtPushTilesToDrawColumn
		jr compruebaRoboCocheBucSinConductorrestC
		
	compruebaRoboCocheBucSinConductorrestB:	
		xor a							//pinto el borde der
      ld hl,#_vtPushTilesIniY
      ld (hl), a
      ld hl,#_vtPushTilesEndY
		ld (hl), #25
		ld a,#38
		ld hl,#_vtPushTilesIniX
      ld (hl), a
      inc a
      ld hl,#_vtPushTilesEndX
      ld (hl), a
 
		call _vtPushTilesToDrawColumn

   compruebaRoboCocheBucSinConductorrestC:
      pop iy
      pop ix

      xor a                            //he robado el coche y salgo
      
      ret




   compruebaRoboCocheBucFin:
      add iy,de
      inc b
      ld a,#MAX_OBJECTS
      cp b
      jp nz,compruebaRoboCocheBuc1

      //salgo SIN ROBAR NINGUN COCHE
      ld a,#1        //si no ha robado el coche tiene que devolver un numero mayor de cero para que haga el disparo
      ret
   __endasm;
}



//crea un disparo para donde este mirando IX (o hace un puñetazo si no tiene arma) devuelve un 0 en A
void gtaCreaArma() __naked {
   __asm
      
      dec OBJ_AMMO(ix)
      jr NZ,gtaCreaArmas0

      //si es cero me he quedado sin esta arma (sin municion o rota), indico que el arma seleccionada ahora son los puños
      ld OBJ_SELECTED_WEAPON(ix),#0
      ld OBJ_AMMO(ix),#255    //y le pongo municion a tope (los puños nunca se gastan)

   gtaCreaArmas0:

      //antes de poner la velocidad a cero, nos guardamos el angulo al que vamos actualmente
      ld b,OBJ_SPEEDX(ix)
      ld c,OBJ_SPEEDY(ix)
      call _vtAtan2     //devuelve en A el angulo y lo metemos en OBJ_PERSON_TORQUE que normalmente no hace falta rellenarlo en las personas, pero ahora nos va a venir bien

      ld OBJ_PERSON_TORQUE(ix),a    //meto el angulo en OBJ_PERSON_TORQUE

      ld a,#128
      ld OBJ_FRAME_SPEED(ix),a
      xor a
      ld OBJ_FRAME_INDEX(ix),a
      ld OBJ_FRAME_INDEX+1(ix),a

      ld OBJ_SPEEDY(ix),a           //pongo la velocidad a 0 para que no se mueva mientras dispara o melea
      ld OBJ_SPEEDY+1(ix),a
      ld OBJ_SPEEDX(ix),a
      bit 7,OBJ_SPEEDX+1(ix)
      jr Z,gtaCreaArmas1

      dec OBJ_SPEEDX(ix)      //lo pongo en 255
   gtaCreaArmas1:


      bit 1,OBJ_ACTIVE_TYPE(ix)           //compruebo si es hombre/mujer
      jr NZ,gtaCreaArmasMan        //si este bit es cero es mujer

      ld hl,#_animWomanFist
      jr gtaCreaArmas2

   gtaCreaArmasMan:
      ld hl,#_animManFist

   gtaCreaArmas2:
      ld OBJ_ANIMATION_LIST(ix),l
      ld OBJ_ANIMATION_LIST+1(ix),h


      ld OBJ_IA_LOCK_MOVE(ix),#8       //aprox 2 segundos

      ld a,OBJ_SELECTED_WEAPON(ix)
      


    

	gtaCreaArmas3:

      //si no es cero, tenemos que crear el sprite del disparo, para ello creo un objeto sobre la marcha que va a dibujar el sprite y luego se autodestruye
      // (le ponemos el step y el draw de disparo y como parent el ix)
      push ix
      

      push af
      call _vtGetFreeGameObjPtr
      ld ix,(#_vtGameObjPtr)

      // SAFE_GET_FREE WEAPON
      //xor a
      //.dw #0xB4DD    //or ixh      compruebo si IXH es cero, porque eso significa que no tengo huecos libres para crear el objeto, con lo cual salgo del tiron
      //jr NZ,gtaCreaArmas3b

      //pop af      //no hay sitio para armas, salgo del tiron
      //pop ix
      //ld (#_vtGameObjPtr),ix
      //ret


   gtaCreaArmas3b:
      ld OBJ_ACTIVE_TYPE(ix),#TYPE_AUX_OBJECT               //se lo asigno al tipo

      pop af
      pop iy      //tengo en IY el creador del arma

      ld OBJ_SELECTED_WEAPON(ix),a
      dec a
      ld hl,#_animWeaponNavaja                 //es mujer, le pongo directamente el pelo largo por simplificar de momento
      add a,a     //x2
      add a,a     //x4
      add a,a     //x8
      ld e,a
      ld d,#0
      add hl,de      //pongo la animacion correcta segun el arma elegida
      ld OBJ_ANIMATION_LIST(ix),l
      ld OBJ_ANIMATION_LIST+1(ix),h


      ld hl,#_gtaWeaponStepFunc
      ld OBJ_STEP_FUNC(ix),l
      ld OBJ_STEP_FUNC+1(ix),h
      
      

      ld a,OBJ_SELECTED_WEAPON(iy)
      or a
      jr Z,gtaCreaArmas4		//SI EL ARMA ES 0 son los puños, por lo tanto no le agregamos funcion draw, no le hace falta
      
      ld hl,#_vtDrawObjectFunc			//SI ES MAYOR QUE 0, AGREGAMOS LA FUNCION DE DIBUJAR
      ld OBJ_PREPARE_FUNC(ix),l
      ld OBJ_PREPARE_FUNC+1(ix),h
      
     gtaCreaArmas4:
      ld a,OBJ_FRAME_SPEED(iy)
      ld OBJ_FRAME_SPEED(ix),a

      ld OBJ_ANIMATION_MAX(ix),#4
      
      ld a,OBJ_POSX(iy)
      ld OBJ_POSX(ix),a
      ld a,OBJ_POSX+1(iy)
      ld OBJ_POSX+1(ix),a

      ld a,OBJ_POSY(iy)
      inc a
      ld OBJ_POSY(ix),a
      ld a,OBJ_POSY+1(iy)
      ld OBJ_POSY+1(ix),a

      ld a,OBJ_SPEEDX(iy)
      ld OBJ_SPEEDX(ix),a
      ld a,OBJ_SPEEDX+1(iy)
      ld OBJ_SPEEDX+1(ix),a
      
      ld a,OBJ_SPEEDY(iy)
      ld OBJ_SPEEDY(ix),a
      ld a,OBJ_SPEEDY+1(iy)
      ld OBJ_SPEEDY+1(ix),a
      
      ld a,OBJ_PERSON_TORQUE(iy)    //le paso el angulo en el que iba justo antes de disparar
      ld OBJ_PERSON_TORQUE(ix),a

      push iy                    //me guardo quien es el que ha creado el arma para despues no hacerle daño a si mismo en caso de colision
      pop hl
      ld OBJ_PARENT_PTR(ix),l          //meto en el PARENT del POINTER el padre del disparo
      ld OBJ_PARENT_PTR+1(ix),h

      ld a,#2
      ld OBJ_WIDTH(ix),a
      ld OBJ_HEIGHT(ix),a
      ld OBJ_FLAGS(ix),#FLAG_NO_COLISION | FLAG_AUTO_MOVE | FLAG_AUTO_ANIM //| FLAG_FORCE_DRAW


      ld OBJ_IA_LOCK_MOVE(ix),#8       //aprox 2 segundos
      ld OBJ_FRAMES_FOR_NEXT_STATE(ix),#8

      //ahora que ya me la pelan todos los registros ejecuto el sonido
      
      
      ld hl,#_sonidosFXarmas
      ld a,OBJ_SELECTED_WEAPON(iy)
      add l
      jr NC,gtaCreaArmas6
      inc h
   gtaCreaArmas6:
      ld l,a
      ld a,(hl)

      
      call _gtaSoundFX


      ld (#_vtGameObjPtr),iy
      push iy
      pop ix

      


      xor a    //ld a,#0      //tiene que devolver 0 para actualizar el contador de ticks del fuego activo
      ret
   __endasm;
}


//este metodo si es un arma de melee golpea a los que esten cerca y si es un arma de rango crea la bala (objeto TYPE_BULLET)
//entra en IX el objeto WEAPON
void gtaCreaBala() __naked {
   __asm
      //call _trapSPinit
      ld a,OBJ_SELECTED_WEAPON(ix)
      or a 
      jr NZ,gtaCreaBalas2
      
      //si es cero son los puños
      ld bc,#0x0402  //tiles distancia  y  puntos de vida a perder
      ld d,#1       //pixeles sangrientos
      jp gtaCreaBalaHazDanoMelee
      
      
   gtaCreaBalas2:
      dec a
      jr NZ,gtaCreaBalaBate

   gtaCreaBalaNavaja:
      ld bc,#0x0404  //tiles distancia  y  puntos de vida a perder
      ld d,#2       //pixeles sangrientos
      jp gtaCreaBalaHazDanoMelee
      

   gtaCreaBalaBate:
      dec a
      jr NZ,gtaCreaBalaNunchaku
      ld bc,#0x0506  //tiles distancia  y  puntos de vida a perder
      ld d,#3        //pixeles sangrientos
      jp gtaCreaBalaHazDanoMelee

   gtaCreaBalaNunchaku:
      dec a
      jr NZ,gtaCreaBalaKatana
      ld bc,#0x0508  //tiles distancia  y  puntos de vida a perder
      ld d,#4        //pixeles sangrientos
      jp gtaCreaBalaHazDanoMelee

   gtaCreaBalaKatana:
      dec a
      jr NZ,gtaCreaBalaPistola
      ld bc,#0x050A  //tiles distancia  y  puntos de vida a perder
      ld d,#6        //pixeles sangrientos
      jp gtaCreaBalaHazDanoMelee
      






   gtaCreaBalaPistola:
      dec a
      jr NZ,gtaCreaBalaEscopeta
      ld bc,#0x1E0A  //tiles distancia  y  puntos de vida a perder
      ld d,#5
      //ld a,#8        //tengo que ver como calcular el angulo del personaje que dispara
      jp gtaCreaBalaHazDanoDirectoBala

   gtaCreaBalaEscopeta:
      dec a
      jr NZ,gtaCreaBalaGranada

      //aqui hago 3 balas que duran poquito con angulos distintos
      ld a,OBJ_PERSON_TORQUE(ix)
      add #15
      ld OBJ_PERSON_TORQUE(ix),a
      
      push ix

      ld bc,#0x0C0A  //tiles distancia  y  puntos de vida a perder
      call gtaCreaBalaHazDanoDirectoBala        //creo la primera bala de la escopeta

      pop ix
      
      ld a,OBJ_PERSON_TORQUE(iy)
      sub #30
      ld OBJ_PERSON_TORQUE(iy),a

      push ix

      ld bc,#0x0C0A  //tiles distancia  y  puntos de vida a perder
      call gtaCreaBalaHazDanoDirectoBala        //creo la segunda bala de la escopeta

      pop ix


      ld a,OBJ_PERSON_TORQUE(iy)
      add #15
      ld OBJ_PERSON_TORQUE(iy),a

      ld bc,#0x0C0A  //tiles distancia  y  puntos de vida a perder
      jp gtaCreaBalaHazDanoDirectoBala          //creo la tercera bala de la escopeta

      

   gtaCreaBalaGranada:
      dec a
      jr NZ,gtaCreaBalaBazoka
      //TODO (crear granada)
      ld bc,#0x0F15  //tiles distancia  y  puntos de vida a perder
      jp gtaCreaBalaHazDanoDirectoBala
      ret

   gtaCreaBalaBazoka:
      //TODO (crear disparo de bazoka)
      ld bc,#0x0F30  //tiles distancia  y  puntos de vida a perder
      jp gtaCreaBalaHazDanoDirectoBala
      ret








      //si entra aqui va a hacer daño a melee (mete en B la distancia en tiles para que se considere daño, en C los puntos de vida que debe bajar
      //en D numero de pixeles sangrientos que debe aparecer)
      //en A entra el angulo del personaje que ataca
      //el melee por defecto tiene 32 grados de posible error respecto al angulo, o dicho de otra forma, tiene un angulo de colision de 64 (90 grados) en la direccion que nos movamos
   gtaCreaBalaHazDanoMelee:
      ld a,b
      ld (#gtaCreaBalaDistance),a
      ld a,d
      ld (#gtaCreaBalaBlood),a
      ld a,c
      ld (#gtaCreaBalaDamage),a
      cp #8
      jr C,gtaCreaBalaHazDanoMelees2		//si hace 8 de daño o más asustamos a las viejas, sino no
      push ix					//conservo el IX que el asusta viejas se lo carga
		call _gtaAsustaViejas
		pop ix
		
	gtaCreaBalaHazDanoMelees2:

      //vamos a recorrer todos los objetos usando IY
      ld hl,#GAME_OBJECTS_PTR
      ld de,#48
      ld b,#16

   gtaCreaBalaHazDanoMeleeBucle:
      ld a,(hl)
      and #TYPE_CITIZEN|TYPE_CAR|TYPE_PROTA
      jp Z,gtaCreaBalaHazDanoMeleeNext    //miro si esta activo y si ademas es una persona, ya que sino puedo pasar al siguiente del tiron

      
      //antes de hacer ná, miro si la colision es con el mismo creador o no, en ese caso no debo hacer nada
      push bc
      ld c,OBJ_PARENT_PTR(ix)          //meto en el PARENT del POINTER el padre del disparo
      ld b,OBJ_PARENT_PTR+1(ix)
      push hl
      xor a
      sbc hl,bc
      pop hl
      pop bc
      jp Z,gtaCreaBalaHazDanoMeleeNext
      
      


      push hl
      exx
      pop iy

      

      ld l,OBJ_POSX(iy)
      ld h,OBJ_POSX+1(iy)
      ld e,OBJ_POSX(ix)
      ld d,OBJ_POSX+1(ix)     
      xor a 
      sbc hl,de
      ld (#gtaCreaBalaSaveX),hl
      ld b,h
      ld c,l
      

      ld l,OBJ_POSY(iy)
      ld h,OBJ_POSY+1(iy)
      ld e,OBJ_POSY(ix)
      ld d,OBJ_POSY+1(ix)
      xor a
      sbc hl,de
      ld (#gtaCreaBalaSaveY),hl
      ex de,hl
      
      call _vtDistance16
      
      ld a,(#gtaCreaBalaDistance)
      ld b,a

      ld a,h
      cp b
      //si resto el numero de tiles maximo y hay Carry es que estamos cerca
      jr NC,gtaCreaBalaHazDanoMeleeNextPre         //si no hay carry no hay colision con este objeto, pasamos al siguiente
      
      //ahora que se que esta cerca, tengo que mirar si el personaje que ataca esta más o menos mirando al personaje que recibe
      ld bc,(#gtaCreaBalaSaveX)
      ld de,(#gtaCreaBalaSaveY)
      call _vtAtan2_16     //ponemos en A el angulo entre el personaje que golpea y el que recibe

      ld b,a
      ld a,OBJ_PERSON_TORQUE(ix)
      sub b       //con esta resta tengo la diferencia de angulo entre el angulo hacia el que apunto y el angulo donde esta el objeto

      //si este numero está entre -32 y 32 le he dado, sino no
      //o dicho de otra forma, si le sumo 32 y es menor de 64 le he dado
      add #32
      cp #64
      jr NC,gtaCreaBalaHazDanoMeleeNextPre   //si es mayor que 64 y no hay carry en la resta es que esta mirando para otro lado y no le he dado

      //si llega aqui, esta cerca y ademas le hemos dado, podemos quitarle la vida y ademas dejar las manchas de sangre
      ld a,(#gtaCreaBalaDamage)
      ld b,a
      ld a,OBJ_LIFE(iy)
      sub b
      ld OBJ_LIFE(iy),a    //esto puede quedar como un numero negativo, en ese caso el propio objeto debe matarse a si mismo

      bit 7,a
      jr Z,gtaCreaBalaHazDanoMeleeBucles2

      //si llegamos a la vida 0 es que lo hemos matado, subimos el nivel de busqueda de la poli (no hace falta mirar quien ha matao a quien, porke si nos matan a nosotros el juego se acaba de todas formas)
      ld hl,#_hateLevel+1
      inc (hl)

   gtaCreaBalaHazDanoMeleeBucles2:
      //call _gtaCreaHit
      set 3,OBJ_FLAGS(iy)     //le activamos el flag de flash sprite para dar feedback durante 1 frame
      
      call gtaCreaSangre

      
      //si le he zumbao a melee, le cambio la IA por huyendo o cabreao (pero solo si es un CITIZEN sino no)
      ld a,OBJ_ACTIVE_TYPE(iy)
      and #TYPE_CITIZEN
      jr Z,gtaCreaBalaHazDanoMeleeNextPre		//no es un ciudadano, pasando palante
      
      bit 6,OBJ_FLAGS(iy)     //si tiene el FLAG_NO_DESTROY no lo asustamos 
      jr NZ,gtaCreaBalaHazDanoMeleeNextPre

      //antes de asustarlo o cabrearlo, miramos que no sea un NPC concreto (en este caso no hay que hacerle nada a él)
      //ld a,OBJ_ID_NPC_CAR(iy)
      //or a 
      //jr NZ,gtaCreaBalaHazDanoMeleeNextPre

      
      call cpct_getRandom_lcg_u8_asm
	 	cp #VALOR_PERSONAS_IA
     	jr C,gtaCreaBalaHazDanoMeleeHuye		//entre 0 y 210 huye, entre 210 y 255 ataca
     	
   gtaCreaBalaHazDanoMeleeAtaca:
     	//si entra aqui es una persona, le cambiamos la IA a asustado huyendo (de momento solo huyendo, luego segun numero random puede enfadarse y pegarnos)
     	ld hl,#_iaPersonAtacandoProta
	   jr gtaCreaBalaHazDanoMeleeCambiaIA
     	
   gtaCreaBalaHazDanoMeleeHuye:
     	//si entra aqui es una persona, le cambiamos la IA a asustado huyendo (de momento solo huyendo, luego segun numero random puede enfadarse y pegarnos)
     	ld hl,#_iaPersonHuyendo
     	
     	
   gtaCreaBalaHazDanoMeleeCambiaIA:
	   ld OBJ_MACHINE_STATE_PTR(iy),l
    	ld OBJ_MACHINE_STATE_PTR+1(iy),h
		ld OBJ_FRAMES_FOR_NEXT_STATE(iy),#127
      

   gtaCreaBalaHazDanoMeleeNextPre:
      exx

   gtaCreaBalaHazDanoMeleeNext:
      add hl,de      //paso al siguiente
      dec b
      jp NZ,gtaCreaBalaHazDanoMeleeBucle
      //djnz gtaCreaBalaHazDanoMeleeBucle

      //call _trapSPend
      ret








      //si entra aqui va a hacer daño directo por bala, osea en el mismo frame va a detectar si esta en un angulo determinado y a una distancia determinada y 
      //si esta le quita la vida indicada
      //B distancia en tiles maxima para que cuente el daño,   
      //C puntos de vida a quitar
      //E error maximo de angulo para que considere que ha dado en el blanco
      //D numero de pixeles sangrientos que debe aparecer
   gtaCreaBalaHazDanoDirectoBala:
      
      ld a,b
      ld (#gtaCreaBalaDistance),a
      ld a,c
      ld (#gtaCreaBalaDamage),a
      //ld a,d
      //ld (#gtaCreaBalaBlood),a

      //cualquier disparo asusta a las viejas
		push ix					//conservo el IX que el asusta viejas se lo carga
		call _gtaAsustaViejas
		pop ix


      //mejor vamos a crear una entidad BALA para probar que es mejor
      push ix
      pop iy      //me dejo en IY el objeto weapon para poder sacar los datos

      ld hl,(#_vtGameObjPtr)  //me guardo el vtGameObjPtr
      push hl

      call _vtGetFreeGameObjPtr        //preparamos el objeto del prota
      ld ix,(#_vtGameObjPtr)
      
      pop hl
      ld (#_vtGameObjPtr),hl  //recupero el vtGameObjPtr

      // SAFE_GET_FREE
      //xor a
      //.dw #0xB4DD    //or ixh      compruebo si IXH es cero, porque eso significa que no tengo huecos libres para crear el objeto, con lo cual salgo del tiron

      //call _trapSPend
      //ret Z      //si no tenemos hueco para crearla nos piramos
      


      ld OBJ_ACTIVE_TYPE(ix),#TYPE_AUX_OBJECT

      ld l,OBJ_PARENT_PTR(iy)       //saco el objeto que disparo para ponerselo como padre a la bala, de esta forma sabremos a quien NO tiene que hacer daño en ningun caso
      ld h,OBJ_PARENT_PTR+1(iy)
      ld OBJ_PARENT_PTR(ix),l 
      ld OBJ_PARENT_PTR+1(ix),h

      ld hl,#_gtaBulletStepFunc         //el step y el draw no necesitan nada especial, simplemente se dibuja y se mueve
      ld OBJ_STEP_FUNC(ix),l
      ld OBJ_STEP_FUNC+1(ix),h

      ld hl,#_vtDrawObjectFunc
      ld OBJ_PREPARE_FUNC(ix),l
      ld OBJ_PREPARE_FUNC+1(ix),h

      ld hl,#_gtaDisparoColisionFunc      //la colision si necesita un toke especial porque tiene que comprobar varias cositas
      ld OBJ_COLISION_FUNC(ix),l
      ld OBJ_COLISION_FUNC+1(ix),h
      

      ld a,(#gtaCreaBalaDistance)
      ld OBJ_WEAPON_DISTANCE(ix),a
      
      ld a,(#gtaCreaBalaDamage)
      ld OBJ_WEAPON_DAMAGE(ix),a    //la sangre la podemos sacar de forma aproximada del DAMAGE
      
      



      ld hl,#_animDisparoSimple           //ponemos la bala en la animacion
      ld a,OBJ_SELECTED_WEAPON(iy)
      ld OBJ_SELECTED_WEAPON(ix),a		//lo guardo de paso
      cp #8
      jr NZ,gtaCreaBalaHazDanoDirectoBalas1

      ld hl,#_animDisparoPepinazo           //ponemos la bala en la animacion

   gtaCreaBalaHazDanoDirectoBalas1:
      cp #7
      jr NZ,gtaCreaBalaHazDanoDirectoBalas2

      ld hl,#_animDisparoGranada           //ponemos la bala en la animacion

   gtaCreaBalaHazDanoDirectoBalas2:

      ld OBJ_ANIMATION_LIST(ix),l
      ld OBJ_ANIMATION_LIST+1(ix),h

      ld a,OBJ_POSX(iy)                //la posicion la misma que el arma
      ld OBJ_POSX(ix),a
      ld a,OBJ_POSX+1(iy)
      inc a
      ld OBJ_POSX+1(ix),a

      ld a,OBJ_POSY(iy)
      ld OBJ_POSY(ix),a
      ld a,OBJ_POSY+1(iy)
      inc a
      ld OBJ_POSY+1(ix),a

      
      ld c,#1
      ld b,#150
      ld a,OBJ_PERSON_TORQUE(iy)
      call _vtAccelToSpeedAng

      //ld l,OBJ_SPEEDX(ix)        //para que no vaya muy lento le damos caña a la velocidad
      //ld h,OBJ_SPEEDX+1(ix)
      //add hl,hl
      //add hl,hl
      //add hl,hl
      //add hl,hl
      //ld OBJ_SPEEDX(ix),l        //para que no vaya muy lento le damos caña a la velocidad
      //ld OBJ_SPEEDX+1(ix),h


      //ld l,OBJ_SPEEDY(ix)        //para que no vaya muy lento le damos caña a la velocidad
      //ld h,OBJ_SPEEDY+1(ix)
      //add hl,hl
      //add hl,hl
      //add hl,hl
      //add hl,hl
      //ld OBJ_SPEEDY(ix),l        //para que no vaya muy lento le damos caña a la velocidad
      //ld OBJ_SPEEDY+1(ix),h




      ld b,#2
   gtaCreaBalaHazDanoDirectoBalas3:
      ld l,OBJ_POSX(ix)				//posX=posX+speedX
      ld h,OBJ_POSX+1(ix)
      ld e,OBJ_SPEEDX(ix)
      ld d,OBJ_SPEEDX+1(ix)
      add hl,de
      ld OBJ_POSX(ix),l
      ld OBJ_POSX+1(ix),h
      
      ld l,OBJ_POSY(ix)				//posY=posY+speedY
      ld h,OBJ_POSY+1(ix)
      ld e,OBJ_SPEEDY(ix)
      ld d,OBJ_SPEEDY+1(ix)
      add hl,de
      ld OBJ_POSY(ix),l
      ld OBJ_POSY+1(ix),h
      djnz gtaCreaBalaHazDanoDirectoBalas3


      ld a,#1                       //el sprite solo tiene 1 tile, no ocupa más
      ld OBJ_WIDTH(ix),a
      ld OBJ_HEIGHT(ix),a
      ld OBJ_FLAGS(ix),#FLAG_AUTO_MOVE | FLAG_AUTO_ANIM //| FLAG_FORCE_DRAW     //FLAG_AUTO_MOVE  FLAG_AUTO_ANIM

      xor a 
      ld OBJ_ANIMATION_MAX(ix),a
      ld OBJ_FRAME_INDEX(ix),a
      ld OBJ_FRAME_INDEX+1(ix),a
      ld OBJ_FRAME_SPEED(ix),a

      
      //call _trapSPend
      ret




         //se puede llamar desde fuera (IX es el arma que hace el daño, y IY es el personaje al que se le da)
   gtaCreaSangre:

      //antes de crear sangre, vemos si al objeto que le hemos dado es una persona o un coche (los coches no suelen sangrar, normalmente...)
      bit 2,OBJ_ACTIVE_TYPE(iy) 
      ret Z      //si es 0 es que es algo diferente a un ciudadano 

      
      ld h,OBJ_POSX+1(ix)
      ld l,OBJ_POSX(ix)
      inc h
      ld (#gtaCreaBalaSaveX),hl

      ld h,OBJ_POSY+1(ix)
      ld l,OBJ_POSY(ix)
      inc h
      ld (#gtaCreaBalaSaveY),hl


      call _vtAngleIXtoIY
      ld (#gtaCreaBalaAngle),a      //calculo el angulo hacia donde tiene que ir la sangre

      ld a,(#gtaCreaBalaBlood)      //cantidad de sangre a sacar
      ld b,a


   gtaCreaSangreBucle:

      push bc     //me guardo B que tengo cuanta sangre tiene que salpicar

      call _cpct_getRandom_lcg_u8      //y ahora calculo la desviación del angulo original para tener el angulo final en el que salpicara la sangre
      and #31
      sub #16
      ld b,a
      ld a,(#gtaCreaBalaAngle)
      add b
      ld (#gtaCreaBalaDamage),a     //me guardo en Damage (que ya no me hace falta para quitar vida) el angulo final en el que saldrá esta gota de sangre

      call _cpct_getRandom_lcg_u8      //y ahora calculo la distancia a la que va a salpicar la sangre
      and #31
      add #32
      ld (#gtaCreaBalaDistance),a          

      

      //una vez tengo los dos calculados, los aplico
      ld b,a
      ld a,(#gtaCreaBalaDamage)

      call _vtCosAdvance      //A pasamos el angulo (256 based), en B pasamos la velocidad, y devuelve en DE=cos(A)*B
      //pop bc

      ld d,e
      ld e,#0
      sra d    //divido entre 2
      rr e
      sra d    //divido entre 2
      rr e
      sra d    //divido entre 2
      rr e
      ld (#gtaCreaBalaAddX),de

   
      //push bc
      
      ld a,(#gtaCreaBalaDistance)
      ld b,a
      ld a,(#gtaCreaBalaDamage)
      
      call _vtSinAdvance      //A pasamos el angulo (256 based), en B pasamos la velocidad, y devuelve en DE=cos(A)*B
      
      ld d,e
      ld e,#0
      sra d    //divido entre 2
      rr e
      sra d    //divido entre 2
      rr e
      sra d    //divido entre 2
      rr e
      ld (#gtaCreaBalaAddY),de
      

   gtaCreaSangreBucleFin:
      ld hl,(#gtaCreaBalaSaveX)
      ld de,(#gtaCreaBalaAddX)
      add hl,de
      ld b,h
      ld c,l
      ld hl,(#gtaCreaBalaSaveY)
      ld de,(#gtaCreaBalaAddY)
      add hl,de
      ex de,hl
      
      call _vtGetGameScreenPtrBCDE        //esto me calcula la direccion de memoria de pantalla    

      ld a,#CUSTOM_A_PAL_9|CUSTOM_B_PAL_9
      ld (hl),a
      
      pop bc

      djnz gtaCreaSangreBucle

      ret


   gtaCreaBalaSaveX:
      .dw #0
   gtaCreaBalaSaveY:
      .dw #0

   gtaCreaBalaDistance:
      .db #0
   gtaCreaBalaDamage:
      .db #0
   gtaCreaBalaBlood:
      .db #0
   gtaCreaBalaAngle:
      .db #0
   
   gtaCreaBalaAddX:
      .dw #0
   gtaCreaBalaAddY:
      .dw #0


   __endasm;
}



void gtaBulletStepFunc() __naked {
   __asm
      call _vtStepObjectFunc

		ld a,OBJ_SELECTED_WEAPON(ix)
		sub #7
		jr NZ,gtaBulletStepFuncs2
		//es granada 
		
		ld b,OBJ_POSX+1(ix)
		ld c,OBJ_POSY+1(ix)
		inc c 
		inc c
		
		bit 7,OBJ_SPEEDX+1(ix)
		jr NZ,gtaBulletStepFuncs0
		
		inc b
		inc b
		
	gtaBulletStepFuncs0:
		call _vtCheckTileAsm
		
		ld a,#64
		cp l
		jr NC,gtaBulletStepFuncs0b
		
		//le cambio la velocidad horizontal
		ld hl,#0
		ld d,OBJ_SPEEDX+1(ix)
		ld e,OBJ_SPEEDX(ix)
		xor a
		sbc hl,de
		ld OBJ_SPEEDX+1(ix),h
		ld OBJ_SPEEDX(ix),l
		
		
		
	gtaBulletStepFuncs0b:
		ld b,OBJ_POSX+1(ix)
		inc b
		ld c,OBJ_POSY+1(ix)
		inc c 

		
		bit 7,OBJ_SPEEDY+1(ix)
		jr NZ,gtaBulletStepFuncs1
		inc c
		inc c
		
	gtaBulletStepFuncs1:
		call _vtCheckTileAsm
		
		ld a,#64
		cp l
		jr NC,gtaBulletStepFuncs2
		
		//le cambio la velocidad vertical
		ld hl,#0
		ld d,OBJ_SPEEDY+1(ix)
		ld e,OBJ_SPEEDY(ix)
		xor a
		sbc hl,de
		ld OBJ_SPEEDY+1(ix),h
		ld OBJ_SPEEDY(ix),l
		jr gtaBulletStepFuncs3
		
		
		
		
		
	gtaBulletStepFuncs2:
		dec a
		jr NZ,gtaBulletStepFuncs3
		//es bazoka si colisiono con cualquier cosa exploto del tiron
		
		call _checkTilePerson
      ld a,#64
      cp l
		jr NC,gtaBulletStepFuncs3		//si no hay carry es que L < 64 y no hay problema, que tire palante
		
		
		//si entra aqui tiene que petar
	gtaBulletStepFuncExplode:
		call explotaObjeto		//esto transforma el objeto en una explosion y vuelve
		
		//ahora que ya es una explosion tendriamos que hacer daño a todo lo que este cerca TODO KABESA
		
		ret
		
		
		
	gtaBulletStepFuncs3:

      ld ix,(#_vtGameObjPtr)
      dec OBJ_WEAPON_DISTANCE(ix)
      ret NZ
      
      ld OBJ_ACTIVE_TYPE(ix),#0     //si ya hemos recorrido la distancia desactivamos la instancia
      
      ld a,OBJ_SELECTED_WEAPON(ix)
      cp #7
      call NC,explotaObjeto		//si es 7 u 8 que explote porque es una granada o un bazoka
      
      ret
   __endasm;
}

/*
void gtaWeaponDrawFunc() __naked {
   __asm
      ld ix,(#_vtGameObjPtr)

      ld a,OBJ_FLAGS(ix)
      bit 1,a        //compruebo si hay que dibujar el sprite o no
      jr Z,gtaWeaponDrawFuncEnd      //no hay que dibujar el sprite, asi que nos piramos

      //calcula las variables donde se tiene que dibujar el sprite y de paso para restaurar el fondo en el proximo frame
      call _vtGetGameObjScreenPtr
      //agrega a la lista de sprites para dibujar
      call _vtPushSpriteToDraw
      

   gtaWeaponDrawFuncEnd:
      ld a,OBJ_FLAGS(ix)
      and #0b11111100
      ld OBJ_FLAGS(ix),a
      ret
   
   __endasm;
}
*/


void gtaWeaponStepFunc() __naked {
   __asm
      
      ld ix,(#_vtGameObjPtr)

      dec OBJ_FRAMES_FOR_NEXT_STATE(ix)         //le bajo uno a el temporizador de la ia y lo cargo en A
      //ld a,OBJ_FRAMES_FOR_NEXT_STATE(ix)        //la cargo en A para que cuando llame a la funcion de la IA haga el cambio si es 0
      //ld a,OBJ_FRAMES_FOR_NEXT_STATE(ix)
      //dec a
      jr NZ,gtaWeaponStepFuncs2
      
      
      //si entramos aqui podemos eliminar ya este objeto
      ld OBJ_WIDTH(ix),#4     //pongo 3 para que se borre bien
      dec OBJ_REDRAWX(ix)     //le resto 1 para que lo borre bien si la velocidad es negativa
      call _vtStepObjectFunc     //finalmente llamo al generico para eliminar el fondo
      ld ix,(#_vtGameObjPtr)
      push ix

      //y por ultimo creo la BALA (si es melee simplemente hara el daño directamente, y si es rango creará el objeto bullet)
      call _gtaCreaBala
      pop ix 
      ld OBJ_ACTIVE_TYPE(ix),#0     //y luego lo desactivo el objeto para que no llame al draw y lo pinte

      ret


   gtaWeaponStepFuncs2:
      //ld OBJ_FRAMES_FOR_NEXT_STATE(ix),a

      //quizas se pueda hacer un jump directo
      call _vtStepObjectFunc     //finalmente llamo al generico
      ret
   __endasm;
}

//---------------------------------------------------  BLOQUE DE FUNCIONES DE IA -------------------------------------------------
//42-47  
//u8* iaMachineStatePtr;         //puntero a la funcion de la maquina de estados que controlara la IA de este personaje
//u8 iaTimeToNextState;          //numero de frames para calcular el siguiente estado
//u8 iaState3;                   //3 variables auxiliares para controlar ciertos elementos de la IA (cada cual que la use para lo que le haga falta)
//u8 iaState2;
//u8 iaState1;


//se espera en el sitio mirando a izq y derecha parado un ratito, luego pasa a paseando
void iaPersonRelajado() __naked {
   __asm
      //en ix va el jugador (y si tuviera coche va en iy)
      or a
      jr NZ,iaPersonRelajados2

      ld hl,#_iaPersonPaseando      //si llega a cero cambiamos a paseando
      ld OBJ_MACHINE_STATE_PTR(ix),l
      ld OBJ_MACHINE_STATE_PTR+1(ix),h
      ld	l, #0x00
	   call _cpct_getRandom_lcg_u8
      ld OBJ_FRAMES_FOR_NEXT_STATE(ix),l
      xor a       //pongo el speedX a 0 para que en el paseando calcule donde tiene que ir
      ld OBJ_SPEEDX(ix),a
      ld OBJ_SPEEDX+1(ix),a
      ret

   iaPersonRelajados2:
      //si no lo unico que tengo que hacer es cambiar la velocidad a +1 o -1 por el tiempo simplemente
      bit 2,a
      jr Z,iaPersonRelajados3
      //lo pongo mirando pa la derecha
      xor a
      ld OBJ_SPEEDX+1(ix),a
      inc a
      ld OBJ_SPEEDX(ix),a
      jr iaPersonRelajados4

   iaPersonRelajados3:
      //lo pongo mirando pa la izquierda
      ld a,#255
      ld OBJ_SPEEDX(ix),a
      ld OBJ_SPEEDX+1(ix),a

   iaPersonRelajados4:
      xor a
      ld OBJ_SPEEDY(ix),a
      ld OBJ_SPEEDY+1(ix),a
      ld OBJ_FRAME_SPEED(ix),a
      ret

   __endasm;
}


void iaPersonSiguiendoPunto() __naked {
   __asm
      //no hago nada, la animacion y la velocidad la marcan la funcion gtaMisionNPCWalkPoint cuando llegue al punto cambiara a iaPersonRelajado
      ret
   __endasm;
}

//se pone a andar despacio por la acera, cuando llegue a una esquina se para a pensar si cruzar la calle o cambiar de direccion
void iaPersonPaseando() __naked {
   __asm
      //en ix va el jugador (y si tuviera coche va en iy)
      or a
      jr NZ,iaPersonPaseandos2

      ld	l, #0x00
	   call _cpct_getRandom_lcg_u8
      bit 7,a
      jr Z,iaPersonPaseandoCambiaRelajado    //si es menor de 128 lo cambiamos a relajado un ratin, sino seguimos paseando
      ld OBJ_FRAMES_FOR_NEXT_STATE(ix),a
      ret

   iaPersonPaseandoCambiaRelajado:
      and #15
      ld OBJ_FRAMES_FOR_NEXT_STATE(ix),a
      ld hl,#_iaPersonRelajado     //si llega a cero de momento para probar voy a volver a ponerlo parado
      ld OBJ_MACHINE_STATE_PTR(ix),l
      ld OBJ_MACHINE_STATE_PTR+1(ix),h
      ret

   iaPersonPaseandos2:
      //si la velocidad es 0, le pongo que empiece a andar hacia alguno de los 4 lados
      ld b,a      //me guardo en B el next_frame de la maquina de estado
      
      xor a
      //ld l,OBJ_SPEEDX(ix)
      //ld h,OBJ_SPEEDX+1(ix)
      //ld e,OBJ_SPEEDY(ix)
      //ld d,OBJ_SPEEDY+1(ix)
      //sbc hl,de
		//jr NZ,iaPersonPaseandos8
		
		or OBJ_SPEEDX(ix)
		jr NZ,iaPersonPaseandos8	//si OBJ_SPEEDX es != 0 es que ya está andando a algun sitio
		or OBJ_SPEEDY(ix)		
		jr NZ,iaPersonPaseandos8	//si OBJ_SPEEDY es != 0 es que ya está andando a algun sitio
      
      //si llego aqui es que el muñeco esta parado, voy a calcular hacia donde tiene que andar para que siga por la acera y no se vaya de madre...

      //si tengo que cambiar de dirección me ajusto al tile
      call redondeaAlTile


      //si es cero vamos a poner que vaya hacia algun sitio random, para ello vamos a pillar el next_frame que ya es un numero random
      //pongo primero la velocidad de animacion 
      ld OBJ_FRAME_SPEED(ix),#96

      bit 1,b
      jr Z,iaPersonPaseandos6
      //nos movemos en horizontal pa onde sea
      bit 0,b
      jr Z,iaPersonPaseandos3

      
      

      ld OBJ_SPEEDX+1(ix),#0xFF
      ld OBJ_SPEEDX(ix),#VELOCIDAD_IA_PASEANDO_NEG  //-48
      jr iaPersonPaseandos8
      //ret
   iaPersonPaseandos3:
      ld OBJ_SPEEDX+1(ix),#0
      ld OBJ_SPEEDX(ix),#VELOCIDAD_IA_PASEANDO
      jr iaPersonPaseandos8
      //ret


   iaPersonPaseandos6:
      //nos movemos en vertical pa onde sea

      

      bit 0,b
      jr Z,iaPersonPaseandos7
      ld OBJ_SPEEDY+1(ix),#0xFF
      ld OBJ_SPEEDY(ix),#VELOCIDAD_IA_PASEANDO_NEG  //-48
      jr iaPersonPaseandos8
      //ret
   iaPersonPaseandos7:
      ld OBJ_SPEEDY+1(ix),#0
      ld OBJ_SPEEDY(ix),#VELOCIDAD_IA_PASEANDO
      jr iaPersonPaseandos8
      //ret


   iaPersonPaseandos8:        //chequeamos si podemos seguir andando paya
      call _iaPersonCheckCanWalk
      or a
      ret NZ

      

      //cuando llegue aqui ya me aseguro que estoy alineado al tile más cercano tanto de X como de Y, de esa forma creo que no se saldrán de las aceras
      call redondeaAlTile


      xor a          //sigo con el personaje parado para que en el siguiente frame elija otro lado para andar, ya que este camino no lo puedo pillar
      ld OBJ_FRAME_SPEED(ix),a      
      ld OBJ_SPEEDX(ix),a
      ld OBJ_SPEEDX+1(ix),a
      ld OBJ_SPEEDY(ix),a
      ld OBJ_SPEEDY+1(ix),a
      ld OBJ_FRAMES_FOR_NEXT_STATE(ix),#1
      ret


   redondeaAlTile:
      //si A = 0 es que no puedo andar en esa dirección y por lo tanto vuelvo a parar el personaje
      //antes de poner todo a cero para que se pare, ajusto un poco la posicion, lo que voy a hacer es ajustarlo al tile más cercano (redondeo al tile)
      ld a,OBJ_POSX(ix)
      bit 7,a
      jr Z,iaPersonPaseandosRedonX1
      //si el bit 7 es 1, tengo que sumarle 1 a OBJ_POSX+1
      inc OBJ_POSX+1(ix)
   iaPersonPaseandosRedonX1:
      ld OBJ_POSX(ix),#0

      //ahora redondeamos Y
      ld a,OBJ_POSY(ix)
      bit 7,a
      jr Z,iaPersonPaseandosRedonY1
      //si el bit 7 es 1, tengo que sumarle 1 a OBJ_POSY+1
      inc OBJ_POSY+1(ix)
   iaPersonPaseandosRedonY1:
      ld OBJ_POSY(ix),#0
      ret







   vuelveAlTileAnterior:
      //miro la velocidad y lo echo patras hasta que este en un tile que no tenga colision
      ld a,OBJ_SPEEDX+1(ix)
      bit 7,a
      jr NZ,vuelveAlTileAnteriors2
   vuelveAlTileAnteriors1:
      //es positivo la velocidad, por lo tanto tengo que restarle 1 a posX
      dec OBJ_POSX+1(ix)
      jr vuelveAlTileAnteriors3
   vuelveAlTileAnteriors2:
      //es negativa la velocidad, por lo tanto tengo que sumarle 1
      inc OBJ_POSX+1(ix)
      

   vuelveAlTileAnteriors3:
      ld a,OBJ_SPEEDY+1(ix)
      bit 7,a
      jr NZ,vuelveAlTileAnteriors5
   vuelveAlTileAnteriors4:
      //es positivo la velocidad, por lo tanto tengo que restarle 1 a posX
      dec OBJ_POSY+1(ix)
      jr vuelveAlTileAnteriors6
   vuelveAlTileAnteriors5:
      //es negativa la velocidad, por lo tanto tengo que sumarle 1
      inc OBJ_POSY+1(ix)

   vuelveAlTileAnteriors6:
      call _checkTilePerson
      ld a,#64
      cp l
      jr C,vuelveAlTileAnterior     //sigo estando en un lugar que hay colision, vuelvo a mover a la persona hasta que este en un lugar que no haya colision
      ret



   __endasm;
}


//cuando se queda perdido en medio de la carretera, busca la acera más cercana y va rapido hacia alla (igual que el Paseando, pero primero busca una acera y luego va el doble de rapido)
//vamos a usar los sentidos de la carretera para saber donde esta la acera más cercana, de forma muy simple
//sentido izq (acera arriba), sentido abajo (acera izquierda), sentido derecha (acera abajo), sentido arriba (acera derecha)
//doble sentido horizontal (acera arriba), doble sentido vertical (acera izquierda)
//resto de casos ¿ni puta idea? ya veré...
//el algoritmo debe buscar la acera y tirar paya, en cuanto llegue tiene que pasar a estado parado (ya está a salvo de los coches)
void iaPersonBuscandoAcera() __naked {
   __asm
      //en ix va el jugador (y si tuviera coche va en iy)
      or a
      jr NZ,iaPersonBuscandoAceras2

      //miro el sentido de la carretera
      ld b,OBJ_POSX+1(ix)
		inc b					//le sumo 1 para ponerlo en el medio del coche aprox
		ld c,OBJ_POSY+1(ix)
		inc c					//le sumo 1 para ponerlo en el medio del coche aprox
		call _vtCheckMetaTileAsm
      ld a,h
      
      ld hl,#_metatileTraficDirection
      add l
      jr NC,iaPersonBuscandoAceras1
      inc h
   iaPersonBuscandoAceras1:
      ld l,a
      ld a,(hl)         //miro hacia donde va el trafico en el metatile donde me encuentro

      ld de,#0    //voy a guardar en D la velocidad horizontal y en E la velocidad vertical 

      or a
      jr NZ,iaPersonBuscandoAceras1b
      //trafico a derecha, acera abajo
      ld e,#48
      jr iaPersonBuscandoAceras1go

   iaPersonBuscandoAceras1b:
      dec a 
      jr NZ,iaPersonBuscandoAceras1c
      //trafico abajo, acera izquierda
      ld d,#0xD0     //-48
      jr iaPersonBuscandoAceras1go

   iaPersonBuscandoAceras1c:
      dec a 
      jr NZ,iaPersonBuscandoAceras1d
      //trafico izquierda, acera arriba
      ld e,#0xD0     //-48
      jr iaPersonBuscandoAceras1go

   iaPersonBuscandoAceras1d:
	   dec a 
      jr NZ,iaPersonBuscandoAceras1e
      //trafico arriba, acera derecha
      ld d,#48     
      jr iaPersonBuscandoAceras1go

   iaPersonBuscandoAceras1e:
      dec a    //esto seria el 4 que es seguir en la misma dirección, asi que no se para donde ir 
      dec a    //esto seria el 5 horizontal mitad y mitad
      jr NZ,iaPersonBuscandoAceras1f
      //trafico horizontal mitad y mitad, acera arriba y abajo
      ld e,#0xD0     //-48
      jr iaPersonBuscandoAceras1go

   iaPersonBuscandoAceras1f:
      dec a    //esto es debajo del puente no debo estar aqui nunca
      dec a      //esto es vertical mitad y mitad
      jr NZ,iaPersonBuscandoAceras1g
      //trafico vertical mitad y mitad, acera izquierda y derecha
	   ld d,#0xD0     //-48
      jr iaPersonBuscandoAceras1go

   iaPersonBuscandoAceras1g:
      //si llego aqui no tengo ni puta idea a donde ir, ¿¿ entro en panico ??
      ld d,#48
      ld e,#48


   iaPersonBuscandoAceras1go:
      ld OBJ_SPEEDX(ix),d         
      ld OBJ_SPEEDY(ix),e
      
      ld OBJ_SPEEDX+1(ix),#0
      bit 7,d
      jr Z,iaPersonBuscandoAceras1go2
      ld OBJ_SPEEDX+1(ix),#0xFF

   iaPersonBuscandoAceras1go2:
      ld OBJ_SPEEDY+1(ix),#0
      bit 7,e
      jr Z,iaPersonBuscandoAceras1go3
      ld OBJ_SPEEDY+1(ix),#0xFF
   iaPersonBuscandoAceras1go3:
      ld OBJ_FRAMES_FOR_NEXT_STATE(ix),#240     //le damos tiempo para que llegue a la acera

      ld OBJ_FRAME_SPEED(ix),#96
      ret



   iaPersonBuscandoAceras2:

      //aqui hay que meter que si el tile en el que esta tiene colision que vaya pal otro lao o algo asi
      call _checkTilePerson
      ld a,#64
      cp l
      jr NC,iaPersonBuscandoAceras3     //lo normal es que L sea menor de 64 y 64-L no de carry, sino habria colision y tendriamos que por ejemplo, ir hacia el lado contrario

      call vuelveAlTileAnterior     //esto hace que se vaya un poco hacia atras antes de cambiarle la velocidad para que se coloque en un tile correcto donde pueda estar

      ld e,OBJ_SPEEDX(ix)
      ld d,OBJ_SPEEDX+1(ix)
      ld hl,#0
      xor a
      sbc hl,de
      ld OBJ_SPEEDX(ix),l
      ld OBJ_SPEEDX+1(ix),h

      ld e,OBJ_SPEEDY(ix)
      ld d,OBJ_SPEEDY+1(ix)
      ld hl,#0
      xor a
      sbc hl,de
      ld OBJ_SPEEDY(ix),l
      ld OBJ_SPEEDY+1(ix),h
      ret


   iaPersonBuscandoAceras3:
      call _iaPersonCheckCanWalk
      or a
      ret Z    //si es cero es que sigo en la carretera, no quiero cambiar nada porque ya estoy andando hacia la acera

      //si no es cero, he llegado a un punto donde puedo andar, asi que cambio la IA a parado

      //cuando llegue aqui ya me aseguro que estoy alineado al tile más cercano tanto de X como de Y, de esa forma creo que no se saldrán de las aceras
      call redondeaAlTile

      xor a          //sigo con el personaje parado para que en el siguiente frame elija otro lado para andar, ya que este camino no lo puedo pillar
      ld OBJ_FRAME_SPEED(ix),a      
      ld OBJ_SPEEDX(ix),a
      ld OBJ_SPEEDX+1(ix),a
      ld OBJ_SPEEDY(ix),a
      ld OBJ_SPEEDY+1(ix),a
      ld OBJ_FRAMES_FOR_NEXT_STATE(ix),#15
      ld hl,#_iaPersonRelajado     //si llega a cero de momento para probar voy a volver a ponerlo parado
      ld OBJ_MACHINE_STATE_PTR(ix),l
      ld OBJ_MACHINE_STATE_PTR+1(ix),h
      ret

   __endasm;
}



void iaPersonEmpujada() __naked {
   __asm
      //en ix va el jugador (y si tuviera coche va en iy)
      or a
      jr NZ,iaPersonEmpujadas2


   iaPersonEmpujadaCambiaRelajado:
      jr iaPersonEmpujadas3      //vamos a salir buscando la acera más cercana



   iaPersonEmpujadas2:      //chequeamos si podemos seguir cayendo hacia donde va
      //cambiamos la ani a cayendo (fall)
      
      call _gtaSetAnimFallIX

      call _checkTilePerson
      ld a,#64
      cp l
      ret NC      //si no hay carry es que el tile L<65 y por lo tanto puedo seguir cayendo en esa dirección (aceras, carreteras y tiles sin colision), sino tengo que ver donde voy
      

      //cuando llegue aqui ya me aseguro que estoy alineado al tile más cercano tanto de X como de Y, de esa forma creo que no se saldrán de las aceras
      //call redondeaAlTile     //el redondea al tile esta en el paseando pero se puede usar aki igualmente
      call vuelveAlTileAnterior


   iaPersonEmpujadas3:
      xor a          //sigo con el personaje parado para que en el siguiente frame elija otro lado para andar, ya que este camino no lo puedo pillar
      ld OBJ_SPEEDX(ix),a
      ld OBJ_SPEEDX+1(ix),a
      ld OBJ_SPEEDY(ix),a
      ld OBJ_SPEEDY+1(ix),a

      
      //cambiamos la ani a andando (walk)
      call _gtaSetAnimWalkIX

      
      ld hl,#_iaPersonBuscandoAcera       //si llega a cero de momento para probar voy a poner que busque una acera
      bit 6,OBJ_FLAGS(ix)     //le agrego el FLAG_NO_DESTROY, para que no se lo cargue cuando salgamos de la pantalla
      jr Z,iaPersonEmpujadas4
      //si tiene el flag de no destroy es porke es un NPC que actua en la mision, por lo tanto le quitamos la IA para que lo maneje el mision controller
      ld hl,#_iaPersonPaseando

   iaPersonEmpujadas4:
      ld OBJ_MACHINE_STATE_PTR(ix),l
      ld OBJ_MACHINE_STATE_PTR+1(ix),h
      ld OBJ_FRAMES_FOR_NEXT_STATE(ix),#1
      ret

   __endasm;
}

/*
//va a cruzar por el paso de cebra a la otra acera
void iaPersonCruzandoCalle() __naked {
   __asm
      //en ix va el jugador (y si tuviera coche va en iy)
      ret
   __endasm;
}

//espera a que la circulación se detenga en ese sentido y entonces pasa a cruzando
void iaPersonEsperandoSemaforo() __naked {
   __asm
      //en ix va el jugador (y si tuviera coche va en iy)
      ret
   __endasm;
}
*/


//esta funcion recorre todas las personas y les cambia la IA a huyendo o a enfadado random
void gtaAsustaViejas() __naked {
   __asm
      ld l,OBJ_PARENT_PTR(ix)
      ld h,OBJ_PARENT_PTR+1(ix)
      ld de,#GAME_OBJECTS_PTR
      xor a
      sbc hl,de 
      ret NZ      //si no es el prota el que dispara, no asustamos a las viejas


      ld hl,#GAME_OBJECTS_PTR+48
     	ld de,#48
     	ld b,#15
     	
   gtaAsustaViejasBucle:
     	push hl
     	pop ix
     	exx
     	bit 2,OBJ_ACTIVE_TYPE(ix)		//miramos si es TYPE_CITIZEN
     	jr Z,gtaAsustaViejasBucleSiguiente
     	

      //si tiene el flag FLAG_NO_DESTROY no lo asustamos tampoco
      bit 6,OBJ_FLAGS(ix)
      jr NZ,gtaAsustaViejasBucleSiguiente    

      //antes de asustarlo o cabrearlo, miramos que no sea un NPC concreto (en este caso no hay que hacerle nada a él)

      //ld a,OBJ_ID_NPC_CAR(ix)
      //or a 
      //jr NZ,gtaAsustaViejasBucleSiguiente

     	
		call cpct_getRandom_lcg_u8_asm
		cp #VALOR_PERSONAS_IA
     	jr C,gtaAsustaViejasBucleHuye		//entre 0 y 210 huye, entre 210 y 255 ataca
     	
   gtaAsustaViejasBucleAtaca:
     	//si entra aqui es una persona, le cambiamos la IA a asustado huyendo (de momento solo huyendo, luego segun numero random puede enfadarse y pegarnos)
     	ld hl,#_iaPersonAtacandoProta
	   jr gtaAsustaViejasBucleSigue
     	
   gtaAsustaViejasBucleHuye:
     	//si entra aqui es una persona, le cambiamos la IA a asustado huyendo (de momento solo huyendo, luego segun numero random puede enfadarse y pegarnos)
     	ld hl,#_iaPersonHuyendo
     	
     	
   gtaAsustaViejasBucleSigue:
	   ld OBJ_MACHINE_STATE_PTR(ix),l
    	ld OBJ_MACHINE_STATE_PTR+1(ix),h
		ld OBJ_FRAMES_FOR_NEXT_STATE(ix),#127
     	
     	
   gtaAsustaViejasBucleSiguiente:
      exx
     	add hl,de
     	djnz gtaAsustaViejasBucle
     	
      ret		//viejas asustadas, nos piramos
   __endasm;
}



//despues de escuchar un disparo o golpe intenta salir de la pantalla lo más rapido posible
//vamos a hacerlo facil, vamos a calcular el angulo desde IX a B500 y salgo huyendo en la direccion contraria simplemente
//debe funcionar parecido al paseando pero yendo más rapido y siempre en la dirección contraria de huida
void iaPersonHuyendo() __naked {
   __asm
      //en ix va el jugador (y si tuviera coche va en iy)
      or a
      jr NZ,iaPersonHuyendos2			


   iaPersonHuyendoCambiaRelajado:
      jr iaPersonHuyendos3      //vamos a salir buscando la acera más cercana



   iaPersonHuyendos2:      //chequeamos si podemos seguir cayendo hacia donde va
      call _gtaSetAnimWalkIX
      
      
      //si es cero vamos a poner que vaya hacia algun sitio random, para ello vamos a pillar el next_frame que ya es un numero random
      //pongo primero la velocidad de animacion 
      ld OBJ_FRAME_SPEED(ix),#128

		push iy		//me guardo IY aunque deberia ser 0 pero por si las moscas
		ld iy,#GAME_OBJECTS_PTR			//calculo el angulo desde la persona hasta el prota
		call _vtAngleIXtoIY
	   pop iy		//recupero iy
	      
	   add #128			//miro para el angulo contrario porque quiero huir
	   ld b,#76			//velocidad de correr
		ld c,#15			//interpolacion

      call _vtAccelToSpeedAng		//ajusto la velocidad hacia el angulo contrario y a correr como un loco

      call _checkTilePerson
      ld a,#64
      cp l
      ret NC      //si no hay carry es que el tile L<65 y por lo tanto puedo seguir cayendo en esa dirección (aceras, carreteras y tiles sin colision), sino tengo que ver donde voy
      

      //cuando llegue aqui ya me aseguro que estoy alineado al tile más cercano tanto de X como de Y, de esa forma creo que no se saldrán de las aceras
      //call vuelveAlTileAnterior			//vuelvo al tile anterior y cambio de estado a buscando acera
      call _gtaVuelveAtrasUnPaso

      ret NC      //si no hay carry es que el tile L<65 y por lo tanto puedo seguir cayendo en esa dirección (aceras, carreteras y tiles sin colision), sino tengo que ver donde voy


   iaPersonHuyendos3:
      xor a          //sigo con el personaje parado para que en el siguiente frame elija otro lado para andar, ya que este camino no lo puedo pillar
      ld OBJ_SPEEDX(ix),a
      ld OBJ_SPEEDX+1(ix),a
      ld OBJ_SPEEDY(ix),a
      ld OBJ_SPEEDY+1(ix),a


   iaPersonHuyendos3a:
      ld hl,#_iaPersonBuscandoAcera       //si llega a cero de momento para probar voy a poner que busque una acera
      ld OBJ_MACHINE_STATE_PTR(ix),l
      ld OBJ_MACHINE_STATE_PTR+1(ix),h
      ld OBJ_FRAMES_FOR_NEXT_STATE(ix),#1
      ret
   __endasm;
}



void iaPersonAtacandoProta() __naked {
   __asm
      //en ix va el jugador (y si tuviera coche va en iy)
      or a
      jr NZ,iaPersonAtacandoProtas2			


   iaPersonAtacandoProtaCambiaRelajado:
      //antes de dejar de atacar, miramos si somos un NPC, porke en ese caso tenemos que seguir atacando hasta que nos mate el prota (o matemos nosotros al prota)
      ld a,OBJ_ID_NPC_CAR(ix)
      or a
      jp Z,iaPersonAtacandoProtas3      //no soy un NPC, me voy a salir buscando la acera más cercana

      ld OBJ_FRAMES_FOR_NEXT_STATE(ix),#200     //seguimos atacando



   iaPersonAtacandoProtas2:      //chequeamos si podemos seguir cayendo hacia donde va
      call _debug2
      ld a,OBJ_IA_LOCK_MOVE(ix)
      or a
      jp NZ,iaPersonAtacandoProtaAtacando      //si esta atacando no hacemos nada, nos vamos del tiri

      //si es cero aun no estamos haciendo ná, asi que cambiamos la ani a andando
      call _gtaSetAnimWalkIX


      push ix
		push iy		//me guardo IY aunque deberia ser 0 pero por si las moscas
		ld iy,#GAME_OBJECTS_PTR			//calculo el angulo desde la persona hasta el prota
		call _vtAngleIXtoIY
	   pop iy		//recupero iy
      pop ix
	      
	   ld b,#VELOCIDAD_NPC_ATACANDO			//velocidad intermedia
		ld c,#15			//interpolacion

	   call _vtAccelToSpeedAng		//ajusto la velocidad hacia el angulo contrario y a correr como un loco



		//aqui tengo que mirar si mi arma es melee me acerco o es range y disparo del tiron
		ld a,OBJ_SELECTED_WEAPON(ix)
		cp #5
		jr NC,iaPersonAtacandoProtaRange

	iaPersonAtacandoProtaMelee:
		push iy
		ld iy,#GAME_OBJECTS_PTR       //apunto al prota
		
		ld l,OBJ_POSX(iy)
      ld h,OBJ_POSX+1(iy)
      ld e,OBJ_POSX(ix)
      ld d,OBJ_POSX+1(ix)     
      xor a 
      sbc hl,de
      ld b,h
      ld c,l
      

	   ld l,OBJ_POSY(iy)
    	ld h,OBJ_POSY+1(iy)
      ld e,OBJ_POSY(ix)
      ld d,OBJ_POSY+1(ix)
      xor a
      sbc hl,de
      ex de,hl
      
	   call _vtDistance16
	    
		pop iy
		
		ld a,#2
		cp h
		jr NC,iaPersonAtacandoProtaAttack		//si estamos a 2 tiles o menos ataca
			
      //si llega aki que siga andando palante y compruebe si se choca con algo
		jr iaPersonAtacandoProtaCheckTile

	iaPersonAtacandoProtaRange:
		jr iaPersonAtacandoProtaAttack		//de momento ataca sin mirar nada



	iaPersonAtacandoProtaAttack:
      

		//si entro aqui es que voy a atacar, sea melee o range
      push ix
		push iy
		call _gtaCreaArma
		pop iy
		pop ix


      call _debug2
      ld OBJ_IA_LOCK_MOVE(ix),#RELAX_ATTACK_TIME_NPC


	iaPersonAtacandoProtaCheckTile:
      call _checkTilePerson
      ld a,#64
      cp l
      ret NC      //si no hay carry es que el tile L<65 y por lo tanto puedo seguir cayendo en esa dirección (aceras, carreteras y tiles sin colision), sino tengo que ver donde voy
      

      //cuando llegue aqui ya me aseguro que estoy alineado al tile más cercano tanto de X como de Y, de esa forma creo que no se saldrán de las aceras
      //call vuelveAlTileAnterior			//vuelvo al tile anterior y cambio de estado a buscando acera
      call _gtaVuelveAtrasUnPaso
      ret


   iaPersonAtacandoProtas3:
      xor a          //sigo con el personaje parado para que en el siguiente frame elija otro lado para andar, ya que este camino no lo puedo pillar
      ld OBJ_SPEEDX(ix),a
      ld OBJ_SPEEDX+1(ix),a
      ld OBJ_SPEEDY(ix),a
      ld OBJ_SPEEDY+1(ix),a


   iaPersonAtacandoProtas3a:

      ld hl,#_iaPersonBuscandoAcera       //si llega a cero de momento para probar voy a poner que busque una acera
      ld OBJ_MACHINE_STATE_PTR(ix),l
      ld OBJ_MACHINE_STATE_PTR+1(ix),h
      ld OBJ_FRAMES_FOR_NEXT_STATE(ix),#1
      ret


   iaPersonAtacandoProtaAtacando:
      call _debug2
      dec OBJ_IA_LOCK_MOVE(ix)      //estoy pegando, simplemente continuo la animacion, paro la velocidad y salgo
      
      bit 7,OBJ_SPEEDX+1(ix)
      jr Z,iaPersonAtacandoProtaAtacandos2

      //es negativo
      ld OBJ_SPEEDX+1(ix),#0xFF
      ld OBJ_SPEEDX(ix),#0xFF
      jr iaPersonAtacandoProtaAtacandos3

   iaPersonAtacandoProtaAtacandos2:
      xor a
      ld OBJ_SPEEDX+1(ix),a
      ld OBJ_SPEEDX(ix),a

   iaPersonAtacandoProtaAtacandos3:
      xor a
      ld OBJ_SPEEDY+1(ix),a
      ld OBJ_SPEEDY(ix),a

      ret 

   __endasm;
}


void iaPersonSiguiendoProta() __naked {
   __asm
      set 7,OBJ_FLAGS(ix)     //le ponemos sin colision ya que si lo pilla alguien nos putea la IA

		push iy		//me guardo IY aunque deberia ser 0 pero por si las moscas
		ld iy,#GAME_OBJECTS_PTR			//calculo el angulo desde la persona hasta el prota

      call _gtaDistanceIX_IY
      ld a,#3
      cp h
      jr NC,iaPersonSiguiendoProtaParaDeSeguir   //si estamos a menos de 4 tiles


		call _vtAngleIXtoIY
	   pop iy		//recupero iy
	      
	   ld b,#24			//velocidad intermedia
		ld c,#15			//interpolacion

	   call _vtAccelToSpeedAng		

      ld OBJ_FRAME_SPEED(ix),#64

      ret 

   iaPersonSiguiendoProtaParaDeSeguir:
      pop iy

      xor a                //frenamos
      ld OBJ_SPEEDX(ix),a
      ld OBJ_SPEEDX+1(ix),a
      ld OBJ_SPEEDY(ix),a
      ld OBJ_SPEEDY+1(ix),a
      ld OBJ_FRAME_SPEED(ix),a     //ponemos la ani parada
      //ld OBJ_FRAME_INDEX(ix),a     //ponemos la ani parada

      ret
   __endasm;
}




void iaPersonSiguiendoProtaSubeCoche() __naked {
   __asm
      call _iaPersonSiguiendoProta     //sigo al prota

      //KABESA aqui deberia ver primero si estamos cerca del prota, porke no vale subirse al coche desde cuenca
      push iy		//me guardo IY aunque deberia ser 0 pero por si las moscas

      ld iy,#GAME_OBJECTS_PTR
      ld a,OBJ_PARENT_PTR+1(iy)        //miro si el prota va en un coche o no
      or a
      jr Z,iaPersonSiguiendoProtaSubeCoches2    //no va en un coche asi que salgo

      call _gtaDistanceIX_IY           //si va en un coche, miro si estoy suficientemente cerca del prota
      ld a,#3
      cp h
      jr C,iaPersonSiguiendoProtaSubeCoches2       //va en un coche pero esta lejos no me puedo montar asi que salgo

      ld a,OBJ_PARENT_PTR+1(iy)        //si entra aqui el prota está en un coche y esta cerca asi que me subo al coche
      ld OBJ_PARENT_PTR+1(ix),a     
      ld a,OBJ_PARENT_PTR(iy)
      ld OBJ_PARENT_PTR(ix),a
   
      ld hl,#_iaPersonEnCoche
      ld OBJ_MACHINE_STATE_PTR(ix),l
      ld OBJ_MACHINE_STATE_PTR+1(ix),h
   
   iaPersonSiguiendoProtaSubeCoches2:
      pop iy                              //recupero IY y salgo

      ret
   __endasm;
}




//va a montarse en el coche que indiquen las variables
void iaPersonEnCoche() __naked {
   __asm

      ld h,OBJ_PARENT_PTR+1(ix)
      ld l,OBJ_PARENT_PTR(ix)

      ld a,(#0xb500+OBJ_PARENT_PTR+1)        //si el prota ya no esta en el coche, yo me bajo del coche tambien
      or a
      jr Z,iaPersonEnCocheMeBajoDelCoche

      xor a
      or h
      jr NZ,iaPersonEnCocheSigoEnCoche    //si sigo en el coche...



   iaPersonEnCocheMeBajoDelCoche:
      //res 7,OBJ_FLAGS(ix)        //le seteo que si tenga colision porke me bajo del coche OJO, no lo puedo resetear porque tiene que estar siempre sin colision
      ld hl,#_iaPersonSiguiendoProtaSubeCoche      //le pongo la IA para que siga al prota y se vuelva a montar en el coche si es necesario
      ld OBJ_MACHINE_STATE_PTR(ix),l
      ld OBJ_MACHINE_STATE_PTR+1(ix),h
      xor a
      ld OBJ_PARENT_PTR+1(ix),a
      ld OBJ_PARENT_PTR(ix),a
      ret

   
   
   iaPersonEnCocheSigoEnCoche:
      //en ix va el jugador (y si tuviera coche va en iy)
      set 7,OBJ_FLAGS(ix)     //le seteo que no tenga colision
      //y le pongo la posicion del parent
      ld h,OBJ_PARENT_PTR+1(ix)
      ld l,OBJ_PARENT_PTR(ix)
      push hl 
      pop iy 
      ld a,OBJ_POSX+1(iy)
      ld OBJ_POSX+1(ix),a
      ld a,OBJ_POSY+1(iy)
      ld OBJ_POSY+1(ix),a
      //y poco más, pa fuera
      ret
   __endasm;
}


//comprueba si puedo andar en la direccion en la que voy o no (osea, ve el siguiente tile si es acera o no)
void iaPersonCheckCanWalk() __naked {
   __asm
      //en ix va el jugador (y si tuviera coche va en iy)
      ld b,OBJ_POSX+1(ix)
      ld c,OBJ_POSY+1(ix)

      //por ser más finos, si OBJ_POSX(ix) o OBJ_POSY(ix) son mayores de 127 (bit 7 a 1) entonces sumamos 1 a B o C para ajustar el tile bien
      ld a,OBJ_POSX(ix)
      bit 7,a
      jr Z,iaPersonCheckCanWalks2
      inc b       //esta casi en el siguiente tile, asi que miramos el siguiente mejor
   iaPersonCheckCanWalks2:

      ld a,OBJ_POSY(ix)
      bit 7,a
      jr Z,iaPersonCheckCanWalks4
      inc c       //esta casi en el siguiente tile, asi que miramos el siguiente mejor
   iaPersonCheckCanWalks4:


      ld a,OBJ_SPEEDX(ix)
      or a 
      jr Z,iaPersonCheckCanWalksMovimientoVertical

   iaPersonCheckCanWalksMovimientoHorizontal:
      bit 7,a
      jr Z,iaPersonCheckCanWalksMovimientoHorizontalDer
   iaPersonCheckCanWalksMovimientoHorizontalIzq:
      //el personaje va para la izquierda, asi que comprobamos el tile izq
      dec b
      jr iaPersonCheckCanWalkCheckTile

   iaPersonCheckCanWalksMovimientoHorizontalDer:
      //el personaje va para la derecha, asi que comprobamos el tile der
      //inc b
      inc b
      inc b
      jr iaPersonCheckCanWalkCheckTile

   iaPersonCheckCanWalksMovimientoVertical:
      ld a,OBJ_SPEEDY(ix)
      bit 7,a
      jr Z,iaPersonCheckCanWalksMovimientoVerticalAba

   iaPersonCheckCanWalksMovimientoVerticalArr:
      //el personaje va para arriba, asi que comprobamos el tile arr
      dec c
      jr iaPersonCheckCanWalkCheckTile

   iaPersonCheckCanWalksMovimientoVerticalAba:
      //el personaje va para abajo, asi que comprobamos el tile abajo
      //inc c
      inc c 
      inc c 
   // jr iaPersonCheckCanWalkCheckTile

   iaPersonCheckCanWalkCheckTile:
      inc c
      call _vtCheckTileAsm		//llamo al check tile
		ld a,#24
		cp l
		jp C,iaPersonCheckCanWalkCheckTileCanNOTGo

   iaPersonCheckCanWalkCheckTileCanGo:
      ld a,#1        //podemos andar paya
      ret

   iaPersonCheckCanWalkCheckTileCanNOTGo:
      //ahora miro casos especialitos antes de salir (los pasos de cebra)
      ld a,l
      cp #32                                    //miro los dos pasos de cebra (tile 32 y 33)
      jr Z,iaPersonCheckCanWalkCheckTileCanGo

      cp #33
      jr Z,iaPersonCheckCanWalkCheckTileCanGo

      xor a          //no podemos andar paya
      ret
   __endasm;
}


/*
//va paseando con su coche
void iaDrivePaseando() __naked {
   __asm
      //en ix va el jugador (y si tuviera coche va en iy)
      ret
   __endasm;
}

//espera que el semaforo se ponga en verde
void iaDriveEsperaSemaforo() __naked {
   __asm
      //en ix va el jugador (y si tuviera coche va en iy)
      ret
   __endasm;
}

//hace un giro a izq o der para cambiar de calle
void iaDriveCambiaCalle() __naked {
   __asm
      //en ix va el jugador (y si tuviera coche va en iy)
      ret
   __endasm;
}

//huye de la pantalla lo más rapido posible con el coche, intenta esquivar el resto de coches
void iaDriveHuyendo() __naked {
   __asm
      //en ix va el jugador (y si tuviera coche va en iy)
      ret
   __endasm;
}

//va con el coche a por ti a golpearte o atropellarte
void iaDriveAtacando() __naked {
   __asm
      //en ix va el jugador (y si tuviera coche va en iy)
      ret
   __endasm;
}

//cuando hay una carrera va al siguiente checkpoint
void iaDriveNextCheckPoint() __naked {
   __asm
      //en ix va el jugador (y si tuviera coche va en iy)
      ret
   __endasm;
}

*/


/*
//esta funcion simplemente comprueba si el personaje que esta en IX esta suficientemente lejos de la camara como para desactivarlo
void gtaPersonaCheckDisableObject() __naked {
   __asm
      ld ix,(#_vtGameObjPtr)     //recupero el objeto actual
      xor a
      or OBJ_ACTIVE_TYPE(ix)
      ret Z          //si es cero no podemos desactivarlo porque ya esta desactivo


      bit 6,OBJ_FLAGS(ix)
      ret NZ      //si tiene el FLAG_NO_DESTROY a 1 no podemos quitarlo ya que forma parte de los NPC de las misiones

      ld a,(#_vtSceneScrollX)
      add #20     //la mitad de la pantalla horizontal
      sub OBJ_POSX+1(ix)
      jr NC,gtaPersonaCheckDisableObjects2
      neg
   gtaPersonaCheckDisableObjects2:
      //tengo en A la diferencia desde el medio de la pantalla hasta donde esta IX
      cp #LIMITE_HORIZONTAL_ELIMINAR_OBJETO
      jr C,gtaPersonaCheckDisableObjects3
      //si no hay carry este tio esta muy lejos, lo desactivamos (no hace falta borrar el sprite porque de todas formas ya no debe estar dibujandose)
      jp _vtCleanObject    //el metodo hará el ret
     

   gtaPersonaCheckDisableObjects3:
      ld a,(#_vtSceneScrollY)
      add #13     //la mitad de la pantalla horizontal
      sub OBJ_POSY+1(ix)
      jr NC,gtaPersonaCheckDisableObjects4
      neg
   gtaPersonaCheckDisableObjects4:
      //tengo en A la diferencia desde el medio de la pantalla hasta donde esta IX
      cp #LIMITE_VERTICAL_ELIMINAR_OBJETO
      ret C
      //si no hay carry este tio esta muy lejos, lo desactivamos (no hace falta borrar el sprite porque de todas formas ya no debe estar dibujandose)
      jp _vtCleanObject    //el metodo hará el ret
     
   __endasm;
}

*/



//comprueba si la persona IX colisiona con el escenario o no, corrije su posicion en funcion de la velocidad
void gtaPersonaCheckColisionScene() __naked {
   __asm
      ld ix,(#_vtGameObjPtr)     //recupero el objeto actual
      //llamo directamente a chequear el tile y esto me deja en L el numero de tile que tengo a los pies de mi muñeco
      call _checkTilePerson
      ld a,l

      cp #64
      ret C    //si resto 64 y hay carry es que estoy en un tile <64 y puedo pasar libremente sin ninguna colision, salgo del tiron

      //si hay colision, recupero el eje X
      ld d,OBJ_POSX+1(ix)  //nos kedamos con la posicion actual en DE
      ld e,OBJ_POSX(ix)

      ld hl,(#_oldPosX)    //recuperamos la posicion inicial del eje X antes de movernos
      ld OBJ_POSX+1(ix),h
      ld OBJ_POSX(ix),l
      

   gtaPersonaCheckColisionSceneY:
      //y ahora, que hemos recolocado el eje X, volvemos a comprobar en que tile estamos
      push de
      call _checkTilePerson
      ld a,l
      pop de
      cp #64
      ret C    //si ya no hay colision, no hace falta hacer nada más con la correccion del eje X ya no colisionamos con lo cual perfecto

      //si seguimos colisionando, tenemos que recuperar el eje X movido, y restaurar el eje Y
      ld OBJ_POSX+1(ix),d     //recupero el eje X movido
      ld OBJ_POSX(ix),e


      //ahora restauro el eje Y
      ld hl,(#_oldPosY)    //recuperamos la posicion inicial del eje X antes de movernos
      ld OBJ_POSY+1(ix),h
      ld OBJ_POSY(ix),l


      //ahora tengo movido el eje X y restaurado el eje Y, compruebo ahora la colision, ahora pueden pasar dos cosas:
      //que haya colision tambien, con lo cual tengo que restaurar el eje X tambien y dejar los dos restaurados, 
      //o que no haya colision y puedo salir ya con el eje Y restaurado y el X movido
      call _checkTilePerson
      ld a,l
      cp #64
      ret C    //si ya no hay colision, salgo con el eje Y restaurado y el X movido

      //si llego aqui tengo que restaurar el ejeX tambien
      ld hl,(#_oldPosX)    //recuperamos la posicion inicial del eje X antes de movernos
      ld OBJ_POSX+1(ix),h
      ld OBJ_POSX(ix),l

      ret

   __endasm;

}



void gtaCocheCheckColisionScene() __naked {
   __asm
      ld ix,(#_vtGameObjPtr)     //recupero el objeto actual
      
      //primero miro todos los tiles que estoy pisando con el coche (osea compruebo la colision 3x3)
      call _checkTileMyCarFull            //miramos la colision del cuadrado completo 3x3 |=|
      xor a
      or l
      ret Z    //si esto da cero no hay ninguna colision de ningun tipo y podemos salir ya


      ld a,(#_vtColisionLastX)
      ld (#_auxPosX),a
      ld a,(#_vtColisionLastY)
      ld (#_auxPosY),a

   
      ld ix,(#_vtGameObjPtr)     //recupero el objeto actual

      //si llegamos aqui, ya sabemos que alguna colision hay, ahora la cosa es si colisiono por la izq, der, arr, aba o por una esquina
      call _checkTileMyCarVer       //miramos la colision vertical |
      xor a
      or l
      jr NZ,gtaCocheCheckColisionSceneColArribaAbajo     //hay colision vertical
      

      //ahora miro si hay colision horizontal      //miramos la colision horizontal --
   gtaCocheCheckColisionScenes2:
      call _checkTileMyCarHor          //ahora comprobamos si hay colision horizontal
      xor a
      or l
      jr NZ,gtaCocheCheckColisionSceneColIzquierdaDerecha
      

      //necesito recuperar donde estaba la ostia
      ld a,(#_auxPosX)
      ld (#_vtColisionLastX),a
      ld a,(#_auxPosY)
      ld (#_vtColisionLastY),a
      

      jp gtaCocheCheckColisionSceneColOstiaTerrible      //sino hemos dado por una esquina, en este caso lo mejor es ir hacia atras para no dejar bloqueado el coche


      //en vtColisionLastX y vtColisionLastY tengo el tile donde me he ostiado, en funcion de donde me encuentre relativo 
      //a la posicion de la ostia quizas puedo aumentar la velocidad hacia un sitio u otro
      //no normal será ostiar por arriba, abajo, izq o der, pero no por el medio ya que ostiarias antes por un lado
      //puedo hacer que corrija el TORQUE del coche en funcion del lado ostiado usando la formula del 
      //rebote horizontal o vertical T=256-T o T=128-T y de paso si soltamos un bukake de chispas lo bordamos
     



      
   gtaCocheCheckColisionSceneColArribaAbajo:
      ld ix,(#_vtGameObjPtr)     //recupero el objeto actual para la ostia por arriba o por abajo

      //antes de cambiarle el angulo y la velocidad tengo que empujar el coche hacia fuera de la colision para que no siga colisionando
      ld b,OBJ_POSY+1(ix)
      ld a,(#_vtColisionLastY)
      cp b
      jr Z,gtaCocheCheckColisionSceneColArribaAbajoPushDown

   gtaCocheCheckColisionSceneColArribaAbajoPushUp:
      dec b
      jr gtaCocheCheckColisionSceneColArribaAbajos2

   gtaCocheCheckColisionSceneColArribaAbajoPushDown:
      inc b
      
   gtaCocheCheckColisionSceneColArribaAbajos2:
      ld OBJ_POSY+1(ix),b


      //aqui pueden pasar 2 cosas, que el coche colisione por el lado izq/der, con lo cual rebotará contra la pared casi sin frenarte y saltarán chispas,
      //o si vas en la misma dirección que esta la pared te das una ostia fuerte contra la pared y te frenarás
      //para calcularlo voy a mirar en que cuadrante esta
      //                            \ O /         O = ostia terrible           
      //                             \ /
      //                    roza...   X  roza la pared y sigue adelante
      //                             / \
      //                            / O \

      //traslado el angulo a algo más manejable (le sumo 32 grados) (recuerda, no hay 360 grados, son 256), eso me deja los angulos de forma que 
      // de 0 a 64 roza, de 64 a 128 ostia terrible, de 128 a 192 roza y de 192 a 256 ostia terrible
      //o lo que es lo mismo, si el bit 6 es 1 ostia terrible, si es 0 roza
      ld a,OBJ_CAR_TORQUE(ix)
      add #32
      bit 6,a
      jp NZ,gtaCocheCheckColisionSceneColOstiaTerribleSinDesplazamiento       //ostia terrible pero sin el desplazamiento (me lo salto porque ya lo he hecho antes)


     	//hago rebote en pared HORIZONTAL T=256-T
     	ld b,OBJ_CAR_TORQUE(ix)
     	ld a,#0
     	sub b
     	ld OBJ_CAR_TORQUE(ix),a
     	
     	//cambio signo a velocidad vertical  o lo que es lo mismo SPEEDY=0-SPEEDY
     	ld e,OBJ_SPEEDY(ix)
     	ld d,OBJ_SPEEDY+1(ix)
     	ld hl,#0
     	xor a
     	sbc hl,de
     	ld OBJ_SPEEDY(ix),l
     	ld OBJ_SPEEDY+1(ix),h
     	
     	//quizas haga falta reducir un pelin la velocidad del coche (ya que si ha rozado con la pared lo normal es que se frene un poco)
      ld a,OBJ_CAR_SPEED(ix)
      cp #4
      ret C
      srl a    //divido la velocidad entre 2
      srl a    //divido la velocidad entre 2
      ld OBJ_CAR_SPEED(ix),a

      jr gtaCocheCheckColisionSceneRasponYsalgo
     	//suelto chispas  (esto lo dejamos para más adelante)
     	ret
     








   gtaCocheCheckColisionSceneColIzquierdaDerecha:
      ld ix,(#_vtGameObjPtr)     //recupero el objeto actual

      //antes de cambiarle el angulo y la velocidad tengo que empujar el coche hacia fuera de la colision para que no siga colisionando
      ld b,OBJ_POSX+1(ix)
      ld a,(#_vtColisionLastX)
      cp b
      jr Z,gtaCocheCheckColisionSceneColIzquierdaDerechaPushRight
   
   gtaCocheCheckColisionSceneColIzquierdaDerechaPushLeft:
      dec b
      jr gtaCocheCheckColisionSceneColIzquierdaDerechas2

   gtaCocheCheckColisionSceneColIzquierdaDerechaPushRight:
      inc b

   gtaCocheCheckColisionSceneColIzquierdaDerechas2:
      ld OBJ_POSX+1(ix),b



      //aqui pueden pasar 2 cosas, que el coche colisione por el lado izq/der, con lo cual rebotará contra la pared casi sin frenarte y saltarán chispas,
      //o si vas en la misma dirección que esta la pared te das una ostia fuerte contra la pared y te frenarás
      //para calcularlo voy a mirar en que cuadrante esta
      //                            \ R /         R = roza contra la pared
      //                             \ /
      //                    ostia...  X  ostia terrible
      //                             / \
      //                            / R \

      //traslado el angulo a algo más manejable (le sumo 32 grados) (recuerda, no hay 360 grados, son 256), eso me deja los angulos de forma que 
      // de 0 a 64 ostia terrible, de 64 a 128 roza, de 128 a 192 ostia terrible y de 192 a 256 roza
      //o lo que es lo mismo, si el bit 6 es 0 ostia terrible, si es 1 roza
      ld a,OBJ_CAR_TORQUE(ix)
      add #32
      bit 6,a
      jr Z,gtaCocheCheckColisionSceneColOstiaTerribleSinDesplazamiento       //ostia terrible pero sin el desplazamiento (me lo salto porque ya lo he hecho antes)



     	//hago rebote en pared VERTICAL T=128-T
     	ld b,OBJ_CAR_TORQUE(ix)
     	ld a,#128
     	sub b
     	ld OBJ_CAR_TORQUE(ix),a
     	
     	//cambio signo a velocidad horizontal
     	ld e,OBJ_SPEEDX(ix)
     	ld d,OBJ_SPEEDX+1(ix)
     	ld hl,#0
     	xor a
     	sbc hl,de
     	ld OBJ_SPEEDX(ix),l
     	ld OBJ_SPEEDX+1(ix),h
     	
     	//quizas haga falta reducir un pelin la velocidad del coche (ya que si ha rozado con la pared lo normal es que se frene un poco)
      ld a,OBJ_CAR_SPEED(ix)
      cp #4
      ret C
      srl a    //divido la velocidad entre 2
      //srl a    //divido la velocidad entre 2
      ld OBJ_CAR_SPEED(ix),a
     	//suelto chispas  (esto lo dejamos para más adelante)

   gtaCocheCheckColisionSceneRasponYsalgo:
      //quito vida
      srl a    //divido la velocidad entre 2 de nuevo y se lo quito de vida
      srl a
      srl a
      srl a
      ld b,a
      ld a,OBJ_LIFE(ix)
      sub b
      ld OBJ_LIFE(ix),a
      set 3,OBJ_FLAGS(ix)     //le activamos el flag de flash sprite para dar feedback durante 1 frame

     	ret
     	










   gtaCocheCheckColisionSceneColOstiaTerrible:
      ld ix,(#_vtGameObjPtr)     //recupero el objeto actual

      
      //antes de cambiarle el angulo y la velocidad tengo que empujar el coche hacia fuera de la colision para que no siga colisionando
      ld b,OBJ_POSY+1(ix)
      ld a,(#_vtColisionLastY)
      cp b
      jr Z,gtaCocheCheckColisionSceneColOstiaTerriblePushDown

   gtaCocheCheckColisionSceneColOstiaTerriblePushUp:
      dec b
      jr gtaCocheCheckColisionSceneColOstiaTerribles2

   gtaCocheCheckColisionSceneColOstiaTerriblePushDown:
      inc b
      
   gtaCocheCheckColisionSceneColOstiaTerribles2:
      ld OBJ_POSY+1(ix),b





      //antes de cambiarle el angulo y la velocidad tengo que empujar el coche hacia fuera de la colision para que no siga colisionando
      ld b,OBJ_POSX+1(ix)
      ld a,(#_vtColisionLastX)
      cp b
      jr Z,gtaCocheCheckColisionSceneColOstiaTerriblePushRight
   
   gtaCocheCheckColisionSceneColOstiaTerriblePushLeft:
      dec b
      jr gtaCocheCheckColisionSceneColOstiaTerribles3

   gtaCocheCheckColisionSceneColOstiaTerriblePushRight:
      inc b

   gtaCocheCheckColisionSceneColOstiaTerribles3:
      ld OBJ_POSX+1(ix),b


   gtaCocheCheckColisionSceneColOstiaTerribleSinDesplazamiento:



      //aqui lo mejor es rebotar hacia atras (como si te hubieses dado un coche frontal)
      //cambio signo a velocidad horizontal
     	ld e,OBJ_SPEEDX(ix)
     	ld d,OBJ_SPEEDX+1(ix)
      //sra e       //divido entre 2 E (sin alterar el bit de signo)
      //sra d       //divido entre 2 D (sin alterar el bit de signo), el bit que se queda en el carry, tenemos que ponerlo en el bit 6 de E
      //jr NC,gtaCocheCheckColisionSceneColOstiaTerribleSinDesplazamientos2
      //set 6,e     //si hay carry seteo el bit 6 de E

   //gtaCocheCheckColisionSceneColOstiaTerribleSinDesplazamientos2:

     	ld hl,#0
     	xor a
     	sbc hl,de
     	ld OBJ_SPEEDX(ix),l
     	ld OBJ_SPEEDX+1(ix),h

      //cambio signo a velocidad horizontal
     	ld e,OBJ_SPEEDY(ix)
     	ld d,OBJ_SPEEDY+1(ix)
      //sra e       //divido entre 2 E (sin alterar el bit de signo)
      //sra d       //divido entre 2 D (sin alterar el bit de signo), el bit que se queda en el carry, tenemos que ponerlo en el bit 6 de E
      //jr NC,gtaCocheCheckColisionSceneColOstiaTerribleSinDesplazamientos3
      //set 6,e     //si hay carry seteo el bit 6 de E

   //gtaCocheCheckColisionSceneColOstiaTerribleSinDesplazamientos3:

     	ld hl,#0
     	xor a
     	sbc hl,de
     	ld OBJ_SPEEDY(ix),l
     	ld OBJ_SPEEDY+1(ix),h

      ld a,OBJ_CAR_SPEED(ix)
      cp #4
      ret C
      srl a    //divido la velocidad entre 2
      srl a    //divido la velocidad entre 2
      ld OBJ_CAR_SPEED(ix),a

      srl a    //divido la velocidad entre 2
      srl a    //divido la velocidad entre 2
      ld b,a
      ld a,OBJ_LIFE(ix)
      sub b
      ld OBJ_LIFE(ix),a
      set 3,OBJ_FLAGS(ix)

      ret


   __endasm;
}




//calcula la distancia entre el objeto IX e IY y la devuelve en HL  (H son los tiles de diferencia)
void gtaDistanceIX_IY() __naked {
   __asm
      ld l,OBJ_POSX(iy)
      ld h,OBJ_POSX+1(iy)
      ld e,OBJ_POSX(ix)
      ld d,OBJ_POSX+1(ix)     
      xor a 
      sbc hl,de
      
      ld b,h
      ld c,l
      
      ld l,OBJ_POSY(iy)
      ld h,OBJ_POSY+1(iy)
      ld e,OBJ_POSY(ix)
      ld d,OBJ_POSY+1(ix)
      xor a
      sbc hl,de
      
      ex de,hl
      
      call _vtDistance16

      ret
   __endasm;
}



void gtaLostTimeFunc() __naked {
   __asm
      ld hl,(#_lostTimeIndex)
      ld a,#GAME_OBJECTS_PTR_LIMIT_H
      sub h
      jr NZ,gtaLostTimeFuncs2
    
    	//si entro aqui ya he dado toda la vuelta a todos los objetos, empiezo desde el primero
    	ld hl,#GAME_OBJECTS_PTR+48		//el primero no lo voy a eliminar nunca  
 
      
	gtaLostTimeFuncs2:
      //ld (#_vtGameObjPtr),hl					//esto ya no me hace falta
      //call _gtaPersonaCheckDisableObject     //miramos si desactivamos esta entidad
      
      xor a								//miro primero si esta activo, sino paso al siguiente sin mas
      or (hl)
      jr Z,gtaLostTimeFuncFin          //si es cero no podemos desactivarlo porque ya esta desactivo

		push hl								//si esta activo tengo que meterlo en IX para mirarle cositas
		pop ix
      bit 6,OBJ_FLAGS(ix)
      jr NZ,gtaLostTimeFuncFin      	//si tiene el FLAG_NO_DESTROY a 1 no podemos quitarlo ya que forma parte de los NPC de las misiones, paso al siguiente


      ld a,(#_vtSceneScrollX)								//ahora calculo la distancia desde la mitad de la pantalla hasta el objeto
      add #20     						//la mitad de la pantalla horizontal
      sub OBJ_POSX+1(ix)
      jr NC,gtaLostTimeFuncs3
      neg
   gtaLostTimeFuncs3:
      cp #LIMITE_HORIZONTAL_ELIMINAR_OBJETO				//tengo en A la diferencia desde el medio de la pantalla hasta donde esta IX
      jr C,gtaLostTimeFuncs4			//si hay carry el objeto esta cerca del centro no lo quitamos y seguimos mirando
      
      jr gtaLostTimeFuncFinAndClean		//si no hay carry este tio esta muy lejos, lo desactivamos (no hace falta borrar el sprite porque de todas formas ya no debe estar dibujandose)	

     

   gtaLostTimeFuncs4:
      ld a,(#_vtSceneScrollY)							//ahora vamos a mirar lo mismo pero en vertical
      add #13     //la mitad de la pantalla horizontal
      sub OBJ_POSY+1(ix)
      jr NC,gtaLostTimeFuncs5
      neg
   gtaLostTimeFuncs5:      
      cp #LIMITE_VERTICAL_ELIMINAR_OBJETO		//tengo en A la diferencia desde el medio de la pantalla hasta donde esta IX
      jr C,gtaLostTimeFuncFin			//si hay carry esta cerca del centro verticalmente, no lo quitamos

      //si no hay carry este tio esta muy lejos, lo desactivamos (no hace falta borrar el sprite porque de todas formas ya no debe estar dibujandose)
      //sigue pabajo llamando al clean object
      
      
      
    gtaLostTimeFuncFinAndClean:
      push hl
      ld	(_vtGameObjPtr), ix
    	call _vtCleanObject
      pop hl
      
    gtaLostTimeFuncFin:
      ld de,#48					//y paso al siguiente y guardo en el puntero lostTimeIndex para la siguiente entrada
      add hl,de
      ld (#_lostTimeIndex),hl		//preparo para la siguiente entrada aqui (si me he pasado se dará cuenta en la siguiente entrada)
      
      ret
      
      
      
      
      
      /*ld a,#16
      cp (hl)
      jr Z,gtaLostTimeFuncSal
      
      ld a,(hl)
      inc (hl)

      ld b,a         //B=2 A=2
      add b       //x2     A=4
      add b       //x3     A=6
      add a       //x6     A=12
      add a       //x12    A=24
      ld l,a
      ld h,#0
      add hl,hl   //x24
      add hl,hl   //x48

      ld de,#GAME_OBJECTS_PTR
      add hl,de
      ld (#_vtGameObjPtr),hl
      call _gtaPersonaCheckDisableObject     //miramos si desactivamos esta entidad
      ret

   gtaLostTimeFuncSal:
      ld a,#1        //vuelvo al 1 y pa fuera
      ld (#_lostTimeIndex),a
      ret*/
   __endasm;
}



//esta funcion pilla donde esta el scroll y va recorriendo el array de objetos para ver si tiene que crear alguno
//recorre 1 objeto cada vez, pero se coloca rapidamente en el eje X correspondiente (los objetos estan ordenados por ejeX)
//de tal forma que si itemX < scrollX puedes ir pasando al siguiente de seguido
//y cuando lleguemos a itemX > scrollX+40 podemos pasar al primero de nuevo
void gtaCheckObjectsInMap() __naked {
   __asm

      ld hl,(#gtaCheckObjectsInMapPtr)

      ld a,(#_vtSceneScrollX)    //ahora calculo el limite izq de los objetos que tengo que crear (si tengo que crear alguno)
      sub #10                    //que es, donde este el scroll -10
      jr NC,gtaCheckObjectsInMaps2a
      xor a    //si hay carry ponemos 0
   gtaCheckObjectsInMaps2a:
      ld c,a                     //y lo dejo en C pa luego

      //ahora, si (hl)<c tengo que avanzar 3 a hl para pasar al siguiente, y hacer este bucle hasta que llegue a uno que no sea menor que C
   gtaCheckObjectsInMapBucleInit:
      ld a,(hl)
      or a
      jr NZ,gtaCheckObjectsInMapBucleInits2

   gtaCheckObjectsInMapsFinalArray:
      //si es cero es que hemos llegado al final del array, reiniciamos el index y nos piramos, en el siguiente frame se seguira viendo si hay algo
      ld hl,#_mapObjects
      ld (#gtaCheckObjectsInMapPtr),hl
      ret 

   gtaCheckObjectsInMapBucleInits2:
      //miramos si A<C
      sub C
      jr NC,gtaCheckObjectsInMaps4      //si no hay carry es que A > C y podemos seguir

      //si A < C, pasamos al siguiente objeto y seguimos mirando
      inc hl 
      inc hl 
      inc hl 
      inc hl
      jr gtaCheckObjectsInMapBucleInit




   gtaCheckObjectsInMaps4:     
      //antes de seguir voy a ver si ya me he pasado por la derecha,
      ld a,(#_vtSceneScrollX)    //ahora calculo el limite izq de los objetos que tengo que crear (si tengo que crear alguno)
      add #50                    //que es, donde este el scroll -10
      jr NC,gtaCheckObjectsInMaps4a
      ld a,#255    //si hay carry ponemos 255 al maximo
   gtaCheckObjectsInMaps4a:
      ld c,a                     //y lo dejo en C pa luego
      ld a,(hl)
      sub c
      jr NC,gtaCheckObjectsInMapsFinalArray     //si no hay carry es que tileX > C osea, el objeto esta más a la derecha que _vtSceneScrollX+50 y por lo tanto podemos volver al comienzo del array y pirarnos

      ld a,(hl)            //cargo sus datos en las variables (y de paso me voy al siguiente)
      ld (#gtaCheckObjectsInMapTileX),a
      inc hl

      //ahora puede estar dentro del eje y o no, vamos a verlo
      ld a,(#_vtSceneScrollY)    //ahora calculo el limite arr de los objetos que tengo que crear (si tengo que crear alguno)
      sub #10                    //que es, donde este el scroll -10
      jr NC,gtaCheckObjectsInMaps5
      xor a
   gtaCheckObjectsInMaps5:
      sub (hl)
      jr NC,gtaCheckObjectsInMapsSiguienteySalPre     //al restar tileY al scroll debe haber carry, si no hay carry es que estoy más arriba de la pantalla

      ld a,(#_vtSceneScrollY)    //ahora calculo el limite arr de los objetos que tengo que crear (si tengo que crear alguno)
      add #35                    //que es, donde este el scroll -10
      jr NC,gtaCheckObjectsInMaps6
      ld a,#255
   gtaCheckObjectsInMaps6:
      sub (hl)
      jr C,gtaCheckObjectsInMapsSiguienteySalPre      //al restar tileY al scroll+35 no debe haber carry, porque si lo hay es que me he salido por abajo de la pantalla
      

      //cuando llegue aqui ya estoy apuntando a un objeto que esta en pantalla o al borde de la pantalla
      //tengo que asegurarme que no esté ya agregado (porque sino lo agregaria dos veces y se liaria parda)
      //lo más facil es ver si es un TYPE_AUX_OBJECT primero
      //y despues ver si el eje X y el eje Y son iguales (si se cumplen las 3 cosas seguramente es el mismo objeto, no tenemos que volver a crearlo)
      ld a,(hl)
      ld (#gtaCheckObjectsInMapTileY),a
      inc hl
      ld a,(hl)
      ld (#gtaCheckObjectsInMapType),a
      inc hl
      ld a,(hl)
      ld (#gtaCheckObjectsInMapState),a
      inc hl
      ld (#gtaCheckObjectsInMapPtr),hl


      ld ix,#GAME_OBJECTS_PTR
      ld de,#48
      ld b,#16

   gtaCheckObjectsInMapsBucleFindRepe:
      ld a,OBJ_ACTIVE_TYPE(ix)
      cp #TYPE_AUX_OBJECT
      jr NZ,gtaCheckObjectsInMapsBucleFindRepeFin
      
      ld a,(#gtaCheckObjectsInMapTileX)
      sub OBJ_POSX+1(ix)
      jr NZ,gtaCheckObjectsInMapsBucleFindRepeFin

      ld a,(#gtaCheckObjectsInMapTileY)
      sub OBJ_POSY+1(ix)
      jr NZ,gtaCheckObjectsInMapsBucleFindRepeFin
      
      //si llegamos aqui es que el objeto ya existe, podemos pasar de este objeto y irnos al siguiente
      jr gtaCheckObjectsInMapsSiguienteySal
      

   gtaCheckObjectsInMapsBucleFindRepeFin:
      add ix,de
      djnz gtaCheckObjectsInMapsBucleFindRepe

      
      
      //si llego aqui el objeto no existe, podemos crearlo y luego finalizar
      call gtaCheckObjectsInMapsCreaObjetoItem
      //y luego sigue pasando al siguiente y saliendo
      ret

   gtaCheckObjectsInMapsSiguienteySalPre:
      inc hl      //paso a type
      inc hl      //paso a estado
      inc hl      //paso al siguiente
      

   gtaCheckObjectsInMapsSiguienteySal:
      //si entra por aqui es que el objeto ya esta activo y no hay que volverlo a meter, asi que pasamos al siguiente y pa fuera
      ld (#gtaCheckObjectsInMapPtr),hl
      ret


   gtaCheckObjectsInMapsCreaObjetoItem:
      //aqui vamos a crear el objeto que tenemos en gtaCheckObjectsInMapTileX, gtaCheckObjectsInMapTileY e gtaCheckObjectsInMapType
      
      ld hl,(#_vtGameObjPtr)     //me guardo el puntero _vtGameObjPtr
      push hl
      
      call _vtGetFreeGameObjPtr        //preparamos el objeto del prota
      ld ix,(#_vtGameObjPtr)      

      pop hl
      ld (#_vtGameObjPtr),hl     //recupero el puntero _vtGameObjPtr

      // SAFE_GET_FREE
      //xor a       //hago una comprobacion rapida por si no hubiese sitio para meter más instancias en el engine
      //.dw #0xB4DD    //or ixh      //simplemente hago 0 or IXH (que tiene que ser 0xB5 o por ahi), si el or da Zero es que no hubo espacio para agregarlo, podemos eliminarlo
      //ret Z

      ld OBJ_ACTIVE_TYPE(ix),#TYPE_AUX_OBJECT
      
      ld a,(#gtaCheckObjectsInMapTileX)
      ld OBJ_POSX+1(ix),a
      ld a,(#gtaCheckObjectsInMapTileY)
      ld OBJ_POSY+1(ix),a
      xor a
      ld OBJ_POSX(ix),a //al medio del tile
      ld OBJ_POSY(ix),a
      ld OBJ_SPEEDX+1(ix),a
      ld OBJ_SPEEDY+1(ix),a
      ld OBJ_SPEEDX(ix),a
      ld OBJ_SPEEDY(ix),a

      ld a,(#gtaCheckObjectsInMapType)
      cp #16
      jr NC,gtaCheckObjectsInMapsCreaObjetos4




      //si es un item... (menor que 16) le pongo colision y la funcion step y draw por defecto de momento
      ld hl,#_gtaItemStepFunc
      ld OBJ_STEP_FUNC(ix),l
      ld OBJ_STEP_FUNC+1(ix),h
      
      ld hl,#_gtaItemColisionFunc            //esta hará que puedas pillar el item y se autodestruya
      ld OBJ_COLISION_FUNC(ix),l
      ld OBJ_COLISION_FUNC+1(ix),h
      
      ld hl,#_vtDrawObjectFunc
      ld OBJ_PREPARE_FUNC(ix),l
      ld OBJ_PREPARE_FUNC+1(ix),h

      ld hl,#_animItemsList
      ld a,(#gtaCheckObjectsInMapType)
      ld OBJ_SELECTED_WEAPON(ix),a
      dec a
      ld OBJ_FRAME_INDEX+1(ix),a
      ld OBJ_ANIMATION_LIST(ix),l
      ld OBJ_ANIMATION_LIST+1(ix),h
      ld OBJ_ANIMATION_MAX(ix),#8
      //ld OBJ_FRAME_SPEED(ix),#0
      //ld OBJ_FRAME_INDEX(ix),#0
      
      ld OBJ_WIDTH(ix),#2
      ld OBJ_HEIGHT(ix),#1
      ld OBJ_FLAGS(ix),#0
      ret





   gtaCheckObjectsInMapsCreaObjetos4:
      ld a,(#gtaCheckObjectsInMapType)
      cp #32
      jr NC,gtaCheckObjectsInMapsCreaObjetos5

      //ESTAS SON LAS BASES AUN NO TIENEN QUE HACER NADA  KABESA
	   ld OBJ_BASE_TYPE(ix),a		//metemos que tipo de base es y ya está, no hay que hacer nada más
      //si es la base de BASE_INK_AND_PAINT le agrego un step especial que pinta el coche del prota si te acercas
      cp #BASE_INK_AND_PAINT
      jr NZ,gtaCheckObjectsInMapsCreaObjetos4b
      ld hl,#_gtaBaseCleanAndPaintFunc
      ld OBJ_STEP_FUNC(ix),l
      ld OBJ_STEP_FUNC+1(ix),h
      ret

   gtaCheckObjectsInMapsCreaObjetos4b:
      cp #BASE_HOSPITAL
      ret NZ
      ld hl,#_gtaBaseHospitalFunc
      ld OBJ_STEP_FUNC(ix),l
      ld OBJ_STEP_FUNC+1(ix),h
      ret

   gtaCheckObjectsInMapsCreaObjetos5:
      ld a,(#gtaCheckObjectsInMapType)
      cp #64
      jr NC,gtaCheckObjectsInMapsCreaObjetos6
      
      //ESTAS SON LAS HOME AUN NO TIENEN QUE HACER NADA  KABESA
      ld OBJ_ACTIVE_TYPE(ix),#0
      ret

   gtaCheckObjectsInMapsCreaObjetos6:
      ld a,(#gtaCheckObjectsInMapType)
      cp #128
      jr NC,gtaCheckObjectsInMapsCreaObjetos7
      //ESTOS SON LOS SEMAFOROS, LO CONFIGURO

      //ld OBJ_SPEEDX(ix),#0
      //ld OBJ_SPEEDX+1(ix),#0

      cp #65
      jr Z,gtaCheckObjectsInMapsCreaObjetos6s2
      cp #66
      jr Z,gtaCheckObjectsInMapsCreaObjetos6s2
      
      ld OBJ_SPEEDX(ix),#255  //al reves si es 64 o 67
      ld OBJ_SPEEDX+1(ix),#255  //al reves si es 64 o 67

   gtaCheckObjectsInMapsCreaObjetos6s2:
      //un semaforo, tiene su propio step (que controlara si se pone en rojo, amarillo o verde)
      ld hl,#_gtaSemaforoStepFunc
      ld OBJ_STEP_FUNC(ix),l
      ld OBJ_STEP_FUNC+1(ix),h
      
      ld hl,#_gtaSemaforoDrawFunc         //visualmente no necesita nada especial
      ld OBJ_PREPARE_FUNC(ix),l
      ld OBJ_PREPARE_FUNC+1(ix),h

      ld hl,#_animSemaforo
      ld OBJ_ANIMATION_LIST(ix),l
      ld OBJ_ANIMATION_LIST+1(ix),h
      ld OBJ_ANIMATION_MAX(ix),#3

      ld OBJ_WIDTH(ix),#2
      ld OBJ_HEIGHT(ix),#2
      //ld OBJ_FLAGS(ix),#FLAG_FORCE_DRAW

      ret

   gtaCheckObjectsInMapsCreaObjetos7:
      //ESTO AUN NO ES NADA, ASI QUE NO HAGO NADA
      //puede ser una farola (VAMOS A DAR POR HECHO QUE ES 128 y es una farola)
      //un semaforo, tiene su propio step (que controlara si se pone en rojo, amarillo o verde)
      ld hl,#_vtStepObjectFunc
      ld OBJ_STEP_FUNC(ix),l
      ld OBJ_STEP_FUNC+1(ix),h
      
      ld hl,#_vtDrawObjectFunc         //visualmente no necesita nada especial
      ld OBJ_PREPARE_FUNC(ix),l
      ld OBJ_PREPARE_FUNC+1(ix),h

      ld hl,#_animFarola
      ld OBJ_ANIMATION_LIST(ix),l
      ld OBJ_ANIMATION_LIST+1(ix),h
      ld OBJ_ANIMATION_MAX(ix),#1

      ld OBJ_WIDTH(ix),#2
      ld OBJ_HEIGHT(ix),#2
      
      ld OBJ_FLAGS(ix),#0
      
      ld OBJ_SELECTED_WEAPON(ix),#255     //le pongo una marca para saber que es una luz
      
      ret
      


   gtaCheckObjectsInMapPtr:
      .dw #_mapObjects

   gtaCheckObjectsInMapTileX:
      .db #0
   gtaCheckObjectsInMapTileY:
      .db #0
   gtaCheckObjectsInMapType:
      .db #0
   gtaCheckObjectsInMapState:
      .db #0


   __endasm;
}



//controlará ciertos eventos del juego, lo llamara el prota 1 vez por cada step
//cada 16 steps mirará si tiene puede crear una nueva persona
void gtaGameController() __naked {
   __asm
      call _gtaMisionController


      ld a,(#_vtStopGame)
      or a 
      jr Z,gtaGameControllers2
      xor a
      ld (#_vtStopGame),a
      //ld (#_talkFrase),a
      ld (#TALK_FRASE),a


      //redibujamos lo que nos hemos cargao con la conversación...
      xor a                         
      ld	(#_vtPushTilesIniX),a      //vtPushTilesIniX=0;
      ld	hl,#_vtPushTilesEndX       //vtPushTilesEndX=40;
      ld	(hl), #0x28
      inc a
      ld	(#_vtPushTilesIniY),a      //vtPushTilesIniY=1;
      inc a
      inc a
      ld	(#_vtPushTilesEndY),a      //vtPushTilesEndY=3;
      call	_vtPushTilesToDraw
      


   gtaGameControllers2:
      ld a,(#_explosionTime)
      or a
      jr Z,gtaGameControllers2c

      //si entra aqui es que acaba de explotar algo
      dec a
      ld (#_explosionTime),a
      jr Z,gtaGameControllers2b

   gtaGameControllers2a:
      //ponemos la paleta de explosion
      ld hl,#_paletaExplosion
      ld (#_vtPointerAux),hl
      call _vtConfigurePaletteGame
      jr gtaGameControllerCheckMap     //paro el reloj un segundo para que no se raye y haga varios cambios de paleta

   gtaGameControllers2b:
      //restauramos la paleta actual
      //le resto 1 a la paleta actual y llamo a cambiar paleta (eso deja la paleta que estaba antes de la explosion)
      ld hl,#_gtaPaletaSeleccionada
      dec (hl)
      call _gtaCambiaPaleta



   gtaGameControllers2c:
      ld a,(#_vtClock)
      or a
      jr NZ,gtaGameControllerCheckMap
      
      ld a,(#_vtClock+1)				//VAMOS A CAMBIAR LA PALETA SOLO SI EL BYTE ALTO DE vtClock % 8 == 0 para que este un rato cada paleta
      and #7
      call Z,_gtaCambiaPaleta


   gtaGameControllerCheckMap:
      call _gtaCheckObjectsInMap    //esta funcion hace que se mire si hay que crear algun objeto del mapa si estamos cerca


      //miramos si hay que crear una persona
      ld a,(#_vtClock)
      ld l,a
      ld a,#0b00111111
      and l
      jp NZ,gtaGameControllerCheckCreaCoche

   gtaGameControllerCheckCreaPersona:
      //vamos a intentar crear una persona, pero antes miramos si no hay mucha gente ya por ahi
      ld a,(#_relaxNPCs)
      or a
      ret NZ      //si relaxNPCs>0 no creamos persona ni nada

      call _gtaPersonCounter
      cp #MAX_PERSON_IN_SCREEN_LIMIT
      ret NC
      //jr NC,gtaGameControllerCheckCreaPersonaCheckTilePersonaSal

      //antes de sacarlo en plan random, miramos si hay alguna base activa, ya que en ese caso tenemos que sacar uno concreto de la base que este en pantalla OBJ_BASE_TYPE > 0
		push ix
	
		ld hl,#GAME_OBJECTS_PTR+48     //vamos a recorrer todos los objetos y el que tenga OBJ_ID_NPC_CAR=C nos lo cargamos (le pongo OBJ_ID_NPC_CAR a 255 para que haga el step y se elimine y asi se borra el muñeco)
	   ld b,#15
      ld de,#48
        
    gtaGameControllerCheckCreaPersonaBuscaBaseBucle:
		ld a,(hl)
		cp #TYPE_AUX_OBJECT
		jr nz,gtaGameControllerCheckCreaPersonaBuscaBaseBucles2
		push hl
		pop ix
		ld a,OBJ_BASE_TYPE(ix)
		or a
		jr Z,gtaGameControllerCheckCreaPersonaBuscaBaseBucles2
		
      cp #22      //solo hasta el 22 porque las siguientes BASE no se que coño sacar
      jr NC,gtaGameControllerCheckCreaPersonaBuscaBaseBucles2

		push af
		//si llega aqui tengo en IX una base que puede sacar peña, voy a romper el bucle y crear la persona directamente
		//leo la posX y la posY y creo la persona directamente
		ld d,OBJ_POSX+1(ix)
		ld e,OBJ_POSY+1(ix)

		push de
		call _vtGetFreeGameObjPtr        //preparamos el objeto del prota
		//aqui hay que comprobar que no sea cero
      ld ix,(#_vtGameObjPtr)
      pop de
		call _gtaConfigRandomPerson
	   pop af
	   sub #15
      call _gtaConfigNPC
      pop ix		//recupero IX y vuelvo
      ret 
		
	gtaGameControllerCheckCreaPersonaBuscaBaseBucles2:
		add hl,de
    	djnz gtaGameControllerCheckCreaPersonaBuscaBaseBucle
    	
      //no he encontrado ninguna base cerca, asi que recupero IX y sigo mirando para sacar un CITIZEN random
		pop ix
	
		
      
      
      //vemos por que lado quiero sacarlo, para ello simplemente sacamos un random y vemos si acaba en 0,1,2 o 3
      call cpct_getRandom_lcg_u8_asm
      
      and #3
      jr Z,gtaGameControllerCheckCreaPersonaArriba
      dec a
      jr Z,gtaGameControllerCheckCreaPersonaAbajo
      dec a
      jr Z,gtaGameControllerCheckCreaPersonaIzq

   gtaGameControllerCheckCreaPersonaDer:
      //en L tengo un random, lo uso
      ld a,#31
      and l
      sub #4
      ld l,a
      ld a,(#_vtSceneScrollY)
      add l
      ld e,a      //meto en E el desplazamiento vertical
      ld a,(#_vtSceneScrollX)
      add #41
      ld d,a      //meto en D el desplazamiento horizontal
      jr gtaGameControllerCheckCreaPersonaCheckTilePersona

   gtaGameControllerCheckCreaPersonaIzq:
      //en L tengo un random, lo uso
      ld a,#31
      and l
      sub #4
      ld l,a
      ld a,(#_vtSceneScrollY)
      add l
      ld e,a      //meto en E el desplazamiento vertical
      ld a,(#_vtSceneScrollX)
      dec a
      ld d,a      //meto en D el desplazamiento horizontal
      jr gtaGameControllerCheckCreaPersonaCheckTilePersona

   gtaGameControllerCheckCreaPersonaArriba:
      //en L tengo un random, lo uso
      ld a,#31
      and l
      add #4
      ld l,a
      ld a,(#_vtSceneScrollX)
      add l
      ld d,a      //meto en D el desplazamiento horizontal
      ld a,(#_vtSceneScrollY)
      dec a
      dec a
      ld e,a      //meto en E el desplazamiento vertical
      jr gtaGameControllerCheckCreaPersonaCheckTilePersona


   gtaGameControllerCheckCreaPersonaAbajo:
      //en L tengo un random, lo uso
      ld a,#31
      and l
      add #4
      ld l,a
      ld a,(#_vtSceneScrollX)
      add l
      ld d,a      //meto en D el desplazamiento horizontal
      ld a,(#_vtSceneScrollY)
      add #26
      ld e,a      //meto en E el desplazamiento vertical
      //jr gtaGameControllerCheckCreaPersonaCheckTilePersona


   gtaGameControllerCheckCreaPersonaCheckTilePersona:
      
      //cuado entre aqui tengo en DE el lugar donde tengo que crear, deberia ver si el tile donde esta DE tiene un tile<16 por donde pueden andar las personas
		push de
		ld b,d
      inc b				//le sumo 1 para que se mire el tile que esta en los pies
		ld c,e
		inc c 					//le sumo 1 para que se mire el tile que esta en los pies
		call _vtCheckTileAsm		//llamo al check tile
		
		ld a,#17
		cp l
		jp C,gtaGameControllerOutWithPop
		

      

      call _vtObjectsCounter
      cp #MAX_OBJECTS_LIMIT_FOR_CREATE_NEW
      jr C,gtaGameControllerCheckCreaPersonaCheckTilePersona1_5
      //si entro aqui hay demasiados objetos, popeamos y salimos

   gtaGameControllerCheckCreaPersonaCheckTilePersonaSal:
      pop de
      ret


   gtaGameControllerCheckCreaPersonaCheckTilePersona1_5:
      ld hl,(#_vtGameObjPtr)
      push hl

      //creo la persona
      call _vtGetFreeGameObjPtr        //preparamos el objeto del prota
      
      ld ix,(#_vtGameObjPtr)

      pop hl 
      ld (#_vtGameObjPtr),hl

      pop de

      // SAFE_GET_FREE
      //xor a       //hago una comprobacion rapida por si no hubiese sitio para meter más instancias en el engine
      //.dw #0xB4DD    //or ixh      //simplemente hago 0 or IXH (que tiene que ser 0xB5 o por ahi), si el or da Zero es que no hubo espacio para agregarlo, podemos eliminarlo
      //jp Z,gtaGameControllerOut

      
      call _gtaConfigRandomPerson
      

   gtaGameControllerCheckCreaPersonaCheckTilePersonas5:
      jp gtaGameControllerOut






      


      //miramos si hay que crear un coche
   gtaGameControllerCheckCreaCoche:
      //ld a,#0b00011111
      //and l
      dec a
      jp NZ,gtaGameControllerOut

      ld a,(#_relaxNPCs)
      or a
      jp NZ,gtaGameControllerOut      //si relaxNPCs>0 no creamos persona ni nada


      ld a,#NUMERO_INTENTOS_SACAR_COCHE
      ld (#_reintentos),a

   gtaGameControllerCheckCreaCocheTry:
      //vamos a intentar crear un coche
      //vemos por que lado quiero sacarlo, para ello simplemente sacamos un random y vemos si acaba en 0,1,2 o 3
      call cpct_getRandom_lcg_u8_asm
      
      and #3
      jr Z,gtaGameControllerCheckCreaCocheArriba
      dec a
      jr Z,gtaGameControllerCheckCreaCocheAbajo
      dec a
      jr Z,gtaGameControllerCheckCreaCocheIzq

   gtaGameControllerCheckCreaCocheDer:
      //en L tengo un random, lo uso
      ld a,#31  
      and l    //esto da un numero entre 0 y 31
      sub #3   //esto da un numero entre -3 y 28
      ld l,a
      ld a,(#_vtSceneScrollY)
      add l
      ld e,a      //meto en E el desplazamiento vertical
      ld a,(#_vtSceneScrollX)
      add #38
      ld d,a      //meto en D el desplazamiento horizontal
      jr gtaGameControllerCheckCreaCocheCheckTileCoche

   gtaGameControllerCheckCreaCocheIzq:
      //en L tengo un random, lo uso
      ld a,#31
      and l    //esto da un numero entre 0 y 31
      sub #3      //esto da un numero entre -3 y 28
      ld l,a
      ld a,(#_vtSceneScrollY)
      add l
      ld e,a      //meto en E el desplazamiento vertical
      ld a,(#_vtSceneScrollX)
      //dec a
      ld d,a      //meto en D el desplazamiento horizontal
      jr gtaGameControllerCheckCreaCocheCheckTileCoche

   gtaGameControllerCheckCreaCocheArriba:
      //en L tengo un random, lo uso
      ld a,#31    
      and l    //esto da un numero entre 0 y 31
      add #4      //esto da un numero entre 4 y 35
      ld l,a
      ld a,(#_vtSceneScrollX)
      add l
      ld d,a      //meto en D el desplazamiento horizontal
      ld a,(#_vtSceneScrollY)
      dec a
      ld e,a      //meto en E el desplazamiento vertical
      jr gtaGameControllerCheckCreaCocheCheckTileCoche


   gtaGameControllerCheckCreaCocheAbajo:
      //en L tengo un random, lo uso
      ld a,#31
      and l       //esto da un numero entre 0 y 31
      add #4      //esto da un numero entre 4 y 35
      ld l,a
      ld a,(#_vtSceneScrollX)
      add l
      ld d,a      //meto en D el desplazamiento horizontal
      ld a,(#_vtSceneScrollY)
      add #23
      ld e,a      //meto en E el desplazamiento vertical
      //jr gtaGameControllerCheckCreaCocheCheckTileCoche


   gtaGameControllerCheckCreaCocheCheckTileCoche:
      
      //cuado entre aqui tengo en DE el personaje que tengo que crear, deberia ver si el tile donde esta DE tiene un tile<16 por donde pueden andar las personas
		push de
		ld b,d
      inc b				//le sumo 1 para que se mire el tile que esta en los pies
		ld c,e
		inc c 					//le sumo 1 para que se mire el tile que esta en los pies
		//call _vtCheckTileAsm		//llamo al check tile
      call _vtCheckMetaTileAsm

      ld a,h      
      ld (#_auxTemp),a        //me guardo el metatile, que lo voy a usar luego para saber el angulo

      
      //si el metatile es menor de 22 es un metatile de carretera y podemos sacar un coche
      cp #22
      jr C,gtaGameControllerCheckCreaCocheCreaObjeto

      pop de

      ld hl,#_reintentos
      dec (hl)
      jp NZ,gtaGameControllerCheckCreaCocheTry

      //jp gtaGameControllerOutWithPop      //sino mejor no sacar el coche
      jp gtaGameControllerOut      //sino mejor no sacar el coche




   gtaGameControllerCheckCreaCocheCreaObjeto:

      push af


      call _gtaCarCounter
      cp #MAX_CARS_IN_SCREEN_LIMIT
      jr NC,gtaGameControllerCheckCreaCocheCreaObjetoSal

      call _vtObjectsCounter
      cp #MAX_OBJECTS_LIMIT_FOR_CREATE_NEW
      jr C,gtaGameControllerCheckCreaCocheCreaObjeto1_5
   
   gtaGameControllerCheckCreaCocheCreaObjetoSal:
      //si entro aqui hay demasiados objetos, popeamos y salimos
      pop bc
      pop de
      ret

   gtaGameControllerCheckCreaCocheCreaObjeto1_5:
      
      ld hl,(#_vtGameObjPtr)
      push hl

      //creo el coche
      call _vtGetFreeGameObjPtr        //preparamos el objeto del prota
      
      ld ix,(#_vtGameObjPtr)

      pop hl
      ld (#_vtGameObjPtr),hl
      

      pop bc
      pop de

      // SAFE_GET_FREE
      //xor a       //hago una comprobacion rapida por si no hubiese sitio para meter más instancias en el engine
      //.dw #0xB4DD    //or ixh      //simplemente hago 0 or IXH (que tiene que ser 0xB5 o por ahi), si el or da Zero es que no hubo espacio para agregarlo, podemos eliminarlo
      //jp Z,gtaGameControllerOut

      

      //antes de sacar el coche, voy a ver si tengo una base cerca para sacar algun coche personalizado o sacarlo generico
      //antes de sacarlo en plan random, miramos si hay alguna base activa, ya que en ese caso tenemos que sacar uno concreto de la base que este en pantalla OBJ_BASE_TYPE > 0
		/*push ix
      push de
	
		ld hl,#GAME_OBJECTS_PTR+48     //vamos a recorrer todos los objetos y el que tenga OBJ_ID_NPC_CAR=C nos lo cargamos (le pongo OBJ_ID_NPC_CAR a 255 para que haga el step y se elimine y asi se borra el muñeco)
	   ld b,#15
      ld de,#48
        
    gtaGameControllerCheckCreaCocheCreaObjeto1_5BaseBucle:
		ld a,(hl)
		cp #TYPE_AUX_OBJECT
		jr nz,gtaGameControllerCheckCreaCocheCreaObjeto1_5BaseBucles2
		push hl
		pop ix
		ld a,OBJ_BASE_TYPE(ix)
		or a
		jr Z,gtaGameControllerCheckCreaCocheCreaObjeto1_5BaseBucles2
		
      cp #22      //solo hasta el 22 porque las siguientes BASE no se que coño sacar
      jr NC,gtaGameControllerCheckCreaCocheCreaObjeto1_5BaseBucles2

      
	   inc a       //le añado 1 para que cuadre con los CAR_POLICE CAR_ARCADE, etc...
      call _gtaConfigNPC
      pop de

      ld OBJ_POSX+1(ix),d     //le aplicamos la posicion donde debe salir
      ld OBJ_POSY+1(ix),e
      
      //como es un coche que ha salido de una base le pongo que lo conduzca alguien random (luego si lo saco del coche detectare que tiene que salir un npc)
      ld OBJ_PARENT_PTR+1(ix),#255        //le pongo que lleva alguien dentro random (luego al sacarlo detectare que es un NPC concreto)

      pop ix		//recupero IX y vuelvo
      jp gtaGameControllerOut
   
   gtaGameControllerCheckCreaCocheCreaObjeto1_5BaseBucles2:
		add hl,de
    	djnz gtaGameControllerCheckCreaCocheCreaObjeto1_5BaseBucle
    	
      //no he encontrado ninguna base cerca, asi que recupero IX y sigo sacando un coche random
      pop de
		pop ix*/




      ld OBJ_ACTIVE_TYPE(ix),#TYPE_CAR
      ld OBJ_PARENT_PTR(ix),#0xFF         //le pongo 255 y 255 en el puntero, esto es para que el coche tenga su propia IA sin necesidad de que vaya nadie montado
      ld OBJ_PARENT_PTR+1(ix),#0xFF       //si intento robar este coche, entonces ya creo un personaje en ese momento para que parezca que iba dentro (pero me ahorro un objeto)

      ld OBJ_LIFE(ix),#CAR_START_LIFE

      ld OBJ_POSX+1(ix),d
      ld OBJ_POSY+1(ix),e
      ld a,#3
      ld OBJ_WIDTH(ix),a
      ld OBJ_HEIGHT(ix),a
      ld OBJ_FLAGS(ix),#16     //FLAG_AUTO_MOVE 

      ld hl,#_gtaCocheStepFunc
      ld OBJ_STEP_FUNC(ix),l
      ld OBJ_STEP_FUNC+1(ix),h
      call cpct_getRandom_lcg_u8_asm
      and #7      //kiero un random del 0 al 7 asi que hago un and 7 al random
      ld hl,#_carColorsPrepare         //ahora hago A=carColorsPrepare[random_0_7]
      add l
      jr NC,gtaGameControllerCheckCreaCocheCreaObjetos2
      inc h
   gtaGameControllerCheckCreaCocheCreaObjetos2:
      ld l,a
      ld a,(hl)
      ld OBJ_CUSTOM_COLOR(ix),a        //y ahora uso esa tabla para sacar los colores del coche (son colores más normales y logicos para un coche)
      
      //ld OBJ_FRAMES_FOR_NEXT_STATE(ix),a

      ld hl,#_gtaCocheColisionFunc
      ld OBJ_COLISION_FUNC(ix),l
      ld OBJ_COLISION_FUNC+1(ix),h
      ld hl,#_gtaCocheDrawFunc
      ld OBJ_PREPARE_FUNC(ix),l
      ld OBJ_PREPARE_FUNC+1(ix),h
      //ld hl,#_iaPersonPaseando
      //ld OBJ_MACHINE_STATE_PTR(ix),l
      //ld OBJ_MACHINE_STATE_PTR+1(ix),h
      
      call cpct_getRandom_lcg_u8_asm
      and #7


      ld OBJ_SELECTED_CAR(ix),a    
      add a
      ld hl,#_carPropAnim
      add l
      jr NC,gtaGameControllerCheckCreaCocheCreaObjetos3
      inc h
   gtaGameControllerCheckCreaCocheCreaObjetos3:
      ld l,a
      ld e,(hl)
      inc hl
      ld d,(hl)

      
      ld OBJ_ANIMATION_LIST(ix),e
      ld OBJ_ANIMATION_LIST+1(ix),d
      //ld OBJ_FRAME_SPEED(ix),#0
      ld OBJ_ANIMATION_MAX(ix),#8
      

      //aqui saco un random, y me quedo con los 4 bits bajos
      call cpct_getRandom_lcg_u8_asm
      and #15

      

      cp #6
      jr NC,gtaGameControllerCheckCreaCocheCreaObjetos3_1     //si es mayor o igual a 6 saco el coche tal cual está, osea en plan random

      

      //si es de 0 a 5 saco uno personalizado
      add #17     //para colocarme a partir del CAR_POLICE
      call _gtaConfigNPC      //llamo al config NPC y eso hara el coche especial en IX y ya está

      //le pongo conductor
      ld OBJ_PARENT_PTR+1(ix),#255        //le pongo que lleva alguien dentro random (luego al sacarlo detectare que es un NPC concreto)

      
      
   gtaGameControllerCheckCreaCocheCreaObjetos3_1:
//0- todos hacia derecha
//1- todos hacia abajo
//2- todos hacia izq
//3- todos hacia arriba
//5- horizontal mitad y mitad
//7- vertical mitad y mitad
//el resto aqui no puede darse el caso

      ld a,(#_auxTemp)
      ld hl,#_metatileTraficDirection
      add l
      jr NC,gtaGameControllerCheckCreaCocheCreaObjetos4
      inc h
   gtaGameControllerCheckCreaCocheCreaObjetos4:
      ld l,a
      ld a,(hl)
      cp #4
      jr NC,gtaGameControllerCheckCreaCocheCreaObjetosDir2
      //si entro aqui es de 0 a 3 y simplemente tengo que desplazarlo 2 veces y ya tengo el torque
      rrca     //desplazo 2 veces a la der y ya tengo el nuevo angulo (es como multiplicarlo por 64)
      rrca
      ld b,a

      jr gtaGameControllerCheckCreaCocheCreaObjetosDirFin
   
   gtaGameControllerCheckCreaCocheCreaObjetosDir2:
      cp #5
      jr NZ,gtaGameControllerCheckCreaCocheCreaObjetosDir3
      //si entra aqui es horizontal mitad y mitad, osea, si e modulo 7 es menor que 4, vamos pa la izq, y sino pa la derecha
      ld a,#7
      and e
      bit 2,a 
      jr NZ,gtaGameControllerCheckCreaCocheCreaObjetosDir3izq
   gtaGameControllerCheckCreaCocheCreaObjetosDir3der:
      ld b,#0b10000000
      //ajustamos la posicion para que no salga en la acera (en este caso el modulo 7 debe ser 1)
      ld a,e
      and #0b11111000
      add #1
      ld OBJ_POSY+1(ix),a
      jr gtaGameControllerCheckCreaCocheCreaObjetosDirFin
   gtaGameControllerCheckCreaCocheCreaObjetosDir3izq:
      ld b,#0b00000000
      //ajustamos la posicion para que no salga en la acera (en este caso el modulo 7 debe ser 4)
      ld a,e
      and #0b11111000
      add #4
      ld OBJ_POSY+1(ix),a
      jr gtaGameControllerCheckCreaCocheCreaObjetosDirFin

   gtaGameControllerCheckCreaCocheCreaObjetosDir3:
      //si entro aqui es directamente 7, no hay más opciones, osea vertical mitad y mitad
      ld a,#7
      and d
      bit 2,a 
      jr NZ,gtaGameControllerCheckCreaCocheCreaObjetosDir4arr
   gtaGameControllerCheckCreaCocheCreaObjetosDir4aba:
      ld b,#0b01000000
      //ajustamos la posicion para que no salga en la acera (en este caso el modulo 7 debe ser 1)
      ld a,d
      and #0b11111000
      add #1
      ld OBJ_POSX+1(ix),a
      jr gtaGameControllerCheckCreaCocheCreaObjetosDirFin
   gtaGameControllerCheckCreaCocheCreaObjetosDir4arr:
      ld b,#0b11000000
      //ajustamos la posicion para que no salga en la acera (en este caso el modulo 7 debe ser 4)
      ld a,d
      and #0b11111000
      add #4
      ld OBJ_POSX+1(ix),a
      //jr gtaGameControllerCheckCreaCocheCreaObjetosDirFin  no hace falta

   gtaGameControllerCheckCreaCocheCreaObjetosDirFin:
      ld OBJ_CAR_TORQUE(ix),b          //en B tengo el angulo que lo calcule antes


      jp gtaGameControllerOut

	gtaGameControllerOutWithPop:
		pop de
   gtaGameControllerOut:
      ret
   __endasm;
}




//en vez de avanzar con OBJ_SPEEDX y OBJ_SPEEDY retrocede 1 vez al objeto IX
void gtaVuelveAtrasUnPaso() __naked {
   __asm
      ld h,OBJ_POSX+1(ix)
      ld l,OBJ_POSX(ix)
      ld d,OBJ_SPEEDX+1(ix)
      ld e,OBJ_SPEEDX(ix)

      xor a
      sbc hl,de

      ld OBJ_POSX+1(ix),h
      ld OBJ_POSX(ix),l

      ld h,OBJ_POSY+1(ix)
      ld l,OBJ_POSY(ix)
      ld d,OBJ_SPEEDY+1(ix)
      ld e,OBJ_SPEEDY(ix)

      xor a
      sbc hl,de

      ld OBJ_POSY+1(ix),h
      ld OBJ_POSY(ix),l

      ret
   __endasm;
}


//Devuelve en L el numero de tile que esta justo debajo del personaje
u8 checkTilePerson() __naked {
   __asm
      //en ix va el jugador (y si tuviera coche va en iy)
      //+1+1 el desplazamiento y llamamos a i
      ld b,OBJ_POSX+1(ix)
      inc b
      ld c,OBJ_POSY+1(ix)
      inc c 
      inc c
      jp _vtCheckTileAsm
      //ret no hace falta
   __endasm;
}


//Devuelve en L el numero de tile que esta justo debajo del coche
u8 checkTileMyCarFull() __naked {
   __asm
      ld b,OBJ_POSX+1(ix)
      ld c,OBJ_POSY+1(ix)
      ld d,b
      ld e,c
      inc d
      inc d
      inc d
      inc e
      inc e
      inc e 
      ld h,#MAX_METATILE_NO_COLISION_CAR	//ANTES 32 maximo metatile para considerar colision
      ld l,#64  //maximo tile para considerar colision
      jp _vtCheckTileZoneAsm
      //ret
   __endasm;
}






//Devuelve en L el numero de tile que esta justo debajo del coche
u8 checkTileMyCarVer() __naked {
   __asm
      ld b,OBJ_POSX+1(ix)
      ld c,OBJ_POSY+1(ix)
      inc b
      ld d,b
      ld e,c
      inc d
      
      inc e
      inc e
      inc e 
      ld h,#MAX_METATILE_NO_COLISION_CAR	//maximo metatile para considerar colision
      ld l,#64  //maximo tile para considerar colision
      jp _vtCheckTileZoneAsm
      //ret
   __endasm;
}

u8 checkTileMyCarHor() __naked {
   __asm
      ld b,OBJ_POSX+1(ix)
      ld c,OBJ_POSY+1(ix)
      inc c
      ld d,b
      ld e,c
      inc d
      inc d
      inc d
      inc e
       
      ld h,#MAX_METATILE_NO_COLISION_CAR	//maximo metatile para considerar colision
      ld l,#64  //maximo tile para considerar colision
      jp _vtCheckTileZoneAsm
      //ret
   __endasm;
}


//cuenta el numero de coches que hay activos y lo devuelve en A
void gtaCarCounter() __naked {
   __asm
      ld hl,#GAME_OBJECTS_PTR
      ld de,#48
      ld b,#16    //miramos los 16 objetos
      ld c,#0     //aqui vamos a ir contando los que llevamos activos
   gtaCarCounterBucle:
      ld a,(hl)
      bit 5,a        //miro si es 32 (bit 5 activo)      TYPE_CAR 32
      jr Z,gtaCarCounterBucle2
      inc c    //no es cero, contamos uno más

   gtaCarCounterBucle2:
      add hl,de      //pasamos al siguiente
      djnz gtaCarCounterBucle    //b=b-1 y si no es cero vuelve al bucle

      ld a,c         //devolvemos el resultado en A

      ret
   __endasm;
}

//cuenta el numero de personas que hay activos y lo devuelve en A
void gtaPersonCounter() __naked {
   __asm
      ld hl,#GAME_OBJECTS_PTR
      ld de,#48
      ld b,#16    //miramos los 16 objetos
      ld c,#0     //aqui vamos a ir contando los que llevamos activos
   gtaPersonCounterBucle:
      ld a,(hl)    
      bit 2,a
      jr Z,gtaPersonCounterBucle2
      inc c    //no es cero, contamos uno más

   gtaPersonCounterBucle2:
      add hl,de      //pasamos al siguiente
      djnz gtaPersonCounterBucle    //b=b-1 y si no es cero vuelve al bucle

      ld a,c         //devolvemos el resultado en A

      ret
   __endasm;
}



//input IX debe tener un puntero a una instancia vacia para usarla para configurar la persona
//DE debe tener el posX y posY (en tiles)
void gtaConfigRandomPerson() __naked {
   __asm

      call cpct_getRandom_lcg_u8_asm         //saco un numero rand para saber si va a ser hombre o mujer
      and #TYPE_MAN                          //me quedo solo con ese bit, el resto me la pela
      add #TYPE_CITIZEN                      //le añado el bit del CITIZEN

      ld OBJ_LIFE(ix),#PERSON_START_LIFE

      ld OBJ_ACTIVE_TYPE(ix),a               //se lo asigno al tipo

      bit 1,a                                //miro si es mujer o hombre
      jr NZ,gtaConfigRandomPersonsMan

      ld hl,#_animWomanWalk                   //es mujer, le pongo directamente el pelo largo por simplificar de momento
      ld OBJ_ANIMATION_LIST(ix),l
      ld OBJ_ANIMATION_LIST+1(ix),h

      
      ld OBJ_CUSTOM_COLOR_2(ix),#CUSTOM_A_PAL_8          //pelo gris
      call cpct_getRandom_lcg_u8_asm               //configuro un pelo largo rubio/moreno/gris
      cp #192
      jr NC,gtaConfigRandomPersonsWomans2
      ld OBJ_CUSTOM_COLOR_2(ix),#CUSTOM_A_PAL_0          //pelo negro
      cp #128
      jr NC,gtaConfigRandomPersonsWomans2
      ld OBJ_CUSTOM_COLOR_2(ix),#CUSTOM_A_PAL_11         //pelo rubio
      cp #64
      jr NC,gtaConfigRandomPersonsWomans2
      ld OBJ_CUSTOM_COLOR_2(ix),#CUSTOM_A_PAL_9          //peliroja


   gtaConfigRandomPersonsWomans2:
      ld hl,#_animHairWalkLargo
      ld OBJ_ANIMATION_LIST_2(ix),l
      ld OBJ_ANIMATION_LIST_2+1(ix),h

      jr gtaConfigRandomPersons0                //ya le he configurado la animacion y el pelo, sigo configurando el muñeco







   gtaConfigRandomPersonsMan:
      ld hl,#_animManWalk                       //es hombre, le pongo algun pelo random
      ld OBJ_ANIMATION_LIST(ix),l
      ld OBJ_ANIMATION_LIST+1(ix),h



      call cpct_getRandom_lcg_u8_asm               //configuro un pelo random
      cp #192
      jr NC,gtaConfigRandomPersonCalvo
      ld OBJ_CUSTOM_COLOR_2(ix),#CUSTOM_A_PAL_11
      cp #128
      jr NC,gtaConfigRandomPersons2
      ld OBJ_CUSTOM_COLOR_2(ix),#CUSTOM_A_PAL_0

   gtaConfigRandomPersons2:
      ld hl,#_animHairWalkSimple
      ld OBJ_ANIMATION_LIST_2(ix),l
      ld OBJ_ANIMATION_LIST_2+1(ix),h
      call cpct_getRandom_lcg_u8_asm
      cp #128
      jr C,gtaConfigRandomPersons3
      
      ld hl,#_animHairWalkSimple
      ld OBJ_ANIMATION_LIST_2(ix),l
      ld OBJ_ANIMATION_LIST_2+1(ix),h
      jr gtaConfigRandomPersons0

   gtaConfigRandomPersons3:
      cp #64
      jr C,gtaConfigRandomPersons4
      ld hl,#_animHairWalkAfro
      ld OBJ_ANIMATION_LIST_2(ix),l
      ld OBJ_ANIMATION_LIST_2+1(ix),h
      jr gtaConfigRandomPersons0

   gtaConfigRandomPersons4:
      ld hl,#_animHairWalkPunk
      ld OBJ_ANIMATION_LIST_2(ix),l
      ld OBJ_ANIMATION_LIST_2+1(ix),h
      call cpct_getRandom_lcg_u8_asm
      ld OBJ_CUSTOM_COLOR_2(ix),a
      jr gtaConfigRandomPersons0

   gtaConfigRandomPersonCalvo:
      ld OBJ_ANIMATION_LIST_2(ix),#0









   gtaConfigRandomPersons0:                     //seguimos configurando todas las cosillas del personaje
      
      ld hl,#_gtaPersonaIAStepFunc
      ld OBJ_STEP_FUNC(ix),l
      ld OBJ_STEP_FUNC+1(ix),h
      call cpct_getRandom_lcg_u8_asm
      ld OBJ_CUSTOM_COLOR(ix),a
      ld OBJ_FRAMES_FOR_NEXT_STATE(ix),a

      ld hl,#_gtaColisionPersonaFunc
      ld OBJ_COLISION_FUNC(ix),l
      ld OBJ_COLISION_FUNC+1(ix),h
      //ld hl,#_gtaPersonaDrawFunc
      ld hl,#_vtDrawObjectFunc
      ld OBJ_PREPARE_FUNC(ix),l
      ld OBJ_PREPARE_FUNC+1(ix),h
      ld hl,#_iaPersonPaseando
      ld OBJ_MACHINE_STATE_PTR(ix),l
      ld OBJ_MACHINE_STATE_PTR+1(ix),h
      


      
      ld OBJ_FRAME_SPEED(ix),#96
      ld OBJ_ANIMATION_MAX(ix),#4
      
      ld OBJ_POSX+1(ix),d
      ld OBJ_POSY+1(ix),e
      ld a,#2
      ld OBJ_WIDTH(ix),a
      ld OBJ_HEIGHT(ix),a
      ld OBJ_FLAGS(ix),#48     //FLAG_AUTO_MOVE | FLAG_AUTO_ANIM


      



      
   //gtaConfigRandomPersons5:
      ret
   __endasm;
}


void gtaPreDrawFunc() __naked {
   __asm

      ld hl,#PREDRAW_PIXELS_ZONE
      ld b,#20
      
	gtaPreDrawFuncWritePixelsBucle:
		xor a
		or (hl)
		jr Z,gtaPreDrawFuncWritePixelsBucles2
     	
     	dec (hl)		//decremento 1 
     	inc hl
     	ld e,(hl)
     	inc hl
     	ld d,(hl)
     	inc hl
      xor a
     	ld (de),a		//pinto el pixel con el 0 negro
     	jr gtaPreDrawFuncWritePixelsBucles3
     	
	gtaPreDrawFuncWritePixelsBucles2:
		inc hl
		inc hl
		inc hl 
	gtaPreDrawFuncWritePixelsBucles3:
		djnz gtaPreDrawFuncWritePixelsBucle
      


      //ld hl,#_restauraLuces
      ld hl,#RESTAURA_LUCES
      ld a,(hl)
   gtaPreDrawFuncRestauraLucesBucle:
      or a
      ret Z       //si no hay luces que restaurar, pa fuera del tiron

      //si no es cero vamos a hacer un bucle restaurando luces hasta que me encuentre con un cero
      ld a,(#_vtSceneScrollX)
      ld b,a
      ld a,(hl)
      sub b

      cp #37      //OJO! no quitar esta comprobacion porque sino puede petar el juego (supongo que porke dibuja tiles donde no debe)
      jr C,gtaPreDrawFuncRestauraLucesBucles1         //no puede ser mayor de 38 el init x, si es mayor lo pongo en 38 que es el maximo
      ld a,#37          
   gtaPreDrawFuncRestauraLucesBucles1:
      or a 
      jr NZ,gtaPreDrawFuncRestauraLucesBucles2
      inc a //no dejo que sea 0, le pongo 1

   gtaPreDrawFuncRestauraLucesBucles2:
      ld (#_vtPushTilesIniX),a


      add #4
      cp #38                                          //el endx no puede ser mayor de 39, si es mayor lo limito a 39
      jr C,gtaPreDrawFuncRestauraLucesBucles3
      ld a,#38    //el end no puede ser mayor de 39 que la lia...
   gtaPreDrawFuncRestauraLucesBucles3:
      ld (#_vtPushTilesEndX),a

      inc hl
   
      ld a,(#_vtSceneScrollY)
      ld b,a
      ld a,(hl)
      sub b

      cp #22      //OJO! no quitar esta comprobacion porque sino puede petar el juego (supongo que porke dibuja tiles donde no debe)
      jr C,gtaPreDrawFuncRestauraLucesBucles4
      ld a,#22       //maximo 23 de initY
   gtaPreDrawFuncRestauraLucesBucles4:
      or a
      jr NZ,gtaPreDrawFuncRestauraLucesBucles5
      inc a

   gtaPreDrawFuncRestauraLucesBucles5:
      ld (#_vtPushTilesIniY),a


      add #4

      cp #23      //OJO! no quitar esta comprobacion porque sino puede petar el juego (supongo que porke dibuja tiles donde no debe)
      jr C,gtaPreDrawFuncRestauraLucesBucles6
      ld a,#23       //maximo 24 de endY
   gtaPreDrawFuncRestauraLucesBucles6:

      ld (#_vtPushTilesEndY),a

      inc hl

      push hl 

      call _vtPushTilesToDraw    //envio a redibujar esos tiles

      pop hl



      ld a,(hl)      //leo la siguiente
      jr gtaPreDrawFuncRestauraLucesBucle

      
   __endasm;
}


//funcion que se llama al final de cada step del juego, de momento la voy a usar para dibujar las conversaciones
void gtaPostDrawFunc() __naked {
   __asm

      //miro si es de noche para poner las luces o no:
      //ld hl,#_restauraLuces
      ld hl,#RESTAURA_LUCES
      ld (hl),#0     //pongo que no se va a dibujar ninguna luz de momento, luego ya veremos.

      ld a,(#_gtaPaletaSeleccionada)
      sub #2
      jp NZ,gtaPostDrawFuncs0

      xor a                //PONGO QUE LLEVO 0 LUCES RENDERIZADAS
      ld (#lucesRenderizadas),a


      //si entro aqui es de noche, voy a recorrer todas las instancias para ver si existe algun coche, si existe alguno puedo dibujarle las luces
      ld ix,#GAME_OBJECTS_PTR+48     //vamos a recorrer todos los objetos y el que tenga OBJ_ID_NPC_CAR=C nos lo cargamos (le pongo OBJ_ID_NPC_CAR a 255 para que haga el step y se elimine y asi se borra el muñeco)
	   ld b,#15
      ld de,#48
      
      
        
   gtaPostDrawFuncLucess1:
      bit 5,OBJ_ACTIVE_TYPE(ix)     //miro si es un coche
      jr NZ,gtaPostDrawFuncLucess1b
      ld a,OBJ_SELECTED_WEAPON(ix)
      inc a
      jr NZ,gtaPostDrawFuncLucess3
      
      ld OBJ_FRAME_INDEX+1(ix),#8      //trucazo para poner una farola simplemente


   
   gtaPostDrawFuncLucess1b:
      //si entro aqui es un coche, calculo donde tengo que poner la luz y la dibujo
      push de
      push bc


		//si no tiene redrawX es que el objeto del que viene la luz no se dibujo, asi que tampoco vamos a dibujar la luz
		bit 1,OBJ_FLAGS(ix)
		jr Z,gtaPostDrawFuncLucess2			//si el original no se ha dibujado, la luz tampoco la vamos a dibujar


      //calculamos de forma rapida donde debe pintarse la luz
      push hl

      ld e,OBJ_FRAME_INDEX+1(ix)
      ld d,#0
      ld hl,#_despXLuzCoche
      add hl,de
      ld b,(hl)      //desplazamiento horizontal en B
      ld hl,#_despYLuzCoche
      add hl,de
      ld c,(hl)      //desplazamiento vertical en C

      pop hl



      ld a,(#_vtSceneScrollX)
      ld e,a
      ld a,OBJ_POSX+1(ix)
      add b
      sub e
      cp #35
      jr NC,gtaPostDrawFuncLucess2     //no se puede dibujar, se sale en horizontal
      

      ld a,(#_vtSceneScrollY)
      ld e,a
      ld a,OBJ_POSY+1(ix)
      add c
      sub e
      cp #21
      jr NC,gtaPostDrawFuncLucess2     //no se puede dibujar, se sale en vertical
      

      //si llego aqui puedo pintarlo
      ld a,OBJ_POSY+1(ix)     //calculo posicion vertical en DE
      add c
      ld d,a
      ld e,OBJ_POSY(ix)

      ld a,OBJ_POSX+1(ix)     //calculo posicion horizontal en BC
      add b
      ld b,a
      ld c,OBJ_POSX(ix)
      
      ld (hl),a      //guardo la posicion horizontal y vertical en restauraLuces
      inc hl
      ld (hl),d
      inc hl


      push hl

      call _vtGetGameScreenPtrBCDE
      //ex de,hl    //dejo en DE la memoria de la pantalla
      push hl

      ld e,OBJ_FRAME_INDEX+1(ix)
      ld d,#0
      ld hl,#_animLuzCoche
      add hl,de 
      add hl,de

      ld e,(hl)      //leo el sprite de luz a usar
      inc hl 
      ld d,(hl)

      pop hl
      
      call _gtaDrawLight
      
      ld hl,#lucesRenderizadas     //aunmento en 1 las luces
      inc (hl)

      pop hl

   gtaPostDrawFuncLucess2:
      pop bc
      pop de

      ld a,OBJ_FRAME_INDEX+1(ix)
      cp #8
      jr NZ,gtaPostDrawFuncLucess3
      ld OBJ_FRAME_INDEX+1(ix),#0


    gtaPostDrawFuncLucess3:
      ld a,(#lucesRenderizadas)
      cp #MAX_LUCES
      jr NC,gtaPostDrawFuncLucess4        //si ya hemos renderizado el maximo de luces nos vamos para no hacer que el engine explote

      add ix,de
      dec b
      jp NZ,gtaPostDrawFuncLucess1
      //djnz gtaPostDrawFuncLucess1

   gtaPostDrawFuncLucess4:
      ld (hl),#0     //seteo a 0 donde me haya quedado HL (la mayoria de las veces sera al principio de restauraLuces)



   gtaPostDrawFuncs0:
      //ld hl,#_talkFrase
      ld hl,#TALK_FRASE
      ld a,(hl)
      or a
      ret Z

      /*ld a,(#_talkTime)
      cp #200
      ret NC      //si talkTime>200 no tengo que mostrar ná*/

   gtaPostDrawFuncs1:
      ld ix,(#_talkInstancePtr)
      .dw #0x7cdd    //ld a,ixh
      or a
      ret z    //si no tenemos instanacia en el talkInstance no tenemos que hacer ná, pa fuera

      ld (#_vtGameObjPtr),ix


      //ld hl,#_talkFrase
      ld hl,#TALK_FRASE
      call _vtGetLengthHL     //meto en A el tamaño de la cadena
      ld (#_vtStringPtr),hl
   
      //ld (#_talkLastSize),a
   
      //srl a       //divido el tamaño entre 2
      sub #40     //le resto 20 con lo cual me debe dar algo entre -1 y -39
      neg         //lo paso a positivo y ya tengo el desplazamiento que necesito para centrar el texto
      ld b,a      //y lo meto en B como el desplazamiento que tengo que hacer

   

      ld	hl, (_vtVidMemStart)       // HL = vtVidMemStart
      ld a,#0xC0
      add h
      ld h,a
      ld a,l
      add #0x50      //salto al segundo caracter (a ver si asi se ve algo....)
      jr NC,gtaPostDrawFuncs2
      inc h
   gtaPostDrawFuncs2:
      ld l,a


      push bc

      push hl
      

      //dibujo un bello cuadrado negro de fondo para que el texto destaque más
      ex de,hl       //(2B DE) memory	Video memory pointer to the upper left box corner byte
      xor a          //(1B A ) colour_pattern	1-byte colour pattern (in screen pixel format) to fill the box with
      ld c,#40       //(1B C ) width	Box width in bytes [1-64] (Beware!  not in pixels!)
      ld b,#15       //(1B B ) height	Box height in bytes (>0)
      call cpct_drawSolidBox_asm

      pop hl
      push hl
      ld de,#40
      add hl,de

      //dibujo un bello cuadrado negro de fondo para que el texto destaque más
      ex de,hl       //(2B DE) memory	Video memory pointer to the upper left box corner byte
      xor a          //(1B A ) colour_pattern	1-byte colour pattern (in screen pixel format) to fill the box with
      ld c,#40       //(1B C ) width	Box width in bytes [1-64] (Beware!  not in pixels!)
      ld b,#15       //(1B B ) height	Box height in bytes (>0)
      call cpct_drawSolidBox_asm


      
      pop hl

      pop bc
      //ld hl,(#_vtPointerAux)
      ld a,l
      add b
      jr NC,gtaPostDrawFuncs3
      inc h
   gtaPostDrawFuncs3:
      add #0x50      //salto al segundo caracter (a ver si asi se ve algo....)
      jr NC,gtaPostDrawFuncs4
      inc h
   gtaPostDrawFuncs4:
      ld l,a
      inc hl 
      inc hl 



      ld (#_vtScreenPtr),hl


      ld a,#0xFF        //el color de las letras
      ld (#_vtDrawSpriteCustomColor1),a

      call _vtDrawString

      //ld hl,#_talkTime
      //inc (hl)

      ld a,#1                    //paramos el juego
      ld (#_vtStopGame),a




      //y ahora dibujo quien esta diciendo eso a la izquierda, me situo arriba a la izquierda para dibujar quien lo dice
      ld	hl, (_vtVidMemStart)       // HL = vtVidMemStart
      ld a,#0xC0
      add h
      ld h,a
      ld a,l
      add #0x50      //salto al segundo caracter (a ver si asi se ve algo....)
      jr NC,gtaPostDrawFuncs6
      inc h
   gtaPostDrawFuncs6:
      ld l,a


      ld ix,(#_talkInstancePtr)              //PREPARO TODO PARA DIBUJAR EL PERSONAJE QUE ESTA HABLANDO ARRIBA A LA DERECHA

      ld (#_vtScreenPtr),hl

      ld a,OBJ_CUSTOM_COLOR(ix)
      and #0b01010101
      ld b,a
      rlca
      or b
      ld (#_vtDrawSpriteCustomColor1),a

      ld a,OBJ_CUSTOM_COLOR(ix)
      and #0b10101010
      ld b,a
      rrca
      or b
      ld (#_vtDrawSpriteCustomColor2),a
      
      ld l,OBJ_ANIMATION_LIST(ix)
      ld h,OBJ_ANIMATION_LIST+1(ix)
      ld a,(hl)
      inc hl
      ld h,(hl)
      ld l,a

      ld (#_vtSpritePtr),hl

      call _vtRenderTS

      //y miro si tengo que renderizar el pelazo
      ld ix,(#_talkInstancePtr)   
      ld a,OBJ_ANIMATION_LIST_2+1(ix)
      or a
      ret Z

      ld a,OBJ_CUSTOM_COLOR_2(ix)
      and #0b01010101
      ld b,a
      rlca
      or b
      ld (#_vtDrawSpriteCustomColor1),a

      ld a,OBJ_CUSTOM_COLOR_2(ix)
      and #0b10101010
      ld b,a
      rrca
      or b
      ld (#_vtDrawSpriteCustomColor2),a
      
      ld l,OBJ_ANIMATION_LIST_2(ix)
      ld h,OBJ_ANIMATION_LIST_2+1(ix)
      ld a,(hl)
      inc hl
      ld h,(hl)
      ld l,a

      ld (#_vtSpritePtr),hl

      call _vtRenderTS

      ret


   lucesRenderizadas:
      .db #0

   __endasm;
}


//entra en HL el puntero a la frase que va a decir

void gtaCreaFrase() __naked {
   __asm
         //ld de,#_talkFrase
         ld de,#TALK_FRASE

      gtaCreaFraseBucle:

         ld a,(hl)
         cp #255
         jr Z,gtaCreaFraseFin    //si es 255 ya no hay que seguir es el final de la frase

         push hl
         ld a,(hl)
         ld h,#0
         ld l,a
         add hl,hl
         ld bc,#_word_list
         add hl,bc
         ld a,(hl)
         inc hl
         ld h,(hl)
         ld l,a
         
         call _gtaCopiaCadenaHLenDE
         dec de
         ld a,#32    //enchufo un espacio en blanco por donde va DE ya que la funcion no lo va a meter
         ld (de),a
         inc de

         pop hl         //recupero la frase por donde iba para ver cual es la siguiente palabra
         inc hl         //paso a la siguiente palabra y vuelvo al bucle
         jr gtaCreaFraseBucle


      gtaCreaFraseFin:
         xor a
         ld (de),a
         inc de
         ld (de),a

         ld a,#FX_SUSTIB
         call _gtaSoundFX

         ret
   __endasm;
}



//copia la cadena a la que apunte HL en DE (para cuando encuentre un 0 en HL)
void gtaCopiaCadenaHLenDE() __naked {
   __asm
      
   gtaCopiaCadenaHLenDEBuc:
      ld a,(hl)
      ld (de),a
      inc hl
      inc de
      or a
      jr NZ,gtaCopiaCadenaHLenDEBuc

      ret
   __endasm;
}





//y devuelve el puntero donde se ha quedado cadAHA1TAcontrol
u8* copiaCadenaBenA(u8* cadA, const char *cadB)
{
   while(*cadB!=0)
   {
      *cadA=*cadB;
      cadA++;
      cadB++;
   }
   *cadA=*cadB; //copia el 0 tambien
   cadA++;
   return cadA;
}


void gtaCambiaPaleta() __naked {
__asm
      //gtaPaletaSeleccionada=gtaPaletaSeleccionada+1
      ld a,(#_gtaPaletaSeleccionada)
      inc a
      cp #3
      jr C,gtaCambiaPaletas2
      ld a,#0

   gtaCambiaPaletas2:
      ld (#_gtaPaletaSeleccionada),a
      ld hl,#_paletaManana
      ld de,#16
      or a           //miramos si usamos la paleta de mañana gtaPaletaSeleccionada=0
      jr Z,gtaCambiaPaletas10
      add hl,de
      dec a          //miramos si usamos la paleta de tarde gtaPaletaSeleccionada=1
      jr Z,gtaCambiaPaletas10
      add hl,de      //usamos la paleta de noche gtaPaletaSeleccionada=2

   gtaCambiaPaletas10:
      ld (#_vtPointerAux),hl
      call _vtConfigurePaletteGame

      ret
__endasm;
}



//hace que comience una mision (se pone en A el numero de la mision antes de llamar aqui)
void gtaMisionStart() __naked {
    __asm
         ld b,a      //me guardo en B la mision a realizar

         //si misionActual<>255 es que ya estoy haciendo una mision, por lo tanto fuera
         ld a,(#_misionActual)
         inc a
         ret NZ      

         //miramos si la mision esta habilitada y se puede hacer (tiene que ser 1)
         /*ld hl,#_misionState
         ld e,b
         ld d,#0
         add hl,de
         ld a,(hl)
         dec a
         ret NZ      //si resto uno y no es cero ocurre una de estas dos cosas (que este inhabilitada aun, o que ya la hayamos hecho) con lo cual salgo directamente
         */
        
         //si llego aqui podemos empezar la mision, hago misionActual=A
         ld a,b
         ld (#_misionActual),a

         //ahora busco el array de la mision
         ld hl,#_misionList
         ld e,a
         ld d,#0
         add hl,de
         add hl,de

         ld e,(hl)
         inc hl 
         ld d,(hl)

         ld (#_misionActualPtr),de     //y ya tengo preparado el puntero a la primera accion de la mision

         ret
   __endasm;
}


void gtaMisionController() __naked {
   __asm
    	//peke check para ver si estamos muertos o no
    	ld a,(#0xB500+OBJ_LIFE)
    	cp #200
    	jr C,gtaMisionControllers2
    	 
    	 
    	//si entro aqui estamos muertos
    	xor a 
    	ld (#0xB500+OBJ_LIFE),a
    	ld hl,#_misionDie
    	ld (#_misionActualPtr),hl	//preparamos para que salga la conversacion de estas muerto y que acabe el juego
    	 
      ld hl,#0xB500+OBJ_FLAGS
      ld a,(hl)
      set 7,a     //le pongo el FLAG_NO_COLISION para que no le sigan quitando vida ni nada
    	ld (hl),a
    	




   gtaMisionControllers2:
      ld hl,(#_misionActualPtr)
      xor a
      or h
      ret Z    //si H es cero es que no esta ninguna mision activa, pa fuera del tiron, no hay que hacer ná

      //bajamos en 1 el temporizador de las misiones (suponiendo que no sea cero ya...)
      ld de,(#_misionTemporizador)
      dec de
      bit 7,d
      jr Z,gtaMisionControllers3
      //si el bit 7 de D es 1 es que ha hecho carry y he vuelto al -1, lo pongo de nuevo a 0
      ld de,#0

   gtaMisionControllers3:
      ld (#_misionTemporizador),de


      //si no es cero tenemos que llamar a la funcion que toque y que gestionara la accion por la que vayamos en la mision
      ld e,(hl)
      ld d,#0
      ld hl,#_misionFuncs
      add hl,de 
      add hl,de 

      ld e,(hl)      //leo la funcion que va a gestionar esta accion
      inc hl 
      ld d,(hl)
      ex de,hl          //la coloco en HL

      call ___sdcc_call_hl       //y la llamo directamente

      ret      //y ya está, fuera
   __endasm;
}



//DE posicion del sprite de luz
//HL posicion actual de la pantalla

void gtaDrawLight() __naked {
__asm
      push de     //sprite de luz
      push hl     //posicion pantalla
      //ld hl,posicionpantalla
      ld de,#NEXT_LINE_ADD_VIDEO_MEM
		ld bc,#MEMOUT_JUMP_VIDEO_MEM

      exx

      pop hl 
      pop de

      ld b,#24

   gtaDrawLightBucleVer:
      //leemos el primer byte de DE
      ld a,(de)
      inc de
      //y ahora vamos seteando bits conforme vayamos encontrandonos
      ld c,a

      xor a    //pongo A a cero                       PIXELES 0 y 1
      sla c    //shift left (zero) C
      rla      //rotate left (carry into) A
      sla c    //shift left (zero) C
      rla      //rotate left (carry into) A
      //ahora tengo los bits 0 y 1  rellenos con los bits 6 y 7 de C
      //ahora puedo hacer una operacion OR sobre (HL) para encenderle esos bits 0 y 1
      or (hl)
      ld (hl),a
      xor a
      inc hl

      sla c    //shift left (zero) C                  PIXELES 2 y 3
      rla      //rotate left (carry into) A
      sla c    //shift left (zero) C
      rla      //rotate left (carry into) A
      //ahora tengo los bits 0 y 1  rellenos con los bits 6 y 7 de C
      //ahora puedo hacer una operacion OR sobre (HL) para encenderle esos bits 0 y 1
      or (hl)
      ld (hl),a
      xor a
      inc hl


      sla c    //shift left (zero) C                  PIXELES 4 y 5
      rla      //rotate left (carry into) A
      sla c    //shift left (zero) C
      rla      //rotate left (carry into) A
      //ahora tengo los bits 0 y 1  rellenos con los bits 6 y 7 de C
      //ahora puedo hacer una operacion OR sobre (HL) para encenderle esos bits 0 y 1
      or (hl)
      ld (hl),a
      xor a
      inc hl



      sla c    //shift left (zero) C                  PIXELES 6 y 7
      rla      //rotate left (carry into) A
      sla c    //shift left (zero) C
      rla      //rotate left (carry into) A
      //ahora tengo los bits 0 y 1  rellenos con los bits 6 y 7 de C
      //ahora puedo hacer una operacion OR sobre (HL) para encenderle esos bits 0 y 1
      or (hl)
      ld (hl),a
      inc hl

      

      ld a,(de)            //paso al siguiente byte
      inc de          
      ld c,a            //y escribo los 4 pixeles que me faltan hasta llegar a los 12 pixeles
      xor a

      
      sla c    //shift left (zero) C                  PIXELES 8 y 9
      rla      //rotate left (carry into) A
      sla c    //shift left (zero) C
      rla      //rotate left (carry into) A
      //ahora tengo los bits 0 y 1  rellenos con los bits 6 y 7 de C
      //ahora puedo hacer una operacion OR sobre (HL) para encenderle esos bits 0 y 1
      or (hl)
      ld (hl),a
      xor a
      inc hl


      sla c    //shift left (zero) C                  PIXELES 10 y 11
      rla      //rotate left (carry into) A
      sla c    //shift left (zero) C
      rla      //rotate left (carry into) A
      //ahora tengo los bits 0 y 1  rellenos con los bits 6 y 7 de C
      //ahora puedo hacer una operacion OR sobre (HL) para encenderle esos bits 0 y 1
      or (hl)
      ld (hl),a
      inc hl



      //paso a la siguiente linea....
      exx
      add hl,de						//3              
      jr NC,gtaDrawLightNextLineFin		//3	
      add hl,bc						//3
      res 3,h
      //bit 7,h
      //call Z,_debug2
   gtaDrawLightNextLineFin:
      push hl
      exx
      pop hl 


      djnz gtaDrawLightBucleVer     //resto uno a B y si no es cero continuo en la siguiente linea
      ret

__endasm;
}



void debug1() __naked {
__asm
      exx
      exx
      ret
__endasm;
}

void debug2() __naked {
__asm
      exx
      exx
      ret
__endasm;
}



//ejecuta un sonido FX por el canal 2 (el del medio de los chichos) a volumen FULL
//mete el id del sonido en A
void gtaSoundFX() __naked {
__asm
      ld l,a      //(1B L ) sfx_num	Number of the instrument in the SFX Song (>0), same as the number given to the instrument in Arkos Tracker.      
      ld h,#15       //(1B H ) volume	Volume [0-15], 0 = off, 15 = maximum volume.
      ld e,#39    //esto es ma o meno la nota C-3 si mis calculos no fallan    //(1B E ) note	Note to be played with the given instrument [0-143]
      ld d,#0     //(1B D ) speed	Speed (0 = As original, [1-255] = new Speed (1 is fastest))
      ld bc,#0    //(2B BC) inverted_pitch	Inverted Pitch (-0xFFFF -> 0xFFFF).  0 is no pitch.  The higher the pitch, the lower the sound.
      ld a,#2     //por el canal del medio de los chichos  //(1B A ) channel_bitmask	Bitmask representing channels to use for reproducing the sound (Ch.A = 001 (1), Ch.B = 010 (2), Ch.C = 100 (4))
      call cpct_akp_SFXPlay_asm
      ret
__endasm;
}


void gtaRestauraCRTCeinterrupt() __naked {
__asm
		LD A,#0xC3
		LD (#0x0038),A
		ld HL,#_debugArmony		//hago que la interrupcion llame al debugArmony que no hace nada pero tiene el ret bien puesto y no toca nada, justo lo que quiero
		LD (#0x0039),HL
		
		//restauro todo lo que haga falta del CRTC, video mem start y algunos valores
		ld a,#0x30
		ld (#_vtScrollPage),a
		xor a
		ld (#_vtScrollOffset),a
		ld (#_vtScrollCRTC5),a
		ld a,#245
		ld (#_vtScrollCRTC3),a
		
		call _vtAdjustCRTC

		ret
__endasm;
}



void gtaMostrarFinal() __naked {
   __asm
      //borramos la pantalla
      ld hl,#0xC000
      ld d,h
      ld e,l
      inc e
      xor a
      ld (hl),a
      ld bc,#0x3FFE
      ldir

      //escribimos los textos
      ld a,#CUSTOM_A_PAL_9|CUSTOM_B_PAL_9        //el color de las letras
      ld de,#0xC1E0+6
      ld hl,#_frase21
      call graMainMenuDrawTextsSub


      ld a,#CUSTOM_A_PAL_7|CUSTOM_B_PAL_7        //el color de las letras
      ld de,#0xC2D0+8
      ld hl,#_frase22
      call graMainMenuDrawTextsSub


      ld a,#CUSTOM_A_PAL_11|CUSTOM_B_PAL_11
      ld de,#0xC3C0+4
      ld hl,#_frase23
      call graMainMenuDrawTextsSub


      ld a,#CUSTOM_A_PAL_15|CUSTOM_B_PAL_15
      ld de,#0xC4B0+20
      ld hl,#_frase24
      call graMainMenuDrawTextsSub

      ld c,#150

   gtaMostrarFinalBuclestart:
      ld b,#255
   gtaMostrarFinalBucle:
      ld a,#255
   gtaMostrarFinalBucles2:
      dec a
      jr NZ,gtaMostrarFinalBucles2

      djnz gtaMostrarFinalBucle

      dec c
      jr NZ,gtaMostrarFinalBuclestart


      ret
   __endasm;
}

void gtaBaseCleanAndPaintFunc() __naked {
__asm
      //miro si el prota va en un coche
      ld hl,(#0xB500+OBJ_PARENT_PTR)
      xor a
      or h
      ret Z    //el prota no va en un coche, esta funcion no tiene que hacer nada en este caso

      ld ix,(#_vtGameObjPtr)

      ld a,OBJ_IA_LOCK_MOVE(ix)
      or a
      jr Z,gtaBaseCleanAndPaintFuncs2
      
      dec OBJ_IA_LOCK_MOVE(ix)
      ret

   gtaBaseCleanAndPaintFuncs2:

      //el prota va en un coche HL, compruebo si está cerca
      push hl 
      pop iy 
      
      ld a,OBJ_ID_NPC_CAR(iy)
      or a
      ret NZ      //si es un coche personalizado ya, no podemos pintarlo


      call _gtaDistanceIX_IY     //miramos la distancia entre ix (la base) y iy (el coche del prota)
      ld a,#2
      cp h     //2 - distancia en tiles
      ret C    //si hay carry es que el coche esta muy lejos, no tengo que mirar nada más
      
      //esta cerca, puedo pintar el coche de algun color random
      call cpct_getRandom_lcg_u8_asm
      and #7      //kiero un random del 0 al 7 asi que hago un and 7 al random
      ld hl,#_carColorsPrepare         //ahora hago A=carColorsPrepare[random_0_7]
      add l
      jr NC,gtaBaseCleanAndPaintFuncs3
      inc h
   gtaBaseCleanAndPaintFuncs3:
      ld l,a
      ld a,(hl)
      ld OBJ_CUSTOM_COLOR(iy),a        //y ahora uso esa tabla para sacar los colores del coche (son colores más normales y logicos para un coche)

      ld a,#100
      ld OBJ_IA_LOCK_MOVE(ix),a

      //aqui vamos a bajar el hate de la poli
      ld hl,#_hateLevel+1
      ld (hl),#0

      //y de paso, si algun coche te sigue, los ponemos para que deje de seguirte,
      //como solo hay una ia de seguirte de coche, simpmlemente miramos si lleva esa IA y se la quitamos
      //si no la lleva no hacemos nada    KABESON
      ld ix,#GAME_OBJECTS_PTR+96
      
      ld de,#48
      ld b,#14

   gtaBaseCleanAndPaintFuncs3Bucle:
      push bc
      ld c,OBJ_MACHINE_STATE_PTR(ix)
      ld b,OBJ_MACHINE_STATE_PTR+1(ix)
      xor a 
      ld hl,#_gtaCocheIAgotoProta
      sbc hl,bc
      pop bc
      jr NZ,gtaBaseCleanAndPaintFuncs3BucleFin

      //si es 0 se lo quito
      xor a
      ld OBJ_MACHINE_STATE_PTR(ix),a
      ld OBJ_MACHINE_STATE_PTR+1(ix),a

   gtaBaseCleanAndPaintFuncs3BucleFin:
      add ix,de
      djnz gtaBaseCleanAndPaintFuncs3Bucle

      ret
__endasm;
}




void gtaBaseHospitalFunc() __naked {
__asm
      //miro si el prota va andando o no
      ld hl,(#0xB500+OBJ_PARENT_PTR)
      xor a
      or h
      ret NZ    //el prota va en un coche, esta funcion no tiene que hacer nada en este caso

      ld ix,(#_vtGameObjPtr)     //cargo la base en IX

      ld a,OBJ_IA_LOCK_MOVE(ix)
      or a
      jr Z,gtaBaseHospitalFuncs2
      
      dec OBJ_IA_LOCK_MOVE(ix)
      ret

   gtaBaseHospitalFuncs2:

      ld iy,#0xB500     //cargo el prota en IY
      
      call _gtaDistanceIX_IY     //miramos la distancia entre ix (la base) y iy (el coche del prota)
      ld a,#2
      cp h     //2 - distancia en tiles
      ret C    //si hay carry es que el coche esta muy lejos, no tengo que mirar nada más
      
      //si llega aqui esta cerca, curo al prota 
      ld OBJ_LIFE(iy),#PROTA_START_LIFE
      ld OBJ_SELECTED_WEAPON(iy),#0    //le quito las armas

      ld a,#250
      ld OBJ_IA_LOCK_MOVE(ix),a

      ld (#_talkInstancePtr),iy
      ld hl,#_frase42    //la frase de me encuentro mejor
      call _gtaCreaFrase

      

      ret      //y pa fuera
__endasm;
}

void gtaFillStatusStr() __naked {
__asm
      
      ld a,(#_misionActual)    //leemos la mision actual
      ld e,a
      call functionE_DIV_10
      ld a,h
      ld b,a      //me guardo las decenas en B
      add #48     //caracter 0
      ld (#_statusStr+0),a

      inc b
      ld a,(#_misionActual)    //leemos la poli
   gtaFillStatusStrs2:
      sub #10           							//y ahora voy restandole 10 tantas veces como decenas haya indicado para quedarme solo con las unidades
		djnz gtaFillStatusStrs2
      add #10

      add #48
      ld (#_statusStr+1),a

      ld b,#6
      ld hl,#_statusStr+7
      ld de,#_hateLevel+1
   gtaFillStatusStrs3:
      ld a,(de)
      inc de
      cp #9
      jr C,gtaFillStatusStrs3a
      ld a,#9
   gtaFillStatusStrs3a:
      add #48
      ld (hl),a

      ld a,#6     //pasamos a la siguiente X
      add l 
      jr NC,gtaFillStatusStrs3b
      inc h
   gtaFillStatusStrs3b:
      ld l,a

      djnz gtaFillStatusStrs3

      ret      //y pa fuera




   functionE_DIV_10:
      ld d,#0
      ld h,d
      ld l,e
      add hl,hl      //x2
      add hl,de      //x3
      add hl,hl      //x6
      add hl,hl      //x12
      add hl,de      //x13
      add hl,hl      //x26
      ret

   __endasm;
}

#include "datafull.h"